// Map.java
// Started on June 16th, 2001
// Copyright (c) Virtu Halttunen

//import TheGame.*;
import java.awt.Point;
import java.util.*;
import java.util.zip.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;
import java.io.*;

public class Map
{
    public static final int NORTH = 0;
    public static final int NORTH_EAST = 1;
    public static final int EAST = 2;
    public static final int SOUTH_EAST = 3;
    public static final int SOUTH = 4;
    public static final int SOUTH_WEST = 5;
    public static final int WEST = 6;
    public static final int NORTH_WEST = 7;
    
    public static final int DOORS = 128;
     

    private MapCell[][] cellMap;
    private Vector doors;
    private Vector unboundObjectCells;
    private Vector objects;
    private HashMap rooms;
    private short[][] utilityMap;
    private int horizontalSize;
    private int verticalSize;
    private Vector floorSetNames;
    private Vector wallSetNames;
    private Vector objectSetNames;
    private Vector doorSetNames;
    private String name;
    
    public Map( int horizontalSize, int verticalSize )
    {
	this.horizontalSize = horizontalSize;
	this.verticalSize = verticalSize;
	cellMap = new MapCell[ horizontalSize ][ verticalSize ];
        doors = new Vector();
        unboundObjectCells = new Vector();
	objects = new Vector();
	rooms = new HashMap();
	utilityMap = new short[ horizontalSize ][ verticalSize ];
	floorSetNames = new Vector();
	wallSetNames = new Vector();
	objectSetNames = new Vector();
	doorSetNames = new Vector();
    }
    
    public void addObject( MapObject object )
    {
	if( !objects.contains( object ) )
	{
	    objects.add( object );
	    for( int x=object.getTopLeft().x; x <=object.getBottomRight().x; x++ )
	    {
		for( int y=object.getTopLeft().y; y <= object.getBottomRight().y; y++ )
		{
		    if( object.isLocatedIn( x, y ) )
		    {
			cellMap[x][y].setObjects( true );
		    }
		}
	    }
	}
    }
    
    public void removeObject( MapObject object )
    {
	if( objects.contains( object ) )
	{
	    objects.remove( object );
	    MapObject otherObject;
	    for( int x=object.getTopLeft().x; x <=object.getBottomRight().x; x++ )
	    {
		for( int y=object.getTopLeft().y; y <= object.getBottomRight().y; y++ )
		{
		    cellMap[x][y].setObjects( false );
		    for( int c=0; c < objects.size(); c++ )
		    {
			otherObject = getObject( c );
			if( otherObject.isLocatedIn( x, y ) )
			{
			    cellMap[x][y].setObjects( true );
			}
		    }
		}
	    }
	    
	}
    }
    
    public void addRoom( Room room )
    {
	if( !rooms.containsKey( room.getId() ) )
	{
	    rooms.put( room.getId(), room );
	    System.out.println( "Added room: " + room.getName() + " (" + room.getId() +")" );
	}
    }
    
    public Room getRoom( String id )
    {
	return (Room)rooms.get( id );
    }
    
    public Room getRoomForPoint( Point point )
    {
	Room [] theRooms = getRooms();
	for( int i=0; i < theRooms.length; i++ )
	{
	    if( point.x >= theRooms[i].getTopLeft().x && point.x <= theRooms[i].getBottomRight().x &&
		point.y >= theRooms[i].getTopLeft().y && point.y <= theRooms[i].getBottomRight().y )
	    {
		// Found the room
		return theRooms[i];
	    }
	}
	return null;
    }
    
    public Room [] getRooms()
    {
	Collection collection = rooms.values();
	Room [] theRooms = new Room[ collection.size() ];
	
	Iterator iterator = collection.iterator();
	int i=0;
	while( iterator.hasNext() )	
	{
	    theRooms[ i++ ] = (Room)(iterator.next());
	}
	return theRooms;
    }
    
    public void removeRoom( Room room )
    {
	if( rooms.containsKey( room.getId() ) )
	{
	    String [] linkedRoomIds = room.getLinkedRoomIds();
	    Room otherRoom;
	    for( int i=0; i < linkedRoomIds.length; i++ )
	    {
		String id = linkedRoomIds[ i ];
		otherRoom = (Room)rooms.get( id );
		if( otherRoom != null )
		{
		    otherRoom.removeLinkedRoom( room );
		}
	    }
	    rooms.remove( room.getId() );
	}
    }
    
    public void clearSelection()
    {
	for( int i=0; i < horizontalSize; i++ )
	    for( int j=0; j < verticalSize; j++ )
	    {
		if( utilityMap[i][j] == MapMaker.SELECTED ) // Bad, bad, bad... Needs better handling
		{
		    utilityMap[i][j] = 0;
		}
	    }
		
    }
    
    public void deleteSelected()
    {
	for( int i=0; i < horizontalSize; i++ )
	    for( int j=0; j < verticalSize; j++ )
	    {
		if( utilityMap[i][j] == MapMaker.SELECTED ) // Bad, bad, bad... Needs better handling
		{
		    cellMap[i][j] = null;
		    removeDoor( i, j );
		    removeUnboundObjectCells( i, j );
		    utilityMap[i][j] = 0;
		}
	    }
    }
    

    public void setCell( int x, int y, MapCell cell )
    {
        cellMap[x][y] = cell;
    }
    
    public MapCell getCell( int x, int y )
    {
	//System.out.println( "Map: getCellType("+x+","+y+")" );
	return cellMap[x][y];
    }
    
    public MapCell getCellType( Point point )
    {
	//System.out.println( "Map: getCellType() " + point.toString() );
	return cellMap[ point.x ][ point.y ];
    }
    
    public void setCellFloor( int x, int y, int type )
    {
        if( cellMap[x][y] == null )
        {
            cellMap[x][y] = new MapCell();
        }
	cellMap[x][y].setFloor( type );
    }

    public void addCellFloor( int x, int y, int type )
    {
        if( cellMap[x][y] == null )
        {
            cellMap[x][y] = new MapCell();
            cellMap[x][y].setFloor( type );
        }
        else
        {
            cellMap[x][y].addFloor( type );
        }
    }
    
    
    public void setCellWall( int x, int y, int type )
    {
        if( cellMap[x][y] == null )
        {
            cellMap[x][y] = new MapCell();
        }
	if( cellMap[x][y].hasDoor() )
	{
	    removeDoor( x, y );
	}
	cellMap[x][y].setWall( type );
    }
    
    public byte getLight( int x, int y )
    {
	return cellMap[x][y].getLight();
    }
    
    public byte getLight( Point point )
    {
	return cellMap[ point.x ][ point.y ].getLight();
    }
    
    public void setLight( int x, int y, byte light )
    {
        if( cellMap[x][y] == null )
        {
            cellMap[x][y] = new MapCell();
        }
        cellMap[x][y].setLight( light );
    }
    
    public int getUtility( int x, int y )
    {
	return utilityMap[x][y];
    }
    
    public void setUtility( int x, int y, int value )
    {
	utilityMap[x][y] = (short)value;
    }

    public void addDoor( Door door )
    {
        if( cellMap[ door.getX() ][ door.getY() ] != null )
        {
            if( cellMap[ door.getX() ][ door.getY() ].hasDoor() )
            {
                removeDoor( door.getX(), door.getY() );
            }
            cellMap[ door.getX() ][ door.getY() ].setDoor( true );
            doors.add( door );
        }
    }

    public void removeDoor( int x, int y )
    {
	if( cellMap[x][y] != null )
	{
	       if( cellMap[x][y].hasDoor() )
	       {
		   Door theDoor = getDoor( x, y );
		   doors.remove( theDoor );
		   cellMap[x][y].setWall( -1 );
	       }
	}
    }

    public Door getDoor( int x, int y )
    {
        Door theDoor = null;
        for( int i = 0; i < doors.size(); i++ )
        {
            if( ((Door)doors.elementAt( i )).isLocatedIn( x, y ) )
                theDoor = (Door)doors.elementAt( i );
        }
        return theDoor;
    }

    public void addObjectCell( MapObjectCell objectCell )
    {
        if( cellMap[ objectCell.getX() ][ objectCell.getY() ] != null )
        {
            cellMap[ objectCell.getX() ][ objectCell.getY() ].setObjects( true );
            unboundObjectCells.add( objectCell );
        }
    }

    public void removeUnboundObjectCells( int x, int y )
    {
        if( cellMap[x][y] != null )
        {
            if( cellMap[x][y].hasObjects() )
            {
                for( int i=0; i < unboundObjectCells.size(); i++ )
                {
                    if( ((MapObjectCell)unboundObjectCells.elementAt(i) ).isLocatedIn( x, y ) )
                    {
                        unboundObjectCells.removeElementAt( i );
                        i--;
                    }
                }
                cellMap[x][y].setObjects( false );
            }
        }
    }

    public int getNumberOfUnboundObjects()
    {
        return unboundObjectCells.size();
    }
	
    public MapObjectCell [] getSelectedUnboundObjectCells()
    {
	MapObjectCell [] theCells = null;
	Vector help = new Vector();
	MapObjectCell cell = null;
	
	for( int i=0; i < unboundObjectCells.size(); i++ )
	{
	    cell = getObjectCell( i );
	    if( utilityMap[ cell.getX() ][ cell.getY() ] == MapMaker.SELECTED )
	    {
		help.add( cell );
	    }
	}
	if( help.size() > 0 )
	{
	    theCells = new MapObjectCell[ help.size() ];
	    for( int i=0; i < help.size(); i++ )
	    {
		theCells[i] = (MapObjectCell)help.elementAt( i );
	    }
	}
	return theCells;
    }

    public MapObject getObject( int index )
    {
	return (MapObject)objects.elementAt( index );
    }
	
    public MapObject getObjectForPoint( Point point )
    {
	if( cellMap[ point.x ][ point.y ] == null )
	{
	    return null;   
	}
	if( cellMap[ point.x ][ point.y ].hasObjects() )
	{
	    for( int i=0; i < objects.size(); i++ )
	    {
		MapObject object = getObject( i );
		if( object.isLocatedIn( point.x, point.y ) )
		{
		    return object;   
		}
	    }
	}
	return null;
    }
	
    public int getNumberOfObjects()
    {
	return objects.size();
    }
	
    public MapObjectCell getObjectCell( int index )
    {
        return (MapObjectCell)unboundObjectCells.elementAt( index );
    }
    
    public void addWallSet( String setName )
    {
	wallSetNames.add( new String( setName ) );
    }
    
    public Vector getWallSets()
    {
	return wallSetNames;
    }
    
    public void addFloorSet( String setName )
    {
	floorSetNames.add( new String( setName ) );
    }
    
    public Vector getFloorSets()
    {
	return floorSetNames;
    }
    
    public void addDoorSet( String setName )
    {
	doorSetNames.add( new String( setName ) );
    }
    
    public Vector getDoorSets()
    {
	return doorSetNames;
    }
    
    public void addObjectSet( String setName )
    {
	objectSetNames.add( new String( setName ) );
    }
    
    public Vector getObjectSets()
    {
	return objectSetNames;
    }
    
    public void setName( String name )
    {
	this.name = new String( name );
    }
    
    public String getName()
    {
	return new String( name );
    }
    
    public int getHorizontalSize()
    {
	return horizontalSize;
    }
    
    public int getVerticalSize()
    {
	return verticalSize;
    }
    
    public void save( File file ) throws FileNotFoundException, IOException, ParserConfigurationException, TransformerConfigurationException, TransformerException
    {
	DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
	DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
	DOMImplementation builder = documentBuilder.getDOMImplementation();
	
	Document document = builder.createDocument( null, "map", null );
	Element root = document.getDocumentElement();
	root.setAttribute( "name", getName() );
	root.setAttribute( "width", String.valueOf( getHorizontalSize() ) );
	root.setAttribute( "height", String.valueOf( getVerticalSize() ) );
	Element setsElement = document.createElement( "graphicssets" );
	/// Store the floor set names
	Element floorSetElement;
	for( int i=0; i < floorSetNames.size(); i++ )
	{
	    floorSetElement = document.createElement( "floorset" );
	    floorSetElement.setAttribute( "index", String.valueOf( i ) );
	    floorSetElement.setAttribute( "file", (String)floorSetNames.elementAt( i ) );
	    setsElement.appendChild( document.createTextNode( "\n" ) );
	    setsElement.appendChild( floorSetElement );

	}
	/// Same for wall sets
	Element wallSetElement;
	for( int i=0; i < wallSetNames.size(); i++ )
	{
	    wallSetElement = document.createElement( "wallset" );
	    wallSetElement.setAttribute( "index", String.valueOf( i ) );
	    wallSetElement.setAttribute( "file", (String)wallSetNames.elementAt( i ) );
	    setsElement.appendChild( document.createTextNode( "\n" ) );
	    setsElement.appendChild( wallSetElement );
	}
	// And object sets
	Element objectSetElement;
	for( int i=0; i < objectSetNames.size(); i++ )
	{
	    objectSetElement = document.createElement( "objectset" );
	    objectSetElement.setAttribute( "index", String.valueOf( i ) );
	    objectSetElement.setAttribute( "file", (String)objectSetNames.elementAt( i ) );
	    setsElement.appendChild( document.createTextNode( "\n" ) );
	    setsElement.appendChild( objectSetElement );
	}
	// And door sets
	Element	doorSetElement;
	for( int i=0; i < doorSetNames.size(); i++ )
	{
	    doorSetElement = document.createElement( "doorset" );
	    doorSetElement.setAttribute( "index", String.valueOf( i ) );
	    doorSetElement.setAttribute( "file", (String)doorSetNames.elementAt( i ) );
	    setsElement.appendChild( document.createTextNode( "\n" ) );
	    setsElement.appendChild( doorSetElement );
	}
	root.appendChild( document.createTextNode( "\n" ) );
	root.appendChild( setsElement );
	
	// Store the cells
	Element cellsElement = document.createElement( "cells" );
	
	Element cellElement = null;
	for( int y=0; y < getVerticalSize(); y++ )
	{
	    for( int x=0; x < getHorizontalSize(); x++ )
	    {
		if( cellMap[x][y] != null )
		{
		    cellElement = document.createElement( "cell" );
		    cellElement.setAttribute( "x", String.valueOf( x ) );
		    cellElement.setAttribute( "y", String.valueOf( y ) );
		    cellElement.setAttribute( "light", String.valueOf( cellMap[x][y].getLight() ) );
		    for( int f=0; f < cellMap[x][y].getFloorSize(); f++ )
		    {
			Element floorElement = document.createElement( "floor" );
			floorElement.setAttribute( "index", String.valueOf( f ) );
			floorElement.setAttribute( "tile", String.valueOf( cellMap[x][y].getFloor( f ) ) );
			cellElement.appendChild( floorElement );
		    }
		    int wall = cellMap[x][y].getWall();
		    if( wall >= 0 )
		    {
			Element wallElement = document.createElement( "wall" );
			wallElement.setAttribute( "type", String.valueOf( wall ) );
			cellElement.appendChild( wallElement );
		    }			
		    cellsElement.appendChild( document.createTextNode( "\n" ) );
		    cellsElement.appendChild( cellElement );
		}
	    }
	}
	root.appendChild( cellsElement );
	root.appendChild( document.createTextNode( "\n" ) );
	// Store doors
	Door door;
	Element doorsElement = document.createElement( "doors" );
	Element doorElement;
	for( int d=0; d<doors.size(); d++ )
        {
            door = (Door)doors.elementAt( d );
	    doorElement = document.createElement( "door" );
	    doorElement.setAttribute( "x", String.valueOf( door.getX() ) );
            doorElement.setAttribute( "y", String.valueOf( door.getY() ) );
            doorElement.setAttribute( "direction", String.valueOf( door.getDirection() ) );
            doorElement.setAttribute( "locked", String.valueOf( door.isLocked() ) );
            doorElement.setAttribute( "open", String.valueOf( door.isOpen() ) );
	    doorsElement.appendChild( document.createTextNode( "\n" ) );
	    doorsElement.appendChild( doorElement );
        }
	root.appendChild( document.createTextNode( "\n" ) );
	root.appendChild( doorsElement );
	// Store objects
	Element objectsElement = document.createElement( "objects" );
	MapObject mapObject;
	Element mapObjectElement;
	Element mapObjectCellElement;
	for( int o=0; o < objects.size(); o++ )
	{
	    mapObject = getObject( o );
	    mapObjectElement = document.createElement( "object" );
	    mapObjectElement.setAttribute( "id", mapObject.getId() );
	    mapObjectElement.setAttribute( "name", mapObject.getName() );
	    mapObjectElement.setAttribute( "strength", String.valueOf( mapObject.getStrength() ) );
	    mapObjectElement.setAttribute( "movecost", String.valueOf( mapObject.getCostToMove() ) );
	    for( int y=mapObject.getTopLeft().y; y <= mapObject.getBottomRight().y; y++ )
	    {
		for( int x=mapObject.getTopLeft().x; x <= mapObject.getBottomRight().x; x++ )
		{
		    MapObjectCell [] objectCells = mapObject.getCells( x, y );
		    if( objectCells != null )
		    {
			for( int c=0; c < objectCells.length; c++ )
			{
			    mapObjectCellElement = document.createElement( "objectcell" );
			    mapObjectCellElement.setAttribute( "x", String.valueOf( x ) );
			    mapObjectCellElement.setAttribute( "y", String.valueOf( y ) );
			    mapObjectCellElement.setAttribute( "type", String.valueOf( objectCells[c].getType() ) );
			    mapObjectCellElement.setAttribute( "destroyedtype", String.valueOf( objectCells[c].getDestroyedType() ) );
			    mapObjectCellElement.setAttribute( "hitpoints", String.valueOf( objectCells[c].getHitPoints() ) );
			    mapObjectCellElement.setAttribute( "blocksmovement", String.valueOf( objectCells[c].isBlockingMovement() ) );
			    mapObjectCellElement.setAttribute( "blocksvision", String.valueOf( objectCells[c].isBlockingSight() ) );
			    mapObjectElement.appendChild( document.createTextNode( "\n" ) );
			    mapObjectElement.appendChild( mapObjectCellElement );
			}
		    }
		}
	    }
	    objectsElement.appendChild( document.createTextNode( "\n" ) );
	    objectsElement.appendChild( mapObjectElement );
	}
	root.appendChild( objectsElement );
	root.appendChild( document.createTextNode( "\n" ) );
	// Store rooms
	Element roomsElement = document.createElement( "rooms" );
	Element roomElement;
	Room [] theRooms = getRooms();
	for( int r=0; r < theRooms.length; r++ )
	{
	    roomElement = document.createElement( "room" );
	    roomElement.setAttribute( "id", theRooms[ r ].getId() );
	    roomElement.setAttribute( "name", theRooms[ r ].getName() );
	    roomElement.setAttribute( "minx", String.valueOf( theRooms[ r ].getTopLeft().x ) );
	    roomElement.setAttribute( "maxx", String.valueOf( theRooms[ r ].getBottomRight().x ) );
	    roomElement.setAttribute( "miny", String.valueOf( theRooms[ r ].getTopLeft().y ) );
	    roomElement.setAttribute( "maxy", String.valueOf( theRooms[ r ].getBottomRight().y ) );
	    roomElement.setAttribute( "hotspotx", String.valueOf( theRooms[ r ].getHotSpot().x ) );
	    roomElement.setAttribute( "hotspoty", String.valueOf( theRooms[ r ].getHotSpot().y ) );
	    String [] linkedRoomIds = theRooms[ r ].getLinkedRoomIds();
	    for( int l=0; l < linkedRoomIds.length; l++ )
	    {
		Element linkElement = document.createElement( "link" );
		linkElement.setAttribute( "target", linkedRoomIds[ l ] );
		roomElement.appendChild( document.createTextNode( "\n" ) );
		roomElement.appendChild( linkElement );
	    }
	    roomsElement.appendChild( document.createTextNode( "\n" ) );
	    roomsElement.appendChild( roomElement );
	}
	root.appendChild( roomsElement );
	//document.normalizeDocument();
	System.out.println( "Saving document" );
	//System.out.println( document.toString() );
	
	DOMSource domSource = new DOMSource( document );
	
	FileOutputStream fos = new FileOutputStream( file );
	GZIPOutputStream gos = new GZIPOutputStream( fos );
	StreamResult streamResult = new StreamResult( gos );
        //StreamResult streamResult = new StreamResult( file );
        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        Transformer transformer = transformerFactory.newTransformer();
        transformer.transform( domSource, streamResult );        
	
	gos.close();
	fos.close();
	System.out.println( "Map saved" );
    }
	
    public void saveOld( File file ) throws FileNotFoundException, IOException
    {
	FileOutputStream fos = new FileOutputStream( file );
	DataOutputStream writer = new DataOutputStream( fos );
        int value;
        Door door;
        MapObjectCell mapObject;
        
	// Start writing. First, the size of the map as two integers
	writer.writeInt( horizontalSize );
	writer.writeInt( verticalSize );
        
	// Then write the actual cell information
	for( int i=0; i < horizontalSize; i++ )
	    for( int j=0; j < verticalSize; j++ )
            {
                if( cellMap[i][j] == null )
                {
                    writer.writeInt( 0 );
                }
                else
                {
                    // Write the number of the floor cells
                    writer.writeInt( cellMap[i][j].getFloorSize() );
                    // Write the actual floor cells
                    for( int f=0; f < cellMap[i][j].getFloorSize(); f++ )
                    {
                        writer.writeInt( cellMap[i][j].getFloor( f ) );
                    }
                    // Write the walls
                    writer.writeInt( cellMap[i][j].getWall() );
                    // Write the light
                    writer.writeByte( cellMap[i][j].getLight() );
                }
            }
        // Then, write the doors
        writer.writeInt( doors.size() );
        for( int d=0; d<doors.size();d++ )
        {
            door = (Door)doors.elementAt( d );
            writer.writeInt( door.getX() );
            writer.writeInt( door.getY() );
            writer.writeInt( door.getDirection() );
            writer.writeBoolean( door.isLocked() );
            writer.writeBoolean( door.isOpen() );
        }
        // Then, objects
        writer.writeInt( getNumberOfUnboundObjects() );
        for( int o=0; o< unboundObjectCells.size(); o++ )
        {
            mapObject = getObjectCell( o );
            writer.writeInt( mapObject.getX() );
            writer.writeInt( mapObject.getY() );
            writer.writeInt( mapObject.getType() );
            writer.writeInt( mapObject.getHitPoints() );
            writer.writeBoolean( mapObject.isBlockingMovement() );
            writer.writeBoolean( mapObject.isBlockingSight() );
        }
	// Next, the number of floor sets used
	writer.writeInt( floorSetNames.size() );
	// Then the actual names as UTF encoded strings
	for( int i=0; i < floorSetNames.size(); i++ )
	    writer.writeUTF( (String)floorSetNames.elementAt( i ) );
	// Same for wall sets
	writer.writeInt( wallSetNames.size() );
	for( int i=0; i < wallSetNames.size(); i++ )
	    writer.writeUTF( (String)wallSetNames.elementAt( i ) );
	// And object sets
	writer.writeInt( objectSetNames.size() );
	for( int i=0; i < objectSetNames.size(); i++ )
	    writer.writeUTF( (String)objectSetNames.elementAt( i ) );
	// And door sets
	writer.writeInt( doorSetNames.size() );
	for( int i=0; i < doorSetNames.size(); i++ )
	    writer.writeUTF( (String)doorSetNames.elementAt( i ) );
	// Finally, write the name of the map
	writer.writeUTF( name );
	writer.flush();
	writer.close();
	fos.close();
    }
	
    public static Map load( File file ) throws Exception
    {
	Map newMap = null;
	if( file.exists() && file.canRead() )
	{
	    DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
	    DOMImplementation builder = documentBuilder.getDOMImplementation();
	    
	    FileInputStream fis = new FileInputStream( file );
	    GZIPInputStream gis = new GZIPInputStream( fis );
	    
	    System.out.println( "Parsing map file: " + file.getName() );
	    Document document = documentBuilder.parse( gis );
	    gis.close();
	    fis.close();
	    
	    Element root = document.getDocumentElement();
	    String mapName = root.getAttribute( "name" );
	    System.out.println( "Map name: " + mapName );
	    int mapWidth = Integer.parseInt( root.getAttribute( "width" ) );
	    int mapHeight = Integer.parseInt( root.getAttribute( "height" ) );
	    newMap = new Map( mapWidth, mapHeight );
	    newMap.setName( mapName );
	    Element setsElement = XMLUtils.getNamedChild( root, "graphicssets" );
	    Element [] floorSets = XMLUtils.getNamedChildren( setsElement, "floorset" );
	    System.out.println( "Reading floor sets" );
	    for( int i=0; i < floorSets.length; i++ )
	    {
		if( i != Integer.parseInt( floorSets[i].getAttribute( "index" ) ) )
		{
		    System.out.println( "Index mismatch!" );
		}
		String setName = floorSets[i].getAttribute( "file" );
		if( setName != null )
		{
		    newMap.addFloorSet( setName );
		}
	    }
	    Element [] wallSets = XMLUtils.getNamedChildren( setsElement, "wallset" );
	    System.out.println( "Reading wall sets" );
	    for( int i=0; i < wallSets.length; i++ )
	    {
		if( i != Integer.parseInt( wallSets[i].getAttribute( "index" ) ) )
		{
		    System.out.println( "Index mismatch!" );
		}
		String setName = wallSets[i].getAttribute( "file" );
		if( setName != null )
		{
		    newMap.addWallSet( setName );
		}
	    }
	    Element [] objectSets = XMLUtils.getNamedChildren( setsElement, "objectset" );
	    System.out.println( "Reading object sets" );
	    for( int i=0; i < objectSets.length; i++ )
	    {
		if( i != Integer.parseInt( objectSets[i].getAttribute( "index" ) ) )
		{
		    System.out.println( "Index mismatch!" );
		}
		String setName = objectSets[i].getAttribute( "file" );
		if( setName != null )
		{
		    newMap.addObjectSet( setName );
		}
	    }
	    Element [] doorSets = XMLUtils.getNamedChildren( setsElement, "doorset" );
	    System.out.println( "Reading door sets" );
	    for( int i=0; i < doorSets.length; i++ )
	    {
		if( i != Integer.parseInt( doorSets[i].getAttribute( "index" ) ) )
		{
		    System.out.println( "Index mismatch!" );
		}
		String setName = doorSets[i].getAttribute( "file" );
		if( setName != null )
		{
		    newMap.addDoorSet( setName );
		}
	    }
	    // Load cells
	    Element cellsElement = XMLUtils.getNamedChild( root, "cells" );
	    Element [] cellElements = XMLUtils.getNamedChildren( cellsElement, "cell" );
	    System.out.println( cellElements.length + " cells found" );
	    for( int c=0; c < cellElements.length; c++ )
	    {
		MapCell cell = new MapCell();
		int x = Integer.parseInt( cellElements[c].getAttribute( "x" ) );
		int y = Integer.parseInt( cellElements[c].getAttribute( "y" ) );
		byte light = Byte.parseByte( cellElements[c].getAttribute( "light" ) );
		cell.setLight( light );
		Element [] floorElements = XMLUtils.getNamedChildren( cellElements[c], "floor" );
		if( floorElements != null )
		{
		    //System.out.println( floorElements.length + " floors found for cell: " + x + ", " + y );
		    for( int f=0; f < floorElements.length; f++ )
		    {
			int index = Integer.parseInt( floorElements[f].getAttribute( "index" ) );
			if( index != f )
			{
			    System.out.println( "Index mismatch!" );
			}
			cell.addFloor( Integer.parseInt( floorElements[f].getAttribute( "tile" ) ) );
		    }
		}
		Element wallElement = XMLUtils.getNamedChild( cellElements[c], "wall" );
		if( wallElement != null )
		{
		    cell.setWall( Integer.parseInt( wallElement.getAttribute( "type" ) ) );
		}
		newMap.setCell( x, y, cell );
	    }
	    // Load doors
	    Element doorsElement = XMLUtils.getNamedChild( root, "doors" );
	    if( doorsElement != null )
	    {
		Element [] doorElements = XMLUtils.getNamedChildren( doorsElement, "door" );
		System.out.println( doorElements.length + " doors found" );
		for( int d=0; d < doorElements.length; d++ )
		{
		    int x = Integer.parseInt( doorElements[d].getAttribute( "x" ) );
		    int y = Integer.parseInt( doorElements[d].getAttribute( "y" ) );
		    int direction = Integer.parseInt( doorElements[d].getAttribute( "direction" ) );
		    boolean locked = doorElements[d].getAttribute( "locked" ).trim().equalsIgnoreCase( "true" );
		    boolean open = doorElements[d].getAttribute( "open" ).trim().equalsIgnoreCase( "true" );
		    Door door = new Door( x, y, direction );
		    door.setLocked( locked );
		    door.setOpen( open );
		    newMap.addDoor( door );
		}
	    }
	    // Load objects
	    Element objectsElement = XMLUtils.getNamedChild( root, "objects" );
	    if( objectsElement != null )
	    {
		Element [] objectElements = XMLUtils.getNamedChildren( objectsElement, "object" );
		System.out.println( objectElements.length + " objects found" );
		for( int o=0; o < objectElements.length; o++ )
		{
		    String objectId = objectElements[o].getAttribute( "id" );
		    String name = objectElements[o].getAttribute( "name" );
		    int strength = Integer.parseInt( objectElements[o].getAttribute( "strength" ) );
		    int moveCost = Integer.parseInt( objectElements[o].getAttribute( "movecost" ) );
		    MapObject mapObject = new MapObject( objectId, name, strength, moveCost );
		    /// Load object cells
		    Element [] objectCellElements = XMLUtils.getNamedChildren( objectElements[o], "objectcell" );
		    if( objectCellElements != null )
		    {
			for( int c=0; c < objectCellElements.length; c++ )
			{
			    int x = Integer.parseInt( objectCellElements[c].getAttribute( "x" ) );
			    int y = Integer.parseInt( objectCellElements[c].getAttribute( "y" ) );
			    int type = Integer.parseInt( objectCellElements[c].getAttribute( "type" ) );
			    int destroyedType = Integer.parseInt( objectCellElements[c].getAttribute( "destroyedtype" ) );
			    int hitPoints = Integer.parseInt( objectCellElements[c].getAttribute( "hitpoints" ) );
			    boolean blocksMovement = objectCellElements[c].getAttribute( "blocksmovement" ).trim().equalsIgnoreCase( "true" );
			    boolean blocksSight = objectCellElements[c].getAttribute( "blocksvision" ).trim().equalsIgnoreCase( "true" );
			    MapObjectCell objectCell = new MapObjectCell( x, y, type, hitPoints, blocksMovement, blocksSight );
			    objectCell.setDestroyedType( destroyedType );
			    mapObject.addObjectCell( objectCell );
			}
		    }
		    mapObject.setStrength( strength );
		    //System.out.println( "Loaded object " + name + " (" + objectId + "), " + objectCellElements.length + " components" );
		    newMap.addObject( mapObject );
		}
	    }
	    // Load rooms
	    Element roomsElement = XMLUtils.getNamedChild( root, "rooms" );
	    if( roomsElement != null )
	    {
		Element [] roomElements = XMLUtils.getNamedChildren( roomsElement, "room" );
		for( int r=0; r < roomElements.length; r++ )
		{
		    String roomId = roomElements[r].getAttribute( "id" );
		    String roomName = roomElements[r].getAttribute( "name" );
		    int minX = Integer.parseInt( roomElements[r].getAttribute( "minx" ) );
		    int maxX = Integer.parseInt( roomElements[r].getAttribute( "maxx" ) );
		    int minY = Integer.parseInt( roomElements[r].getAttribute( "miny" ) );
		    int maxY = Integer.parseInt( roomElements[r].getAttribute( "maxy" ) );
		    int hotSpotX = Integer.parseInt( roomElements[r].getAttribute( "hotspotx" ) );
		    int hotSpotY = Integer.parseInt( roomElements[r].getAttribute( "hotspoty" ) );
		    Room room = new Room( roomId, roomName, minX, minY, maxX, maxY );
		    room.setHotSpot( new Point( hotSpotX, hotSpotY ) );
		    Element [] linkElements = XMLUtils.getNamedChildren( roomElements[r], "link" );
		    if( linkElements != null )
		    {
			for( int l=0; l < linkElements.length; l++ )
			{
			    room.addLink( linkElements[l].getAttribute( "target" ) );
			}
		    }		    
		    newMap.addRoom( room );
		}
	    }
	    System.out.println( "Loaded map" );
	}
	return newMap;
    }

    public static Map loadOld( File file ) throws FileNotFoundException, IOException
    {
        FileInputStream fis = new FileInputStream( file );
        DataInputStream reader = new DataInputStream( fis );

        // Read the size
        int x = reader.readInt();
        int y = reader.readInt();
        Map newMap = new Map( x, y );
        MapCell cell;
        int value;
        int [] cellData;
        // Read the cell information
        for( int i=0; i < x; i++ )
            for( int j=0; j < y; j++ )
            {
                value = reader.readInt();
                if( value <= 0 )
                {
                    newMap.setCell( i, j, null );
                }
                else
                {
                    cell = new MapCell();
                    cellData = new int[ value ];
                    for( int f=0; f < value; f++ )
                    {
                        cellData[f]=reader.readInt();
                    }
                    cell.setFloor( cellData );
                    cell.setWall( reader.readInt() );
                    cell.setLight( reader.readByte() );
                    newMap.setCell( i, j, cell );
                }
            }
        // Then, read the doors
        value = reader.readInt();
        System.out.println( "Reading " + value + " doors" );
        for( int d=0; d<value;d++ )
        {
            Door door;
            int dX, dY, dDirection;
            dX = reader.readInt();
            dY = reader.readInt();
            dDirection = reader.readInt();
            System.out.println( "Door at ("+dX+","+dY+"), facing " + dDirection );
            door = new Door( dX, dY, dDirection );
            door.setLocked( reader.readBoolean() );
            door.setOpen( reader.readBoolean() );
            newMap.addDoor( door );
        }
        // Then, objects
        value = reader.readInt();
        for( int o=0; o< value; o++ )
        {
            MapObjectCell mapObject;
            int mX, mY, mT, mHP;
            boolean mBM, mBS;
            mX = reader.readInt();
            mY = reader.readInt();
            mT = reader.readInt()*2;
            mHP = reader.readInt();
            mBM = reader.readBoolean();
            mBS = reader.readBoolean();
            mapObject = new MapObjectCell( mX, mY, mT, mHP, mBM, mBS );
            newMap.addObjectCell( mapObject );
        }
        
        // Read the number of floor sets
        int floorSets = reader.readInt();
        // Read the names of the floorsets
        for( int i = 0; i < floorSets; i++)
        {
            newMap.addFloorSet( reader.readUTF() );
        }
        // Same for wall sets
        int wallSets = reader.readInt();
        for( int i = 0; i < wallSets; i++)
        {
            newMap.addWallSet( reader.readUTF() );
        }
        // And objects
        int objectSets = reader.readInt();
        for( int i = 0; i < objectSets; i++)
        {
            newMap.addObjectSet( reader.readUTF() );
        }
        // And doors
        int doorSets = reader.readInt();
        for( int i = 0; i < doorSets; i++)
        {
            newMap.addDoorSet( reader.readUTF() );
        }
        // Finally, read the name
        newMap.setName( reader.readUTF() );
        reader.close();
        fis.close();
        return newMap;
    }
	
    public void nudgeSelection( int direction )
    {
	switch( direction )
	{
	    case NORTH:
		nudgeNorth();
		break;
	    case SOUTH:
		nudgeSouth();
		break;
	    case EAST:
		nudgeEast();
		break;
	    case WEST:
		nudgeWest();
		break;
	}
    }
    
    private void nudgeEast()
    {
	for( int x=horizontalSize-2; x >= 0; x-- )
	{
	    for( int y=1; y < verticalSize; y++ )
	    {
		if( utilityMap[x][y] == MapMaker.SELECTED )
		{
		    MapCell start = cellMap[ x ][ y ];
		    cellMap[ x+1 ][ y ] = start;
		    cellMap[ x ][ y ] = null;
		}
	    }
	}
    }
	
	
    private void nudgeNorth()
    {
	for( int x=0; x < horizontalSize; x++ )
	{
	    for( int y=1; y < verticalSize; y++ )
	    {
		if( utilityMap[x][y] == MapMaker.SELECTED )
		{
		    MapCell start = cellMap[ x ][ y ];
		    cellMap[ x ][ y-1 ] = start;
		    cellMap[ x ][ y ] = null;
		}
	    }
	}
    }
	
    private void nudgeSouth()
    {
	for( int x=0; x < horizontalSize; x++ )
	{
	    for( int y=verticalSize-2; y >= 0; y-- )
	    {
		if( utilityMap[x][y] == MapMaker.SELECTED )
		{
		    MapCell start = cellMap[ x ][ y ];
		    cellMap[ x ][ y+1 ] = start;
		    cellMap[ x ][ y ] = null;
		}
	    }
	}
    }
	
    private void nudgeWest()
    {
	for( int x=1; x < horizontalSize; x++ )
	{
	    for( int y=1; y < verticalSize; y++ )
	    {
		if( utilityMap[x][y] == MapMaker.SELECTED )
		{
		    MapCell start = cellMap[ x ][ y ];
		    cellMap[ x-1 ][ y ] = start;
		    cellMap[ x ][ y ] = null;
		}
	    }
	}
    }
	
}