// ResourceManager.java
// Started on June 20th, 2001
// as ImageCropper.java
// Copyright (c) Virtu Halttunen

import java.awt.*;
import java.awt.image.*;
import javax.swing.JPanel;
import javax.swing.ImageIcon;
import java.util.Vector;
import java.net.*;
import java.applet.*;

public class ResourceManager
{
    private static final int ERROR_RETRIES = 5;
    private static Component dummy = new JPanel();
    private static ResourceManager loader = new ResourceManager();
    private static URLClassLoader classLoader;
    private static Vector urls;
    
    private ResourceManager()
    {
	URL url = getClass().getResource( "ResourceManager.class" );
	String urlString = url.toString();
	urlString = urlString.substring( 0, urlString.lastIndexOf( "!/" )+2 );
	System.out.println( "ResourceManager: Base URL: " + urlString );
	URL baseUrl = null;
	try
	{
	    baseUrl = new URL( urlString );
	}
	catch( MalformedURLException muex )
	{
	    System.out.println( "ResourceManager could not find base URL!" );
	    muex.printStackTrace();
	    System.exit( 1 );
	}
	urls = new Vector();
	urls.add( baseUrl );
	URL [] baseUrls = new URL[1];
	baseUrls[0] = baseUrl;
	classLoader = new URLClassLoader( baseUrls, getClass().getClassLoader() );
    }
    
    public static boolean addBaseURL( String path )
    {
	boolean success = true;
	URL url;
	try
	{
	    url = new URL( path );
	    urls.add( url );
	    URL [] baseUrls = new URL[urls.size()];
	    for( int i=0; i < urls.size(); i++ )
	    {
		baseUrls[i] = (URL)urls.elementAt( i );
	    }
	    classLoader = new URLClassLoader( baseUrls, loader.getClass().getClassLoader() );
	    System.out.println( "ResourceManager: Added base URL " + url.toString() );
	}
	catch( MalformedURLException muex )
	{
	    System.out.println( "ResourceManager: malformed URL: " + path );
	    success = false;
	}
	return success;
    }
    
    public static void cropImage( String imageName, Vector vector, int cellSize )
    {
        Image theSetImage = null;
	theSetImage = loadImage( imageName );
	BufferedImage theImage;
	if( ( theSetImage.getWidth( dummy ) % cellSize == 0 ) && ( theSetImage.getHeight( dummy ) % cellSize == 0 ) )
	{
	    for( int i = 0; i < theSetImage.getHeight( dummy )/cellSize; i++ )
		for( int j = 0; j < theSetImage.getWidth( dummy )/cellSize; j++ )
		{
                    theImage = new BufferedImage( cellSize, cellSize, BufferedImage.TYPE_INT_ARGB_PRE );
                    theImage.getGraphics().drawImage( theSetImage, -j*cellSize, -i*cellSize, dummy );
		    vector.addElement( theImage );
		}
	}
	System.out.println( "ResourceManager: Cropped image " + imageName +", " + vector.size() + " images found" );
    }
    
    public static String getBaseResourceURL()
    {
	return ((URL)urls.elementAt( 0 )).toString();
    }
	
    public static URL findResource( String name )
    {
	return classLoader.findResource( name );
    }
	
    public static Image [][] cropImage( String imageName, int xImages, int cellSize )
    {
	Vector vector = new Vector();
	cropImage( imageName, vector, cellSize );
	Image [][] images = new Image[ vector.size()/xImages ][ xImages ];
	int index=0;
	for( int i = 0; i < vector.size()/xImages; i++ )
	{
	    for( int j = 0; i < xImages; j++ )
	    {
		images[ i ][ j ] = (Image)vector.elementAt( index++ );
	    }
	}
	return images;
    }

    public static AudioClip loadAudioClip( String audioFileName )
    {
	return Applet.newAudioClip( classLoader.getResource( audioFileName ) );
    }
	
    public static Image loadImage( String imageName )
    {
	ImageIcon imageLoader;
	boolean found = false;
	Image loadedImage = null;
	BufferedImage theImage = null;
	int width = 0, height = 0;
	int urlIndex = urls.size();
	URL baseUrl;
	int tries = 0;
        boolean errors;
	
	
	/*
	while( !found && ( urlIndex > 0 ) )
	{
	    // Try the different base URLs until the image is found or we cannot find it
	    baseUrl = (URL)urls.elementAt( --urlIndex );
	    try
	    {
		System.out.println( "Trying to load image at " + baseUrl.toString() + imageName );
		imageLoader = new ImageIcon( new URL( baseUrl, imageName ) );
		// If we don't hit an exception, we have found the image
		do
		{		
		    errors = false;
		    try
		    {
			loadedImage = imageLoader.getImage();
			width = loadedImage.getWidth( dummy );
			height = loadedImage.getHeight( dummy );
			//System.out.println( "Image size: " + width + "x" + height );
			if( width > 0 && height > 0 )
			{
			    found = true;
			    System.out.println( "Loaded image " + imageName );
			}
		    }
		    catch( Exception inex )
		    {
			System.out.println( "ResourceManager: Error loading image! " + inex.toString() );
			errors = true;
			tries++;
		    }
		}
		while( errors && ( tries < ERROR_RETRIES ) );
	    }
	    catch( Exception ex )
	    {
		System.out.println( "Image not found at " + baseUrl.toString() );
	    }
	    
	}*/
	System.out.println( "Loading image: " + imageName );
	imageLoader = new ImageIcon( classLoader.findResource( imageName ) );
	do
	{		
	    errors = false;
	    try
	    {
		loadedImage = imageLoader.getImage();
		width = loadedImage.getWidth( dummy );
		height = loadedImage.getHeight( dummy );
		//System.out.println( "Image size: " + width + "x" + height );
		if( width > 0 && height > 0 )
		{
		    found = true;
		    System.out.println( "Loaded image " + imageName + " (" + width + " x " + height + ")");
		}
	    }
	    catch( Exception inex )
	    {
		System.out.println( "ResourceManager: Error loading image! " + inex.toString() );
		errors = true;
		tries++;
	    }
	}
	while( errors && ( tries < ERROR_RETRIES ) );
	
	if( found )
	{
	    //System.out.println( "Image size: " + width + "x" + height );
	    if( ( width > 0 ) && ( height > 0 ) )
	    {
		theImage = new BufferedImage( width, height, BufferedImage.TYPE_INT_ARGB_PRE );
		Graphics2D g2 = theImage.createGraphics();
		g2.drawImage( loadedImage, 0, 0, dummy );
		g2.dispose();
	    }
	}
	else
        {
            System.out.println( "Could not load image in " + tries + " attempts, giving up!" );
            //System.exit( 0 );
        }
	return theImage;
    }
	
    public static void setObserver( Component observer )
    {
        dummy = observer;
    }
    
}