// CharacterInfoPanel.java
// Started on May 22nd, 2002
// Copyright (c) Virtu Halttunen

import TheGame.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class CharacterInfoPanel extends JPanel implements ActionListener, MouseListener, ChangeListener
{
    private CharacterReference character = null;
    private MissionEditor editor = null;
    private CharacterSprite sprite;
    private ImageIcon characterIcon;
    private JLabel iconLabel;
    private JTextField characterClassField;
    private JTextField locationField;
    private JTextField classMinimumRandomField;
    private JTextField classMaximumRandomField;
    private JTextField nameField;
    private JTextField customHitPointsField;
    private JTextField customActionPointsField;
    private JCheckBox dormantButton;
    private JButton deleteButton;
    private JSpinner spinner;
    private SpinnerNumberModel spinnerModel;
    
    private JButton previousButton;
    private JButton nextButton;
    private JTextField indexField;

    public CharacterInfoPanel( MissionEditor editor ) 
    {
        this.editor = editor;
        Box contentBox = Box.createVerticalBox();
        characterClassField = new JTextField();
        contentBox.add( characterClassField );
        characterIcon = new ImageIcon( "ArrowIcon.png");
        iconLabel = new JLabel( characterIcon );
        iconLabel.addMouseListener( this );
        Dimension ilSize = new Dimension( 256, 256 );
        iconLabel.setMinimumSize( ilSize );
        iconLabel.setMaximumSize( ilSize );
        iconLabel.setPreferredSize( ilSize );
        contentBox.add( iconLabel );
        Box nameBox = Box.createHorizontalBox();
  	nameBox.add( new JLabel( "Name" ) );
	nameBox.add( Box.createHorizontalStrut( 3 ) );
	nameField = new JTextField( 10 );
	nameField.addActionListener( this );
	nameBox.add( nameField );	
	nameBox.add( Box.createHorizontalStrut( 3 ) );
	dormantButton = new JCheckBox( "Dormant", false );
	dormantButton.addActionListener( this );
	nameBox.add( dormantButton );
	nameBox.add( Box.createHorizontalGlue() );
        contentBox.add( nameBox );
	contentBox.add( Box.createVerticalStrut( 5 ) );
	Box customBox = Box.createHorizontalBox();
	customBox.add( new JLabel( "Custom HP" ) );
	customBox.add( Box.createHorizontalStrut( 3 ) );
	customHitPointsField = new JTextField( 7 );
	customHitPointsField.setHorizontalAlignment( SwingConstants.RIGHT );
	customHitPointsField.addActionListener( this );
	customBox.add( customHitPointsField );
	customBox.add( Box.createHorizontalStrut( 3 ) );
	customBox.add( new JLabel( "Custom AP" ) );
	customBox.add( Box.createHorizontalStrut( 3 ) );
	customActionPointsField = new JTextField( 5 );
	customActionPointsField.setHorizontalAlignment( SwingConstants.RIGHT );
	customActionPointsField.addActionListener( this );
	customBox.add( customActionPointsField );
	contentBox.add( customBox );
	contentBox.add( Box.createVerticalStrut( 5 ) );
	
        JPanel buttons = new JPanel();
	
	buttons.add( new JLabel( "Location" ) );
	buttons.add( Box.createHorizontalStrut( 3 ) );
        locationField = new JTextField( 10 );
        locationField.addActionListener( this );
        buttons.add( locationField );
	buttons.add( Box.createHorizontalStrut( 30 ) );
	buttons.add( Box.createHorizontalGlue() );
	
	spinnerModel = new SpinnerNumberModel( 0, -1, 0, 1 );
	spinner = new JSpinner( spinnerModel );
	spinner.addChangeListener( this );
	buttons.add( spinner );
        deleteButton = new JButton( "Delete" );
        deleteButton.addActionListener( this );
        buttons.add( deleteButton );
        contentBox.add( buttons );
        // Class random number fields
        JPanel cMinPanel = new JPanel();
        cMinPanel.add( new JLabel( "Minimum " ) );
        classMinimumRandomField = new JTextField( "0", 7 );
        classMinimumRandomField.setHorizontalAlignment( JTextField.RIGHT );
        classMinimumRandomField.addActionListener( this );
        cMinPanel.add( classMinimumRandomField );
	cMinPanel.add( Box.createHorizontalStrut( 10 ) );
        //JPanel cMaxPanel = new JPanel();
        cMinPanel.add( new JLabel( "Maximum " ) );
        classMaximumRandomField = new JTextField( "0", 7 );
        classMaximumRandomField.setHorizontalAlignment( JTextField.RIGHT );
        classMaximumRandomField.addActionListener( this );
        cMinPanel.add( classMaximumRandomField );
        JPanel randomPanel = new JPanel();
        Box randomBox = Box.createVerticalBox();
        randomBox.add( cMinPanel );
        //randomBox.add( cMaxPanel );
        randomPanel.add( randomBox );
        randomPanel.setBorder( new TitledBorder( new BevelBorder( BevelBorder.LOWERED ), "Random" ) );
        contentBox.add( randomPanel );
        add( contentBox );
    }

    public void actionPerformed( ActionEvent e )
    {
        if( e.getSource() == locationField )
        {
            try
            {
                Point point = MissionEditor.pointFromString( locationField.getText() );
                if( character != null )
                {
                    character.setLocation( point );
                    editor.centerOnCharacter( character );
                }
            }
            catch( IllegalArgumentException iae )
            {
                editor.showError( iae.getMessage() );
            }
        }
	if( e.getSource() == nameField || e.getSource() == customHitPointsField || e.getSource() == customActionPointsField )
	{
	    String name = nameField.getText().trim();
	    String customHitPoints = customHitPointsField.getText().trim();
	    String customActionPoints = customActionPointsField.getText().trim();
	    if( character != null && name.length() > 0 )
	    {
		character.setName( name );
	    }
	    if( character != null && customHitPoints.length() > 0 )
	    {
		character.setCustomHitPoints( customHitPoints );
	    }
	    if( character != null && customActionPoints.length() > 0 )
	    {
		character.setCustomActionPoints( customActionPoints );
	    }
	}	
        if( e.getSource() == deleteButton )
        {
            if( character != null )
            {
		int index = ((Integer)spinner.getValue()).intValue();
                if( editor.isPlayerCharacter( character ) )
                {
                    editor.removePlayerCharacter( index );
		    if( index >= editor.getPlayerCharacters().size() )
		    {
			index=0;
			setCharacter( editor.getMonster( 0 ) );
		    }
		    else
		    {
			setCharacter( editor.getPlayerCharacter( index ) );
		    }
                 }
                else
                {
                    editor.removeMonster( index );
		    if( index >= editor.getMonsters().size() )
		    {
			setCharacter( editor.getPlayerCharacter( 0 ) );
		    }
		    else
		    {
			setCharacter( editor.getMonster( index ) );
		    }
  		}	
            }
            else
            {
                Toolkit.getDefaultToolkit().beep();
            }
        }
        if( e.getSource() == classMinimumRandomField || e.getSource() == classMaximumRandomField )
        {
            int min = 0;
            int max = 0;
            try
            {
                min = Integer.parseInt( classMinimumRandomField.getText() );
                max = Integer.parseInt( classMaximumRandomField.getText() );
                if( character != null )
                {
                    editor.setRandomBoundsForCharacterClass( character.getClassName(), min, max );
                    classMinimumRandomField.setText( String.valueOf( editor.getRandomBoundsMinimumForCharacterClass( character.getClassName() ) ) );
                    classMaximumRandomField.setText( String.valueOf( editor.getRandomBoundsMaximumForCharacterClass( character.getClassName() ) ) );
                }
                else
                {
                    editor.setRandomBoundsForCharacterClass( min, max );
                }
                
            }
            catch( NumberFormatException nfe )
            {
                editor.showError( "Not a number" );
            }
        }
	if( e.getSource() == dormantButton )
	{
	    if( character != null )
	    {
		character.setDormant( dormantButton.isSelected() );
	    }
	}
    }

    public void mouseClicked( MouseEvent e )
    {
        if( e.getSource() == iconLabel && character != null )
        {
            if( e.getClickCount() >= 2 )
            {
                editor.centerOnCharacter( character );
            }
        }
    }
    
    public void mousePressed( MouseEvent e )
    {}

    public void mouseReleased( MouseEvent e )
    {}

    public void mouseEntered( MouseEvent e )
    {}

    public void mouseExited( MouseEvent e )
    {}
    
    public void setCharacter( CharacterReference character )
    {
        this.character = character;
        if( character != null )
        {
            characterClassField.setText( character.getType() );
            locationField.setText( MissionEditor.pointToString( character.getLocation() ) );
            try
            {
                String characterClass = character.getClassName();
		if( characterClass.endsWith( MissionEditor.GENERIC_MONSTER_KEY ) )
		{
		    String type = characterClass.substring( 0, characterClass.indexOf( "(" ) );
		    GenericMonster.create( type );
		    sprite = new GenericMonsterSprite();
		    ((GenericMonsterSprite)sprite).setType( type );
		}
		else
		{
		    sprite = (CharacterSprite)Class.forName( "TheGame."+characterClass+"Sprite" ).newInstance();
		}
                sprite.setCharacter( character );
                sprite.setDirection( character.getDirection() );
                GameAction action = sprite.getCharacter().getLastAction();
                characterIcon.setImage( sprite.getSprite( action ) );
                iconLabel.repaint();
		iconLabel.setEnabled( true );
                classMinimumRandomField.setText( String.valueOf( editor.getRandomBoundsMinimumForCharacterClass( characterClass ) ) );
                classMaximumRandomField.setText( String.valueOf( editor.getRandomBoundsMaximumForCharacterClass( characterClass ) ) );
            }
            catch( Exception e )
            {
                System.out.println( e.toString() );
            }
	    nameField.setText( "" );
	    if( character.getName() != null )
	    {
		nameField.setText( character.getName() );
	    }
	    customHitPointsField.setText( "" );
	    if( character.getCustomHitPoints() != null )
	    {
		customHitPointsField.setText( character.getCustomHitPoints() );
	    }
	    customActionPointsField.setText( "" );
	    if( character.getCustomActionPoints() != null )
	    {
		customActionPointsField.setText( character.getCustomActionPoints() );
	    }
	    
            int index = editor.getCharacterIndex( character );
	    spinner.setValue( new Integer( index ) );
	    if( editor.isPlayerCharacter( character ) )
	    {
		spinnerModel.setMaximum( new Integer( editor.getNumberOfPlayerStartPoints()+1 ) );
	    }
	    else
	    {
		spinnerModel.setMaximum( new Integer( editor.getMonsters().size() ) );
	    }
	    dormantButton.setSelected( character.isDormant() );
            //indexField.setText( index + " / " + (editor.getMonsters().size() + editor.getNumberOfPlayerStartPoints() ) );
        }
        else
        {
            characterClassField.setText( "" );
            locationField.setText( "" );
	    customActionPointsField.setText( "" );
	    customHitPointsField.setText( "" );
	    nameField.setText( "" );
	    dormantButton.setSelected( false );
	    iconLabel.setEnabled( false );
        }
    }

    public void setCharacterDirection( int direction )
    {
        if( character != null )
        {
            character.setDirection( direction );
            sprite.setDirection( direction );
            GameAction action = sprite.getCharacter().getLastAction();
            characterIcon.setImage( sprite.getSprite( action ) );
            iconLabel.repaint();
            editor.centerOnCharacter( character );
        }
    }
    
    public void stateChanged( ChangeEvent e )
    {
	if( e.getSource() == spinner )
	{
	    //System.out.println( "Spinner state changed" );
	    int index = ((Integer)spinner.getValue()).intValue();
	    CharacterReference newCharacter = null;
	    if( editor.isPlayerCharacter( character ) )
	    {
		if( index < 0 )
		{
		    if( editor.getMonsters().size() > 0 )
		    {
			newCharacter = editor.getMonster( editor.getMonsters().size()-1 );
		    }
		    else
		    {
			newCharacter = editor.getPlayerCharacter( editor.getPlayerCharacters().size()-1 );
		    }
		}
		else if( index >= editor.getPlayerCharacters().size() )
		{
		    if( editor.getMonsters().size() > 0 )
		    {
			newCharacter = editor.getMonster( 0 );
		    }
		    else
		    {
			newCharacter = editor.getPlayerCharacter( 0 );
		    }
		}
		else
		{
		    newCharacter = editor.getPlayerCharacter( index );
		}
	    }
	    else
	    {
		if( index < 0 )
		{
		    if( editor.getPlayerCharacters().size() > 0 )
		    {
			newCharacter = editor.getPlayerCharacter( editor.getPlayerCharacters().size()-1 );
		    }
		    else
		    {
			newCharacter = editor.getMonster( editor.getMonsters().size()-1 );
		    }
		}
		else if( index >= editor.getMonsters().size() )
		{
		    if( editor.getPlayerCharacters().size() > 0 )
		    {
			newCharacter = editor.getPlayerCharacter( 0 );
		    }
		    else
		    {
			newCharacter = editor.getMonster( 0 );
		    }
		}
		else
		{
		    newCharacter = editor.getMonster( index );
		}
	    }
	    setCharacter( newCharacter );
	}
    }
}