// GameOverviewMapPanel.java
// Started on June 29th, 2001
// Copyright (c) Virtu Halttunen

import java.awt.*;
import javax.swing.*;

public class EditorOverviewMapPanel extends JPanel
{
    private Color backgroundColor = Color.black;
    private Color floorColor = Color.darkGray;
    private Color wallColor = Color.lightGray;
    private Color doorColor = Color.yellow;
    private Color friendlyColor = Color.green;
    private Color hostileColor = Color.red;
    private Color blipColor = Color.cyan;
    private Color objectiveColor = Color.orange;
    
    private int cellSize = 2;
    private MissionEditor editor;
    private boolean showHostile = true;
    private boolean showBlip = false;
    private Insets insets;
    
    public EditorOverviewMapPanel( MissionEditor editor )
    {
	super();
	this.editor = editor;
	Dimension mySize = new Dimension ( editor.getMap().getHorizontalSize()*cellSize, editor.getMap().getVerticalSize()*cellSize );
	setMinimumSize( mySize );
	setPreferredSize( mySize );
	setMaximumSize( mySize );
	insets = new Insets( 5, 5, 5, 5 );
    }
    
    protected void paintComponent( Graphics g )
    {
	//super.paintComponent( g );
	int x=0, y=0;
	TheGame.Character character;
	
	// Draw the general map features
	for( int i = 0; i < editor.getMap().getHorizontalSize(); i++ )
	    for( int j = 0; j < editor.getMap().getVerticalSize(); j++ )
	    {
		drawCell( g, i, j );
	    }
	// Draw the player starting points
	g.setColor( friendlyColor );
	Point point;
	for( int i=0; i < editor.getNumberOfPlayerStartPoints(); i++ )
	{
	    point = editor.getPlayerCharacter( i ).getLocation();
	    g.fillRect( point.x*cellSize, point.y*cellSize, cellSize, cellSize );
	}
        // Draw the monsters
        g.setColor( hostileColor );
        for( int i=0; i < editor.getMonsters().size(); i++ )
        {
            point = editor.getMonster( i ).getLocation();
	    g.fillRect( point.x*cellSize, point.y*cellSize, cellSize, cellSize );
        }
    }

    private void drawCell( Graphics g, int x, int y )
    {
        TheGame.Objective objective;
        TheGame.MapCell cell =  editor.getMap().getCell( x, y );
        g.setColor( backgroundColor );
        if( cell != null )
        {
            if( cell.getFloor() > -1 )
            {
                g.setColor( floorColor );
            }
            if( cell.getWall() > -1 )
            {
                g.setColor( wallColor );
            }
            if( cell.hasDoor() )
            {
                if( !editor.getMap().getDoor( x, y ).isOpen() )
                    g.setColor( doorColor );
            }
            for( int i = 0; i < editor.getNumberOfObjectives(); i++ )
            {
                try
                {
                    objective = editor.getObjective( i );
                    if( objective.isShownOnMap() && objective.isLocatedIn( x, y ) )
                        g.setColor( objectiveColor );
                }
                catch( Exception e )
                {}
            }
	    objective = editor.getPrimaryObjective();
	    if( objective != null )
	    {
		if( objective.isShownOnMap() && objective.isLocatedIn( x, y ) )
		{
		    g.setColor( objectiveColor );
		}
	    }
	    objective = editor.getSecondaryObjective();
	    if( objective != null )
	    {
		if( objective.isShownOnMap() && objective.isLocatedIn( x, y ) )
		{
		    g.setColor( objectiveColor );
		}
	    }
	    
        }
        g.fillRect( x*cellSize, y*cellSize, cellSize, cellSize );
    }

    
    public int getCellSize()
    {
	return cellSize;
    }
    
    public Insets getInsets()
    {
	return insets;
    }
    
    public Insets getInsets( Insets newInsets )
    {
	newInsets.top = insets.top;
	newInsets.bottom = insets.bottom;
	newInsets.left = insets.left;
	newInsets.right = insets.right;
	return newInsets;
    }
}
