// ObjectiveTableModel.java
// Started on May 18th, 2002
// Copyright (c) Virtu Halttunen

import TheGame.*;
import javax.swing.table.*;

public class ObjectiveTableModel extends AbstractTableModel
{
    private Objective objective = null;
    private MissionEditor theEditor = null;

    public ObjectiveTableModel()
    {
    }

    public void setMissionEditor( MissionEditor editor )
    {
        theEditor = editor;
    }
    
    public void setObjective( Objective objective )
    {
        this.objective = objective;
        fireTableStructureChanged();
    }
    
    public Objective getObjective()
    {
	return objective;
    }

    public String getObjectiveClassName()
    {
        if( objective == null )
        {
            return "";
        }
        return objective.getClass().getName();
    }

    public int getColumnCount()
    {
        return 2;
    }

    public int getRowCount()
    {
        if( objective != null )
        {
            if( objective.getPropertyNames() == null )
            {
                return 10;
            }
	    //System.out.println( "Table has " + objective.getPropertyNames().length + " rows" );
            return objective.getPropertyNames().length;
        }
        else
            return 10;
    }

    public Object getValueAt( int row, int column )
    {
        if( objective == null )
        {
            return null;
        }
        if( objective.getPropertyNames() == null )
        {
            return null;
        }
        if( row >= objective.getPropertyNames().length || column >= 2 )
        {
            return null;
        }
        String [] properties = objective.getPropertyNames();
        if( column == 0 )
        {
            return properties[ row ];
        }
        if( column == 1 )
        {
            return objective.getProperty( properties[ row ] );
        }
        return null;
    }

    public String getColumnName( int column )
    {
        if( column == 0 )
        {
            return "Property";
        }
        if( column == 1 )
        {
            return "Value(s)";
        }
        return "Undefined";
    }

    public Class getColumnClass( int columnIndex )
    {
        return String.class;
    }

    public boolean isCellEditable( int rowIndex, int columnIndex )
    {
        if( columnIndex == 1 )
        {
            return true;
        }
        return false;
    }

    public void setValueAt( Object aValue, int rowIndex, int columnIndex )
    {
        if( columnIndex == 1 && objective != null )
        {
            try
            {
                String [] properties = objective.getPropertyNames();
                objective.setProperty( properties[ rowIndex ], (String)aValue );
                fireTableDataChanged();
            }
            catch( Exception e )
            {
                System.out.println( e.toString() );
                if( theEditor != null )
                {
                    String message;
                    if( e.getMessage() != null )
                    {
                        message = e.getMessage();
                    }
                    else
                    {
                        message = e.toString();
                    }
                    theEditor.showError( message );
                }
            }
        }
    }
}