// ActivateMonstersObjective.java
// Started on August 2nd, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;
import java.util.Vector;

/**
 *
 *  This Objective grants the monsters a free turn immediately when it is
 *  discovered by the player or triggered by other means. Nasty.
 */
public class ActivateMonstersObjective extends Objective
{    
    public static final String TRIGGER_KEY = "Trigger";
    public static final String [] properties = { Objective.LOCATION_KEY, TRIGGER_KEY };
    
    private boolean triggered = false;
    private Point location = new Point( -1, -1 );
    
    public ActivateMonstersObjective()
    {
    }

    public void setGame( Game activeGame )
    {
	this.activeGame = activeGame;
    }
    
    public void trigger()
    {
	if( !triggered )
	{
	    activeGame.doMonsterActions();
	    for( int i=0; i < activeGame.getMonsters().size(); i++ )
	    {
		Monster monster = activeGame.getMonster( i );
		if( monster.isAlive() )
		{
		    monster.resetActionPoints();
		}
	    }
	}
	triggered = true;
    }
    
    public int getNumberOfSprites()
    {
	return 0;
    }    
    
    public Sprite getSprite( int x, int y)
    {
	return null;
    }
    
    public String [] getPropertyNames()
    {
	return properties;
    }
    
    public Class getPropertyClass( String key )
    {
	if( key.equals( Objective.LOCATION_KEY ) )
	{
	    return Point.class;
	}
	return String.class;
    }
    
    public String getProperty( String key )
    {
	if( key.equals( Objective.LOCATION_KEY ) )
	{
	    return Objective.pointToString( location );
	}
	return null;
    }
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
	if( key.equals( Objective.LOCATION_KEY ) )
	{
	    location = Objective.pointFromString( value );
	}	
	if( key.equals( TRIGGER_KEY ) )
	{
	    if( value != null && value.equalsIgnoreCase( "True" ) )
	    {
		trigger();
	    }
	}
    }    
    
    public boolean isCompleted()
    {
	return true;
    }
    
    public int getSize()
    {
	return 0;
    }
    
    public boolean isLocatedIn( Point point )
    {
	return location.equals( point );
    }
    
    public boolean isLocatedIn( int x, int y )
    {
	return isLocatedIn( new Point( x, y ) );
    }
    
    public boolean isDestructible()
    {
	return false;
    }
    
    public boolean isShownOnMap()
    {
	return false;
    }
    
    public boolean handleDamage( Point point, int damage )
    {
	return false;
    }
    
    public boolean handleAction( GameAction action )
    {
	Character actor = action.getActor();
	if( activeGame.isPlayerCharacter( actor ) && !triggered )
	{
	    if( activeGame.isCellVisibleToPlayer( location ) )
	    {
		trigger();
	    }
	}
	return false;
    }
}