// AirlockDoorsObjective.java
// Started on July 3rd, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;

public class AirlockDoorsObjective extends Objective
{
    public static final String FIRST_DOOR_KEY = "First door";
    public static final String SECOND_DOOR_KEY = "Second door";
    public static final String REMOTE_OPERATED_KEY = "Remote operated";
    public static final String REMOTE_MESSAGE_KEY = "Remote message";
    public static final String TOGGLE_KEY = "Toggle";

    private String [] properties = { FIRST_DOOR_KEY, SECOND_DOOR_KEY, REMOTE_OPERATED_KEY, REMOTE_MESSAGE_KEY, TOGGLE_KEY };
    
    private Point first = new Point( -1, -1 );
    private Point second = new Point( -1, -1 );
    private boolean remote = false;
    private String remoteMessage = "This airlock is remotely operated";
    private boolean doorSwitch = true;

    public AirlockDoorsObjective()
    {}

    public void setGame( Game game )
    {
        activeGame = game;
	activeGame.closeDoor( null, first );
        if( second.x > 0 )
        {
            activeGame.openDoor( null, second );
        }
        if( remote )
        {
            lockDoor( first, true );
            if( second.x > 0 )
            {
                lockDoor( second, true );
            }
        }
    }

    public String [] getPropertyNames()
    {
        return properties;
    }

    public String getProperty( String key )
    {
        if( key.equals( FIRST_DOOR_KEY ) )
        {
            return Objective.pointToString( first );
        }
        if( key.equals( SECOND_DOOR_KEY ) )
        {
            return Objective.pointToString( second );
        }
        if( key.equals( REMOTE_OPERATED_KEY ) )
        {
            return Objective.booleanToString( remote );
        }
        if( key.equals( REMOTE_MESSAGE_KEY ) )
        {
            return remoteMessage;
        }
        if( key.equals( TOGGLE_KEY ) )
        {
            return "";
        }
        return "";
    }
    
    public Class getPropertyClass( String key )
    {
        if( key.equals( FIRST_DOOR_KEY ) )
        {
            return Door.class;
        }
        if( key.equals( SECOND_DOOR_KEY ) )
        {
            return Door.class;
        }
	return String.class;
    }    

    public void setProperty( String key, String value )
    {
        if( key.equals( FIRST_DOOR_KEY ) )
        {
            first = Objective.pointFromString( value );
        }
        if( key.equals( SECOND_DOOR_KEY ) )
        {
            second = Objective.pointFromString( value );
        }
        if( key.equals( REMOTE_OPERATED_KEY ) )
        {
            remote = Objective.booleanFromString( value );
        }
        if( key.equals( REMOTE_MESSAGE_KEY ) )
        {
            remoteMessage = value;
        }
        if( key.equals( TOGGLE_KEY ) )
        {
            if( activeGame != null )
            {
                toggleDoors();
            }
        }
    }

    private void lockDoor( Point door, boolean lock )
    {
        Door theDoor =  activeGame.getMap().getDoor( door.x, door.y );
        if( theDoor != null )
        {
            theDoor.setLocked( lock );
        }
        else
        {
            System.out.println( "AirlockDoorsObjective: Missing door at " + Objective.pointToString( door ) + "!" );
        }
    }

    public void openBoth()
    {
	lockDoor( first, false );
	lockDoor( second, false );
	activeGame.openDoor( null, first );
	activeGame.openDoor( null, second );
    }
    
    private void toggleDoors()
    {
        if( remote )
        {
            lockDoor( first, false );
            if( second.x > 0 )
            {
                lockDoor( second, false );
            }
        }
        if( doorSwitch )
        {
            if( second.x > 0 )
            {
                activeGame.closeDoor( null, second );
            }
            activeGame.openDoor( null, first );
        }
        else
        {
            activeGame.closeDoor( null, first );
            if( second.x > 0 )
            {
                activeGame.openDoor( null, second );
            }
        }
        if( remote )
        {
            lockDoor( first, true );
            if( second.x > 0 )
            {
                lockDoor( second, true );
            }
        }
        doorSwitch = !doorSwitch;
    }

    public boolean isLocatedIn( Point point )
    {
        //System.out.println( "AirlockDoorsObjective: isLocatedIn " + Objective.pointToString( point ) );
        //System.out.println( "First: " + Objective.pointToString( first ) + ", second: " + Objective.pointToString( second ) );
        if( activeGame == null )
        {
            return false;
        }
        if( point.equals( first ) )
        {
            return true;
        }
        Point otherHalf;
        Door theDoor = activeGame.getMap().getDoor( first.x, first.y );
        Door otherDoor;
        if( theDoor != null )
        {
            otherHalf = activeGame.getMap().getNeighbor( new Point( theDoor.getX(), theDoor.getY() ), theDoor.getDirection() );
            otherDoor = activeGame.getMap().getDoor( otherHalf.x, otherHalf.y );
            if( otherDoor != null )
            {
                if( otherHalf.equals( point ) )
                {
                    return true;
                }
            }
        }
        if( second.x > 0 )
        {
            if( point.equals( second ) )
            {
                return true;
            }
            theDoor = activeGame.getMap().getDoor( second.x, second.y );
            if( theDoor != null )
            {
                otherHalf = activeGame.getMap().getNeighbor( new Point( theDoor.getX(), theDoor.getY() ), theDoor.getDirection() );
                otherDoor = activeGame.getMap().getDoor( otherHalf.x, otherHalf.y );
                if( otherDoor != null )
                {
                    if( otherHalf.equals( point ) )
                    {
                        return true;
                    }
                }
            }
        }
        return false;
    }
    
    public boolean handleAction( GameAction action )
    {
	//System.out.println( "AirlockDoors: handleAction() " );
        Character character = action.getActor();
	if( action instanceof UseAction )
	{
	    Point point = ((UseAction)action).getPoint();
	    if( isLocatedIn( point ) )
	    {
		if( remote && character instanceof Marine )
		{
		    activeGame.handleMessage( new Message( character, remoteMessage ) );
		    ((UseAction)action).setCost( 1 );
		    ((UseAction)action).setConsumed( true );
		    return true;
		}
		System.out.println( character.getClassName() + " using airlock doors" );
		if( !remote )
		{
		    toggleDoors();
		}
		((UseAction)action).setCost( 1 );
		((UseAction)action).setConsumed( true );
		return true;
	    }
        }
        return false;
    }
    
    public boolean isUsable()
    {
	return true;
    }

}