// AssassinateObjective.java
// Started on June 4th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;

public class AssassinateObjective extends Objective
{
    public static final String TARGET_TYPE_KEY = "Target type";
    public static final String TARGET_INDEX_KEY = "Target index"; // Deprecated, use ID
    public static final String TARGET_ID_KEY = "Target id";
    public static final String SHOWN_ON_MAP_KEY = "Show target";
    public static final String DESCRIPTION_KEY = "Description";
    public static final String COMPLETION_KEY = "Win message";
    
    private String [] properties = { TARGET_TYPE_KEY, TARGET_ID_KEY, SHOWN_ON_MAP_KEY };
    
    protected Game activeGame;
    private Character target = null;
    private String targetType = "CyberBrute";
    private String targetId = null;
    private boolean showOnMap = false;
    
    public void setGame( Game game )
    {
        activeGame = game;
        if( targetId == null )
        {
	    // Try to search for a type
	    for( int i=0; i < game.getMonsters().size() && target == null; i++ )
	    {
		Character character = game.getMonster( i );
		if( character.getType().equals( targetType ) && character.isAlive() )
		{
		    target = character;
		}
	    }
	    if( target == null )
	    {
		// Create new target
		try
		{
		    target = (Character)Class.forName( targetType ).newInstance();
		}
		catch( Exception e )
		{
		    target = new CyberBrute();
		}
		target.setGame( game );
		target.setLocation( game.getRandomPointForMonster() );
		target.setDirection( 0 );
		target.setId( "Assassinate"+System.currentTimeMillis() );
		game.addMonster( target );
	    }
        }
    }

    public boolean isCompleted()
    {
        Character character;
        if( targetId != null )
        {
            character = activeGame.getMonsterById( targetId );
	    //System.out.println( "AssassinateObjective target: " + character.getType() + " " + character.getName() );
        }
        else
        {
            character = target;
        }
        return( !character.isAlive() );
    }

    public boolean isSuccessful()
    {
	return isCompleted();
    }    
    
    public boolean isShownOnMap()
    {
        return showOnMap;
    }
    
    public boolean isLocatedIn( Point point )
    {
        return isLocatedIn( point.x, point.y );
    }

    public boolean isLocatedIn( int x, int y )
    {
        Character character;
	if( activeGame == null )
	{
	    return false;
	}
        if( targetId != null )
        {
            character = activeGame.getMonsterById( targetId );
        }
        else
        {
            character = target;
        }
        if( character.getLocation().x == x && character.getLocation().y == y )
        {
            return true;
        }
        return false;
    }
        
    public String [] getPropertyNames()
    {
        return properties;
    }
    
    public Class getPropertyClass( String key )
    {
        if( key.equals( TARGET_INDEX_KEY ) )
        {
            return Monster.class;
        }
        if( key.equals( SHOWN_ON_MAP_KEY ) )
        {
            return Boolean.class;
        }
        return String.class;
    }
    
    public String getProperty( String key )
    {
        if( key.equals( TARGET_TYPE_KEY ) )
        {
            return targetType;
        }
        if( key.equals( TARGET_ID_KEY ) )
        {
            return String.valueOf( targetId );
        }
        if( key.equals( SHOWN_ON_MAP_KEY ) )
        {
            if( showOnMap )
            {
                return "True";
            }
            return "False";
        }
        return null;
    }

    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( TARGET_TYPE_KEY ) )
        {
            targetType = value;
        }
        if( key.equals( TARGET_ID_KEY ) )
        {
            targetId = value;
        }
        if( key.equals( TARGET_INDEX_KEY ) ) // Deprecated
        {
            targetId = value;
        }
         if( key.equals( SHOWN_ON_MAP_KEY ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                showOnMap = true;
            }
            else
            {
                showOnMap = false;
            }
        }
    }

}