// AssaultRifle.java
// Started on February 18th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import java.util.Vector;

public class AssaultRifle extends Weapon
{
    public static final int GRENADE = 1;

    public static final int COST = 3;
    
    private final String [] firingModes = { "Normal", "Grenade" };
    private static Image [] icons = null;
    
    private int mode = 0; // Defaults to normal
    private int grenades = 6;
    private boolean showGrenades = false;	// This is for a nasty hack requested by mjr

    public AssaultRifle()
    {
        if( icons == null )
        {
	    icons = new Image[ 7 ];
	    Image baseIcon = ResourceManager.loadImage( "WeaponGraphics/AssaultRifleIcon.png" );
	    icons[ 0 ] = baseIcon;
	    Image grenadeIcon = ResourceManager.loadImage( "WeaponGraphics/GrenadeIcon.png" );
	    JPanel dummy = new JPanel();
	    for( int i=1; i <= 6; i++ )
	    {
		BufferedImage newIcon = new BufferedImage( 100, 30, BufferedImage.TYPE_INT_ARGB_PRE );
		Graphics2D g2d = newIcon.createGraphics();
		g2d.drawImage( baseIcon, 0, 0, dummy );
		for( int j=1; j <= i; j++ )
		{
		    g2d.drawImage( grenadeIcon, j*10,10, dummy );
		}
		g2d.dispose();
		icons[ i ] = newIcon;
	    }
        }
    }
    
    public int getDamage()
    {
        int damage = 0;
        if( mode == GRENADE )
        {
            damage = (int)(Math.random()*6)+5;
        }
        else
        {
            damage = (int)(Math.random()*5)+2;
        }
        return damage;
    }

    public int getDamage( boolean critical )
    {
        int damage = 0;
        if( mode == GRENADE )
        {
            if( critical )
                damage = 10;
            else
                damage = getDamage();
        }
        else
        {
            if( critical )
                damage = 5;
            else
                damage = getDamage();
        }
        return damage;
    }

    public int getDamage( int range, boolean critical )
    {
        int damage = 0;
        if( mode == GRENADE )
        {
            damage = getDamage( critical );
        }
        else
        {
            if( critical )
                damage = 5;
            else
                damage = getDamage();
            if( range > 10 )
            {
                damage -= (range-10)/5;
            }
        }
        if( damage <= 0 )
            damage = 1;
        return damage;
    }

    public int getActionsToFire()
    {
        int cost = 2;
        if( mode == GRENADE )
        {
            if( grenades > 0 )
                cost = 3;
            else
                cost = -1;
        }
        return cost;
    }

    public int getCost()
    {
        return COST;
    }
        
    public int getMovementCost()
    {
        return 1;
    }
    
    public boolean hasAreaEffect()
    {
        if( mode == GRENADE )
            return true;
        return false;
    }

    public int getBlockingSize()
    {
        if( mode == GRENADE )
            return 80;
        return 50;
    }
    
    
    public String getName()
    {
        return "Assault Rifle";
    }

    public Point [] getArea( Game theGame, Point fireFrom, Point fireTo )
    {
        Point [] area;
        if( mode == GRENADE )
        {
            Vector temp = new Vector();
            Point check;
            Point check2;	// Neighbours
            Point check3;
            temp.add( new Point( fireTo ) );
            for( int i = 0; i < 8; i++ )
            {
                // If we hit on the clear, mark up all neighbours
                if( theGame.getMap().isViewable( fireTo.x, fireTo.y ) )
                {
                    check = theGame.getMap().getNeighbor( fireTo, i );
                    temp.add( new Point( check ) );
                }
                else
                {
                    // If we hit an object, it can block some neighbours
                    check = theGame.getMap().getNeighbor( fireTo, i );
                    if( theGame.getMap().isViewable( check.x, check.y ) )
                    {
                        check2 = theGame.getMap().getNeighbor( fireTo, (i+1)%8 );
                        check3 = theGame.getMap().getNeighbor( fireTo, (i+7)%8 );
                        if( theGame.getMap().hasLineOfSight( fireFrom, check ) || theGame.getMap().hasLineOfSight( fireFrom, check2 ) || theGame.getMap().hasLineOfSight( fireFrom, check3 ))
                            temp.add( new Point( check ) );
                    }
                }
            }
            area = new Point[ temp.size() ];
            for( int i=0; i < temp.size(); i++ )
            {
                area[i] = (Point)temp.elementAt(i);
            }
        }
        else
        {
            area = new Point[1];
            area[0]=fireTo;
        }
        grenades--;
        if( showGrenades )
            System.out.println( ((Marine)theGame.getActiveCharacter()).getName() + " has " + grenades + " grenades left" ); 
        return area;
    }
        
    public String [] getFiringModeNames()
    {
        return firingModes;
    }

    public int getNumberOfFiringModes()
    {
        return 2;
    }

    public int getFiringMode()
    {
        return mode;
    }

    public void setFiringMode( int mode )
    {
        this.mode = mode;
        //System.out.println( "Mode set to " + mode );
    }

    public int getRequiredAttribute()
    {
        if( mode == GRENADE )
            return Marine.HEAVY_WEAPONS;
        return Marine.RIFLE;
    }

    public void resetAmmunition()
    {
        grenades = 6;
    }
    
    public void setShowGrenades( boolean showGrenades )
    {
        this.showGrenades = showGrenades;
    }

    public Image getIcon()
    {
	if( mode == GRENADE && grenades >= 0 && grenades <= 6 )
	{
	    return icons[ grenades ];
	}
        return icons[ 0 ];
    }
}