// AssaultRifleSprite.java
// Started on February 25th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Image;
import java.util.Vector;
import java.awt.Point;
import java.awt.Graphics;
import java.applet.AudioClip;
import java.applet.Applet;
import javax.swing.RepaintManager;


public class AssaultRifleSprite extends WeaponSprite
{
    private static AudioClip shoot = null;
    private static AudioClip miss = null;
    private static AudioClip grenade = null;

    private static Image [] grenadeImages = null;
    private static Image [] grenadeStains = null;

    private Weapon weapon;
    private Point fireTo = new Point( 0, 0 );

    public AssaultRifleSprite()
    {
        if( shoot == null && GameMapPanel.playSounds )
        {
            shoot = ResourceManager.loadAudioClip( "Tsup.wav" );
            miss = ResourceManager.loadAudioClip( "miss.wav" );
            grenade = ResourceManager.loadAudioClip( "grenade.wav" );
        }
        if( grenadeImages == null )
        {
            grenadeImages = new Image[5];
            Vector temp = new Vector();
            ResourceManager.cropImage( "explosion.png", temp, 48*3 );
            for( int i=0; i < 5; i++ )
            {
                grenadeImages[i]= (Image)temp.elementAt(i);
            }
            grenadeStains = new Image[4];
            temp = new Vector();
            ResourceManager.cropImage( "stains.png", temp, 48 );
            grenadeStains[0] = (Image)temp.elementAt( 0 );
            grenadeStains[1] = (Image)temp.elementAt( 1 );
            grenadeStains[2] = (Image)temp.elementAt( 6 );
            grenadeStains[3] = (Image)temp.elementAt( 7 );
        }
    }

    public void setWeapon( Weapon weapon )
    {
        this.weapon = weapon;
    }

    
    public boolean affectsLights( )
    {
        return false;
    }

    public void prepareFrame( Point fireFrom, Point fireTo, int frame, GameMapPanel panel )
    {
        this.fireTo = fireTo;
        int cellSize = panel.getCellSize();
        if( frame == 4 )
        {
            ImageStain stain = new ImageStain( grenadeStains[ (int)(Math.random()*4) ] );
            stain.setLocation( fireTo.x, fireTo.y );
            stain.setBounds( fireTo.x*cellSize, fireTo.y*cellSize, cellSize, cellSize );
            panel.addStain( stain );
        }
        RepaintManager.currentManager( panel ).addDirtyRegion( panel, fireTo.x*cellSize-cellSize, fireTo.y*cellSize-cellSize, cellSize*3, cellSize*3 );
    }
    

    public void drawFrame( Graphics g, GameMapPanel host, int frame )
    {
        if( weapon.getFiringMode() == AssaultRifle.GRENADE && frame < 5)
        {
            //System.out.println( "Animating grenade blast at " + fireTo.x + ", " + fireTo.y );
            g.drawImage( grenadeImages[frame], (fireTo.x-1)*host.getCellSize(), (fireTo.y-1)*host.getCellSize(), host );
        }
    }

    public int getFrames()
    {
        if( weapon.getFiringMode() == AssaultRifle.GRENADE )
            return 6;
        return 0;
    }

    public AudioClip getFireSoundEffect()
    {
        if( weapon.getFiringMode() == AssaultRifle.GRENADE )
            return grenade;
        return shoot;
    }

    public AudioClip getMissSoundEffect()
    {
        if( weapon.getFiringMode() == AssaultRifle.GRENADE )
            return null;
        return miss;
    }

}