// BomberBrute.java
// Started on March 2nd, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class BomberBrute extends Monster implements Armed
{
    public static final int ROAMING = 0;
    public static final int SEEKING = 1;
    public static final int ATTACKING = 2;
    public static final int WAITING = 3;

    private Character target = null;
    private Point targetPoint = null;
    private boolean hasMoreActions = false;
    private int state = ROAMING;
    private Weapon weapon;
    private boolean blasted = false;
    private boolean leftCorpse = false;

    public BomberBrute()
    {
        location = new Point( 0, 0 );
        direction = Map.NORTH;
        alive = true;
        lastAction = new WaitAction( this, location );
        maximumActionPoints = 4;
        currentActionPoints = 4;
        maximumHitPoints = 8;
        currentHitPoints = 8;
        weapon = new AssaultRifle();
        weapon.setFiringMode( AssaultRifle.GRENADE );
    }

    public void setSpottedPlayer( Character character, boolean canSee )
    {
	super.setSpottedPlayer( character, canSee );
        if( canSee )
        {
            target = character;
            state = SEEKING;
            //System.out.println( "Brute spotted player: " + index );
        }
	else
	{
	    targetPoint = character.getLocation();
	    state = SEEKING;
	}
    }

    public void setState( int state )
    {
        this.state = state;
    }

    public Weapon getWeapon()
    {
        return weapon;
    }

    public void setWeapon( Weapon weapon )
    {}

    public GameAction playAction()
    {
        int x;
        int y;

        Point cell;
        Vector enemies = new Vector();
        Character check;

        hasMoreActions = true;

        if( alive )
        {
            for( int i=0; i < activeGame.getPlayerCharacters().size(); i++ )
            {
                check = activeGame.getPlayerCharacter( i );
                if( activeGame.getMap().hasLineOfSight( check.getLocation(), location ) )
                    enemies.add( check );
            }
            //System.out.println( "Ripper: doAction" );
            if( target != null )
                if( !target.isAlive() )
                {
                    target = null;
                    state = ROAMING;
                }
                else
                {
                    state = SEEKING;
                }
            if( currentActionPoints > 0 && state != ATTACKING )
            {
                // If next to an enemy, attack
                //System.out.println( "Brute checking neighbours" );

                for( int i=0; i < enemies.size(); i++ )
                {
                    check = (Character)enemies.elementAt(i);
                    if( check.isAlive() )
                    {
                        cell = check.getLocation();
                        if( activeGame.getMap().isNeighbor( location, cell ) )
                        {
                            if( currentActionPoints >= 1 )
                            {
                                // The bomber brute should almost always be able to blow himself up
                                // and die in the process, so we'll change his hit points
                                // and action points a little
                                direction = Map.getDirection( location, cell, direction );
                                currentHitPoints = 0;
                                blasted = true;
                                currentActionPoints = weapon.getActionsToFire();
                                lastAction = new ShootAction( this, weapon, location, true );
                                return lastAction;
                            }
                            else
                            {
                                lastAction = new WaitAction( this, new Point( location ) );
                                hasMoreActions = false;
                                return lastAction;
                            }
                        }
                        else
                        {
                            state = SEEKING;
                        }
                    }
                }
                if( target == null && enemies.size() > 0 )
                {
                    // Seek the closest enemy
                    state = SEEKING;
                    double range = Double.MAX_VALUE;
                    double checkRange;
                    for( int i = 0; i < enemies.size(); i++ )
                    {
                        check = (Character)enemies.elementAt(i);
                        checkRange = location.distance( check.getLocation() );
                        if( checkRange < range )
                        {
                            target = check;
                            range = checkRange;
                        }
                    }
                }
                if( state == SEEKING )
                {
                    //System.out.println( "Brute seeking targets" );
                    if( target != null || targetPoint != null )
                    {
			if( target != null )
			{
			    targetPoint = target.getLocation();
			}
                        GameAction action = SeekerAI.seek( this, targetPoint );
                        if( action != null )
                        {
                            lastAction = action;
                            return lastAction;
                        }
                        else
                        {
                            state = ROAMING;
                            target = null;
			    targetPoint = null;
                        }
                    }
                }
                if( state == ROAMING )
                {
                    // Roam around mindlessly
                    GameAction action = RoamerAI.roam( this, 0.5 );
                    lastAction = action;
                    return lastAction;
                }
            }
            else
            {
                hasMoreActions = false;
                lastAction = new WaitAction( this, new Point( location ) );
            }
        }
        return lastAction;
    }

        
    public GameAction getLastAction()
    {
        //System.out.println( "Get action from Brute " +index );
        return lastAction;
    }

    public void resetActionsPoints()
    {
        currentActionPoints = maximumActionPoints;
    }

    public int getWeaponSkill()
    {
        return 10;
    }

    public void addKill()
    {}
	
    public void registerHit()
    {}
	
    public boolean hasMoreActions()
    {
        if( currentActionPoints > 0 && hasMoreActions )
        {
            //System.out.println( "Brute has " + currentActionPoints + " action points left, attacking: " + attacking +", seeking: " + seeking );
            return true;
        }
        return false;
    }

    public boolean isAlive()
    {
        return alive;
    }
	
    public boolean leavesCorpse()
    {
	return false;
    }

    public boolean applyDamage( int damage )
    {
        currentHitPoints = currentHitPoints-damage;
        lastAction = new WoundAction( this, location );
        if( currentHitPoints <= 0 )
        {
            alive = false;
	    if( !blasted )
            {
		blasted = true;
                currentActionPoints = weapon.getActionsToFire();
		activeGame.handleRangedAttack( this, location );
            }
 	    if( !leftCorpse )
	    {
		activeGame.addCorpse( this );
		leftCorpse = true;
		lastAction = new DeadAction( this, new Point( location ) );
	    }
            maximumActionPoints = 0;
            currentActionPoints = 0;
        }
        return alive;
    }
	
    public int getMeleeAbility()
    {
	return 4;
    }

    public int getSize()
    {
        return 75;
    }
}