// BrainRobot.java
// Started on September 14th, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class BrainRobot extends Monster implements Robot
{
    public static final int ROAMING = 0;
    public static final int ESCAPING = 1;
    public static final int ATTACKING = 2;
    public static final int PANICKING = 3;

    private static int count = 0;
    
    private boolean hasMoreActions = false;
    private boolean rightHanded = true;
    private boolean alarmed = false;
    private boolean disabled = false;
    Character target = null;
    private Point goal = null;
    private int state = ROAMING;

    public BrainRobot()
    {
        location = new Point( 0, 0 );
        direction = Map.NORTH;
        alive = true;
        lastAction = new MoveAction( this, location, location );
        maximumActionPoints = 6;
        currentActionPoints = 6;
        maximumHitPoints = 7;
        currentHitPoints = 7;
        count++;
    }

    public void setSpottedPlayer( Character character, boolean canSee )
    {
	super.setSpottedPlayer( character, canSee );
        target = character;
        if( canSee )
        {
            alarmed = true;
	}
    }

    public void preAction()
    {
	Character [] visibleEnemies = activeGame.getVisibleEnemies( this );
	if( visibleEnemies.length > 0 )
	{
	    goal = SeekerAI.getSafePointForCharacter( this );
            state = ESCAPING;
	}
    }
    
    public GameAction playAction()
    {
        Point cell;

        Monster character;
        Character[] enemies;

        if( alive )
        {
            if( currentActionPoints > 0 )
            {
                enemies = activeGame.getVisibleEnemies( this );
                if( enemies.length > 0 )
                {
                    goal = activeGame.getRandomPointForMonster();
                    state = ESCAPING;                   
                }
                if( state == ESCAPING )
                {
		    if( goal == null )
		    {
			state = PANICKING;
		    }
		    else
		    {
			Vector path = activeGame.getMap().findShortestPath( this, goal, true );
			if( path.size() >= 2 )	
			{
			    cell = ((MapNode)path.elementAt( 1 )).getPoint();
			    direction = Map.getDirection( location, cell, direction );
			    lastAction = new MoveAction( this, new Point( location ), new Point( cell ) );
			    return lastAction;
			}
			else
			{
			    // If found goal, start roaming
			    if( activeGame.getMap().isNeighbor( goal, location ) )
			    {
				state = ROAMING;
				goal = null;
			    }
			    else
			    {
				// We are probably blocked
				state = ATTACKING;
			    }
			}
                    }
                }
                if( state == ATTACKING )
                {
                    // Check enemies
                    boolean hasEnemies = false;
                    for( int i=0; i < 8; i++ )
                    {
                        for( int j=0; j < activeGame.getPlayerCharacters().size(); j++ )
                        {
                            if( activeGame.getPlayerCharacter(j).isAlive() )
                            {
                                cell = activeGame.getPlayerCharacter(j).getLocation();
                                if( cell.equals( activeGame.getMap().getNeighbor( location, i ) ) )
                                {
                                    hasEnemies = true;
                                    if( currentActionPoints >= 2 )
                                    {
                                        direction = i;
                                        lastAction = new MeleeAction( this, activeGame.getPlayerCharacter(j) );
                                        return lastAction;
                                    }
                                    else
                                    {
                                        lastAction = new WaitAction( this, new Point( location ) );
                                        hasMoreActions = false;
                                        return lastAction;
                                    }
                                }
                            }
                        }
                        if( !hasEnemies )
                        {
                            state = ROAMING;
                        }
                    }
                }
		if( state == PANICKING )
		{
		    // Run away from the closest enemy
		    Character check;
		    Character target = this;
		    double range = Double.MAX_VALUE;
		    double checkRange;
		    int newDirection = -1;
		    int tempDirection = -1;
		    for( int i = 0; i < enemies.length; i++ )
		    {
			check = enemies[ i ];
			checkRange = location.distance( check.getLocation() );
			if( checkRange < range && check.isAlive() )
			{
			    target = check;
			    range = checkRange;
			}
		    }
		    tempDirection = Map.getDirection( location, target.getLocation(), direction );
		    // Check the opposite direction
		    newDirection = (tempDirection+4)%8;
		    if( activeGame.getMap().isPassable( location, newDirection ) )
		    {
			// This will do
			direction = newDirection;
		    }
		    else
		    {
			newDirection = (tempDirection+3)%8;
			if( activeGame.getMap().isPassable( location, newDirection ) )
			{
			    direction = newDirection;
			}
			else
			{
			    newDirection = (tempDirection+5)%8;
			    if( activeGame.getMap().isPassable( location, newDirection ) )
			    {
				direction = newDirection;
			    }
			    else
			    {
				newDirection = -1;
			    }
			}
		    }
		    // If found suitable escape direction, run there
		    if( newDirection > -1 )
		    {
			//System.out.println( "BrainRobot panicking from (" + location.x +","+location.y +") towards " +newDirection );
			lastAction = new MoveAction( this, new Point( location ), activeGame.getMap().getNeighbor( location, direction ));
			return lastAction;
		    }
		    else
		    {
			state = ROAMING;
		    }
		}
		
                if( state == ROAMING )
                {
                    // Move
                    if( activeGame.canCharacterMove( this, direction, false ) )
                    {
                        cell = activeGame.getMap().getNeighbor( location, direction );
                        lastAction = new MoveAction( this, new Point( location ), new Point( cell ) );
                        return lastAction;
                    }
                    direction = (int)(Math.random()*8);                    
                }
            }
            lastAction = new WaitAction( this, new Point( location ) );
        }
        return lastAction;
    }
    
    public void postAction()
    {
	Character [] visibleEnemies = activeGame.getVisibleEnemies( this );

	if( visibleEnemies.length < 1 )
	{
	    state = ROAMING;
	}
    }

    public GameAction getLastAction()
    {
        return lastAction;
    }
    
    public int getMeleeAbility()
    {
	return 4;
    }
    
    public int getMeleeCost()
    {
	return 2;
    }

    public void resetActionsPoints()
    {
        currentActionPoints = maximumActionPoints;
    }

    public boolean hasMoreActions()
    {
        if( currentActionPoints > 0 )
        {
            return true;
        }
        return false;
    }

    public boolean applyDamage( int damage )
    {
        damage = damage-1;
        if( damage < 0 )
        {
            damage = 0;
        }
        currentHitPoints = currentHitPoints-damage;
        lastAction = new WoundAction( this, location );
        if( currentHitPoints <= 0 )
        {
            alive = false;
            lastAction = new DeadAction( this, new Point( location ) );
            maximumActionPoints = 0;
            currentActionPoints = 0;
            count--;
            if( count <= 0 )
            {
                // If all the brainRobots are dead, disable all other robots
                Character monster;
                boolean wasAlive = false;
                for( int i=0; i < activeGame.getMonsters().size(); i++ )
                {
                    monster = activeGame.getMonster( i );
                    if( (monster instanceof Robot) && (monster != this) )
                    {
                        wasAlive = monster.isAlive();
                        ((Robot)monster).setDisabled( true );
                        if( wasAlive )
                        {
                            activeGame.addCorpse( monster );
                        }
                    }
                }
                activeGame.setMapChanged();
            }
        }
        return alive;
    }

    public void setDisabled( boolean disabled )
    {
        this.disabled = disabled;
        if( disabled )
        {
            alive = false;
            lastAction = new DeadAction( this, new Point( location ) );
        }
    }

    public boolean getDisabled()
    {
        return disabled;
    }
    
    public int getSize()
    {
        return 75;
    }

}