// Brute.java
// Started on December 30th, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class Brute extends Monster
{
    public static final int ROAMING = 0;
    public static final int SEEKING = 1;
    public static final int ATTACKING = 2;

    private Character target = null;
    private Point targetPoint = null;
    private boolean hasHit = false;
    private boolean hasMoreActions = false;
    private int state;

    public Brute()
    {
	location = new Point( 0, 0 );
	direction = Map.NORTH;
	alive = true;
	lastAction = new WaitAction( this, location );
	maximumActionPoints = 4;
	currentActionPoints = 4;
	maximumHitPoints = 8;
	currentHitPoints = 8;
    }
 
    public void setSpottedPlayer( Character character, boolean canSee )
    {
	super.setSpottedPlayer( character, canSee );
        if( canSee )
        {
            target = character;
            state = SEEKING;
            //System.out.println( "Brute spotted player: " + index );
        }
	else
	{
	    targetPoint = character.getLocation();
	    state = SEEKING;
	}
    }


    public GameAction playAction()
    {
        int x;
        int y;

        Point cell;
        Character [] enemies;
        Character check;

        hasMoreActions = true;

        if( alive )
        {
            enemies = activeGame.getVisibleEnemies( this );
            //System.out.println( "Ripper: doAction" );
            if( target != null )
            {
                if( !target.isAlive() )
                {
                    target = null;
                    state = ROAMING;
                }
                else
                {
                    state = SEEKING;
                }
            }
            if( currentActionPoints > 0 )
            {
                // If next to an enemy, attack
                //System.out.println( "Brute checking neighbours" );

                for( int i=0; i < enemies.length; i++ )
                {
                    check = enemies[ i ];
                    if( check.isAlive() )
                    {
                        cell = check.getLocation();
                        if( activeGame.getMap().isNeighbor( location, cell ) )
                        {
                            if( currentActionPoints >= getMeleeCost() && !hasHit )
                            {
                                direction = Map.getDirection( location, cell, direction );
                                //System.out.println( "Brute attacking " + check.toString() );
                                target = check;
                                lastAction = new MeleeAction( this, check );
                                state = ATTACKING;
                                hasHit = true;
                                return lastAction;
                            }
                            else
                            {
                                lastAction = new WaitAction( this, new Point( location ) );
                                hasMoreActions = false;
                                return lastAction;
                            }
                        }
                        else
                        {
                            state = SEEKING;
                        }
                    }
                }
                if( target == null && enemies.length > 0 )
                {
                    // Select an enemy and start seeking
                    state = SEEKING;
                    int newTarget = (int)(Math.random()*enemies.length);
                    target = enemies[ newTarget ];
                    /*
                    double range = Double.MAX_VALUE;
                    double checkRange;
                    for( int i = 0; i < enemies.length; i++ )
                    {
                        check = enemies[ i ];
                        checkRange = location.distance( check.getLocation() );
                        if( checkRange < range )
                        {
                            target = check;
                            range = checkRange;
                        }
                    }
                     */
                }
                if( state == SEEKING )
                {
                    //System.out.println( "Brute seeking targets" );
                    if( target != null || targetPoint != null )
                    {
			if( target != null )
			{
			    targetPoint = target.getLocation();
			}
                        //System.out.println(" Brute seeking: " + target.toString() );
                        GameAction action = SeekerAI.seek( this, targetPoint );
                        if( action != null )
                        {
                            lastAction = action;
                            return lastAction;
                        }
                        else
                        {
                            state = ROAMING;
                            target = null;
			    targetPoint = null;
                        }
                    }
                }
                if( state == ROAMING )
                {
                    GameAction action = RoamerAI.roam( this, 0.5 );
                    lastAction = action;
                    return lastAction;
                }
            }
            else
            {
                hasMoreActions = false;
                lastAction = new WaitAction( this, new Point( location ) );
            }
        }
        return lastAction;
    }
        
    public GameAction getLastAction()
    {
	//System.out.println( "Get action from Brute " +index );
	return lastAction;
    }
        
    public int getMeleeAbility()
    {
	return 6;
    }
    
    public int getMeleeCost()
    {
	return 2;
    }
    
    public void resetActionPoints()
    {
	currentActionPoints = maximumActionPoints;
        hasHit = false;
    }
    
    public boolean hasMoreActions()
    {
	if( currentActionPoints > 0 && hasMoreActions )
	{
	    //System.out.println( "Brute has " + currentActionPoints + " action points left, attacking: " + attacking +", seeking: " + seeking );
	    return true;
	}
	return false;   
    }

    public boolean applyDamage( int damage )
    {
	currentHitPoints = currentHitPoints-damage;
	lastAction = new WoundAction( this, location );
	if( currentHitPoints <= 0 )
	{
	    alive = false;
	    lastAction = new DeadAction( this, new Point( location ) );
	    maximumActionPoints = 0;
	    currentActionPoints = 0;
	}
	return alive;
    }

    public int getSize()
    {
	return 75;
    }
}