// Centipedoidid.java
// Started on August 24th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;

public class Centipedoid extends Monster
{
    public static final int HEAD = 0;
    public static final int SEGMENT = 1;
    public static final int TAIL = 2;
    public static final String [] types = { "Head", "Segment", "Tail" };
    
    private int type = -1;
    private Centipedoid nextPart = null;
    private Centipedoid previousPart = null;
    private Point previousLocation = null;
    
    public Centipedoid()
    {
	location = new Point( 0, 0 );
	previousLocation = new Point( location );
	direction = Map.NORTH;
	alive = true;
	lastAction = new WaitAction( this, location );
	maximumActionPoints = 3;
	currentActionPoints = 3;
	maximumHitPoints = 20;
	currentHitPoints = 20;
    }
    
    public GameAction doAction( GameAction action )
    {
	super.doAction( action );
	lastAction = action;
	if( action instanceof MoveAction && type == HEAD )
	{
	    if( location.equals( previousLocation ) )
	    {
		lastAction = new WaitAction( this, location );
	    }
	    else
	    {
		previousPart.moveTo( new Point( ((MoveAction)action).getStartPoint() ) );
	    }
	}
	return lastAction;
    }
    
    public void setSegment( int type, Centipedoid nextPart, Centipedoid previousPart )
    {
	this.type = type;
	this.nextPart = nextPart;
	this.previousPart = previousPart;
    }
    
    public int getSegmentType()
    {
	return type;
    }
    
    private void die()
    {
	alive = false;
	activeGame.addCorpse( this );
	if( previousPart != null )
	{
	    previousPart.die();
	}
    }

    private void moveTo( Point newLocation )
    {
	Point previousLocation = new Point( location );
	lastAction = new MoveAction( this, previousLocation, newLocation );
	System.out.println( types[type] + " moving from " + Objective.pointToString( previousLocation ) + " to " + Objective.pointToString( newLocation ) );

	if( previousPart != null )
	{
	    previousPart.moveTo( previousLocation );
	}
    }
    
    public GameAction playAction()
    {
	if( alive && currentActionPoints > 0 && type == HEAD )
	{
	    lastAction = RoamerAI.roam( this, 0.5 );
	}
	return lastAction;
    }
    
    public void postAction()
    {
	System.out.println( types[type] + " located in " + Objective.pointToString( location ) );
	lastAction = new WaitAction( this, location );
    }
    
    public boolean applyDamage( int damage )
    {
	if( damage < 0 )
	{
	    return true;
	}
	if( type == HEAD )
	{
	    currentHitPoints = currentHitPoints-damage;
	    lastAction = new WoundAction( this, location );
	    if( currentHitPoints <= 0 )
	    {
		alive = false;
		lastAction = new DeadAction( this, new Point( location ) );
		activeGame.addCorpse( this );
		previousPart.die();
		System.out.println( "Centipedoid killed!" );
	    }
	}
	else
	{
	    alive = nextPart.applyDamage( damage-1 );
	}
	return alive;
    }
    
    public int getMeleeAbility()
    {
	if( type == HEAD )
	{
	    return 8;
	}
	return -1;
    }
    
    public boolean hasMoreActions()
    {
	if( currentActionPoints > 0 )
	{
	    return true;
	}
	return false;   
    }
    
    public int getSize()
    {
	return 45;
    }
    
    public boolean isAlive()
    {
	return alive;
    }
    
    public boolean leavesCorpse()
    {
	return false;
    }
    
}