// CharacterSprite.java
// Started on June 20th, 2001
// Copyright (c) Virtu Halttunen
// June 29th, 2001: Changed into CharacterSprite

package TheGame;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Image;

/**
 * This class is the abstract base for all character sprites in the game.
 * The sprites all belong to the view object (usually a GameMapPanel),
 * and they have a reference to the Character object they represent.
 *
 * @author	Virtu Halttunen
 * @version	0.1
 */
public abstract class CharacterSprite extends Sprite
{    
    protected int direction;
    protected Character character;
    protected boolean visible = false;
    protected GameAction lastAction;
    protected int frame = 0;
    
    public abstract Image getSprite( GameAction action );
    
    public Image getSprite()
    {
	return getSprite( new DeadAction( character, character.getLocation() ) );
    }
    
    public void setCharacter( Character character )
    {
	this.character = character;
    }
    
    public Character getCharacter()
    {
	return character;
    }
    
    public GameAction getAction()
    {
	return lastAction;
    }
    
    public void setAction( GameAction action )
    {
	lastAction = action;
    }
    
    public boolean isVisible()
    {
        return visible;
    }
    
    public void setFrame( int frame )
    {
	this.frame = frame;
    }
    
    public void setVisible( boolean visible )
    {
        this.visible = visible;
    }
            
    public int getDirection()
    {
	return direction;
    }
    
    public void setDirection( int direction )
    {
	this.direction = direction;
    }
    
    public int getNumberOfWalkFrames()
    {
	return 10;
    }
    
    public int getNumberOfMeleeAttackFrames()
    {
	return 10;
    }

    public int getNumberOfRangedAttackFrames()
    {
	return 1;
    }
}