// CloneVatObjective.java
// Started on July 2nd, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;

/**
 *
 *  Obsoleted by DestroySceneryObjective. Do not use
 *
 */
public class CloneVatObjective extends Objective
{
    protected Game activeGame;
    private Point location = new Point(-1,-1);
    private Sprite topSprite;
    private Sprite bottomSprite;
    private boolean destroyed = false;
    private boolean mustDestroy = false;
    private int hitPoints = 5;

    private String [] propertyNames = { "Location", "HitPoints", "MustDestroy" };
 
    public CloneVatObjective()
    {
         topSprite = new CloneVatSprite( this, true );
         bottomSprite = new CloneVatSprite( this, false );
    }
 
    public void setGame( Game game )
    {
	activeGame = game;
        if( location.x < 0 )
        {
            location = activeGame.getRandomPointForMonster();
        }
    }
          
    public int getNumberOfSprites()
    {
 	return 2;
    }
    
    public Sprite getSprite( int x, int y )
    {
        if( x==location.x && y==location.y )
        {
            return topSprite;
        }
        if( x==location.x && y==(location.y+1) )
        {
            return bottomSprite;
        }
        return null;
    }

    public String [] getPropertyNames()
    {
        return propertyNames;
    }

    public String getProperty( String key )
    {
        if( key.equals( "Location" ) )
        {
            return Objective.pointToString( getLocation() );
        }
        if( key.equals( "MustDestroy" ) )
        {
            if( mustDestroy )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }
        if( key.equals( "HitPoints" ) )
        {
            return String.valueOf( hitPoints );
        }
        return null;
    }
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( "Location" ) )
        {
            Point newLocation = Objective.pointFromString( value );
            setLocation( newLocation ); 
        }
        if( key.equals( "MustDestroy" ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                mustDestroy = true;
            }
            else
            {
                mustDestroy = false;
            }
        }
        if( key.equals( "HitPoints" ) )
        {
            hitPoints = Objective.intFromString( value );
        }
    }

    public void setLocation( Point point )
    {
        location = point;
    }
    
    public Point getLocation()
    {
        return location;
    }
    
    public void setLocation( int x, int y )
    {
	setLocation( new Point( x, y ) );
    }
    
    public boolean showCompletionMessageImmediately()
    {
	return true;
    }
    
    public boolean isCompleted()
    {
        if( mustDestroy && !destroyed )
            return false;
        return true;
    }
    
    public int getSize()
    {
         return 70;
    }
    
    public boolean isLocatedIn( Point point )
    {
        if( location.x == point.x && ( location.y == point.y || location.y == (point.y-1) ) )
            return true;
	return false;
    }
 
    public boolean isLocatedIn( int x, int y )
    {
        if( location.x == x && ( location.y == y || location.y == (y-1) ) )
            return true;
	return false;
    }
   
    public boolean isDestroyed()
    {
        return destroyed;
    }
    
    public boolean isDestructible()
    {
        if( destroyed )
            return false;
	return true;
    }
    
    public boolean isShownOnMap()
    {
        if( destroyed )
            return false;
        return true;
    }
    
    public boolean handleDamage( Point point, int damage )
    {
        if( isLocatedIn( point ) )
        {
            hitPoints -= damage;
            if( hitPoints <= 0 )
            {
                destroyed = true;
                return true;
            }
        }
	return false;
    }

    public boolean handleAction( GameAction action )
    {
        return false;
    }

}