// CollectTribblesObjective.java
// Started on August 4th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import java.io.*;
import java.util.*;

/**
 * This Objectives adds to the game an object that
 * must be recovered. Default object is a randomly
 * placed cybernetic brain. The object counts as
 * recovered when one of the player characters is
 * carrying it.
 */
public class CollectTribblesObjective extends Objective
{
    public static final String IS_SHOWN_ON_MAP_KEY = "Show on map";
    public static final String MUST_COLLECT_KEY = "Must collect";
    public static final String TRIBBLES_KEY = "Tribbles";
    
    private static Point SAFE_POINT = new Point( -10, -10 );
    
    private boolean visibleOnMap = false;
    private int mustCollect = 5;
    private int totalTribbles = 10;
    private int collected = 0;
    
    private Vector tribbles = null;
    private HashMap collectors = null;

    private String [] propertyNames = { IS_SHOWN_ON_MAP_KEY, TRIBBLES_KEY, MUST_COLLECT_KEY };
 
    public CollectTribblesObjective()
    {
	tribbles = new Vector();
	collectors = new HashMap();
    }
 
    public void setGame( Game game )
    {
	activeGame = game;
	for( int i=0; i < totalTribbles; i++ )
	{
	    Tribble tribble = new Tribble();
	    tribbles.add( tribble );
	    tribble.setLocation( activeGame.getRandomPointForMonster() );
	    activeGame.addMonster( tribble );
	}
    }
          
    public String [] getPropertyNames()
    {
        return propertyNames;
    }

    public String getProperty( String key )
    {
        if( key.equals( TRIBBLES_KEY ) )
        {
	    return String.valueOf( totalTribbles );
        }
        if( key.equals( MUST_COLLECT_KEY ) )
        {
            return String.valueOf( mustCollect );
        }
	if( key.equals( IS_SHOWN_ON_MAP_KEY ) )
        {
            return String.valueOf( visibleOnMap );
	}	
        return null;
    }
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( TRIBBLES_KEY ) )
        {
	    totalTribbles = Objective.intFromString( value );
	}
        if( key.equals( MUST_COLLECT_KEY ) )
        {
	    mustCollect = Objective.intFromString( value );
        }
	if( key.equals( IS_SHOWN_ON_MAP_KEY ) )
        {
            visibleOnMap = new Boolean( value ).booleanValue();
        }
    }
    
    public String getDescriptionClip()
    {
        return String.valueOf( collected );
    }
        
    public boolean isCompleted()
    {
	collected = 0;
	// Run through the collectors
	Set keys = collectors.keySet();
	Object [] carriers = keys.toArray();
	
	for( int i=0; i < carriers.length; i++ )
	{
	    Character carrier = (Character)carriers[i];
	    if( !carrier.isAlive() )
	    {
		// Release the Tribbles
		int number = ((Integer)collectors.get( carrier ) ).intValue();
		collectors.remove( carrier );
		int released = 0;
		for( int t=0; t < tribbles.size() && released < number; t++ )
		{
		    Tribble tribble = (Tribble)tribbles.elementAt( t );
		    if( tribble.getLocation().equals( SAFE_POINT ) )
		    {
			tribble.setLocation( carrier.getLocation() );
			tribble.setDormant( false );
			released++;
			//System.out.println( "Releasing tribbles from " + carrier.getType() );
		    }
		}
	    }
	    else
	    {
		if( activeGame.isPlayerCharacter( carrier ) )
		{
		    int number = ((Integer)collectors.get( carrier ) ).intValue();
		    collected += number;
		    //System.out.println( "Tribbles collected: " + collected );
		}
	    }
	}
	
	return ( collected >= mustCollect );
    }
    
    public boolean isSuccessful()
    {
	collected = 0;
	// Run through the collectors
	Set keys = collectors.keySet();
	Object [] carriers = keys.toArray();
	
	for( int i=0; i < carriers.length; i++ )
	{
	    Character carrier = (Character)carriers[i];
	    if( activeGame.isPlayerCharacter( carrier ) && carrier.isAlive() )
	    {
		int number = ((Integer)collectors.get( carrier ) ).intValue();
		collected += number;
		//System.out.println( "Tribbles collected: " + collected );
	    }
	}
	return ( collected >= mustCollect );
    }
    
    public boolean handleDamage( Point point, int damage )
    {
	return false;
    }
    
    private void pickUp( Character character, Tribble tribble )
    {
	// Pick the critter up
	tribble.setLocation( SAFE_POINT );
	tribble.setPickedUp( true );
	
	
	// Check whether the character is already carrying tribbles
	int carries = 1;
	
	if( collectors.containsKey( character ) )
	{
	    carries = 1+((Integer)collectors.get( character )).intValue();
	}
	collectors.put( character, new Integer( carries ) );	
	//System.out.println( character.getType() + " picked up a Tribble" );
    }
    
    public boolean handleAction( GameAction action )
    {
	// Run through the collectors
	Set keys = collectors.keySet();
	Object [] carriers = keys.toArray();
	collected = 0;
	int captured = 0;
	
	for( int i=0; i < carriers.length; i++ )
	{
	    Character carrier = (Character)carriers[i];
	    if( !carrier.isAlive() )
	    {
		// Release the Tribbles
		int number = ((Integer)collectors.get( carrier ) ).intValue();
		collectors.remove( carrier );
		int released = 0;
		for( int t=0; t < tribbles.size() && released < number; t++ )
		{
		    Tribble tribble = (Tribble)tribbles.elementAt( t );
		    if( tribble.getLocation().equals( SAFE_POINT ) )
		    {
			tribble.setLocation( carrier.getLocation() );
			tribble.setPickedUp( false );
			released++;
			//System.out.println( "Releasing tribbles from " + carrier.getType() );
		    }
		}
	    }
	    else
	    {
		if( activeGame.isPlayerCharacter( carrier ) )
		{
		    int number = ((Integer)collectors.get( carrier ) ).intValue();
		    collected += number;
		    //System.out.println( "Tribbles collected: " + collected );
		}
	    }
	}
	
        Character character = action.getActor();
	if( character == null )
	{
	    return false;
	}
	boolean handled = false;
	if( character.getSize() >= 60 && !character.getType().equals( "Bob" ) )
	{
	    if( action instanceof MoveAction )
	    {
		Point point = ((MoveAction)action).getEndPoint();
		for( int i=0; i < tribbles.size(); i++ )
		{
		    Tribble tribble = (Tribble)tribbles.elementAt( i );
		    if( tribble.isAlive() && tribble.getLocation().equals( point ) )
		    {
			pickUp( character, tribble );	  
			if( activeGame.isPlayerCharacter( character ) )
			{
			    captured++;
			}
		    }
		}
	    }
	    if( action instanceof UseAction )
            {
                Point point = ((UseAction)action).getPoint();
		if( character.getCurrentActionPoints() < 1 )
		{
		    //activeGame.handleOutOfAP( character );
		    return false;
		}
                for( int i=0; i < tribbles.size(); i++ )
		{
		    Tribble tribble = (Tribble)tribbles.elementAt( i );
		    if( tribble.isAlive() && tribble.getLocation().equals( point ) )
		    {
			pickUp( character, tribble );
			((UseAction)action).setCost( 1 );
			((UseAction)action).setConsumed( true );
			 handled = true;
			 if( activeGame.isPlayerCharacter( character ) )
			 {
			     captured++;
			 }
		    }
		}
            }
        }
	if( captured > 0 )
	{
	    String message = captured + " critter";
	    if( captured > 1 )
	    {
		message+="s";
	    }
	    message += " captured, total " + (collected+captured);
	    activeGame.handleMessage( new Message( character, message ) );
	}
        return handled;
    }
    
    public boolean isLocatedIn( Point point )
    {
	for( int i=0; i < tribbles.size(); i++ )
	{
	    Tribble tribble = (Tribble)tribbles.elementAt( i );
	    if( tribble.getLocation().equals( point ) )
	    {
		return true;
	    }
	}
	return false;
    }
    
    public boolean isLocatedIn( int x, int y )
    {
	return isLocatedIn( new Point( x, y ) );
    }
    
    public boolean isDestructible()
    {
	return false;
    }
    
    public boolean isShownOnMap()
    {
        return visibleOnMap;
    }
        
    public boolean isUsable()
    {
	return true;
    }

}