// CreatorMarine.java
// Started on August 2nd, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.io.*;

public class CreatorMarine extends Marine
{
    private int freeLevels = 20;

    public CreatorMarine()
    {
        name = "";
        helmetIndex = -1;
        extrasIndex = -1;
        location = new Point( 0, 0 );
        direction = Map.NORTH;
        alive = true;
        maximumActionPoints = 6;
        currentActionPoints = 6;
        maximumHitPoints = 10;
        currentHitPoints = 10;
        lastAction = new MoveAction( this, location, location );
        weapon = new AssaultRifle();
    }

    public void cycleHelmet( int direction )
    {
        if( direction > 0 )
        {
            helmetIndex++;
            if( helmetIndex >= MarineStatisticsPanel.HELMETS )
            {
                helmetIndex = -1;
            }
        }
        else
        {
	    helmetIndex--;
	    if( helmetIndex < -1 )
	    {
		helmetIndex = MarineStatisticsPanel.HELMETS-1;
	    }
        }
    }
    
    public void cycleHair( int direction )
    {
        if( direction > 0 )
        {
            hairIndex = (hairIndex+1)%MarineStatisticsPanel.HAIRS;
        }
        else
        {
            hairIndex = (hairIndex+MarineStatisticsPanel.HAIRS-1)%MarineStatisticsPanel.HAIRS;
        }
    }

    public void cycleBrows( int direction )
    {
        if( direction > 0 )
        {
            browsIndex++;
            if( browsIndex >= MarineStatisticsPanel.BROWS )
            {
                browsIndex = -1;
            }
        }
        else
        {
	    browsIndex--;
	    if( browsIndex < -1 )
	    {
		browsIndex = MarineStatisticsPanel.BROWS-1;
	    }
        }
    }

    public void cycleExtras( int direction )
    {
        if( direction > 0 )
        {
            extrasIndex++;
            if( extrasIndex >= MarineStatisticsPanel.EXTRAS )
            {
                extrasIndex = -1;
            }
        }
        else
        {
	    extrasIndex--;
	    if( extrasIndex < -1 )
	    {
		extrasIndex = MarineStatisticsPanel.EXTRAS-1;
	    }
        }
    }

    public void cycleEyes( int direction )
    {
        if( direction > 0 )
        {
            eyeIndex = (eyeIndex+1)%MarineStatisticsPanel.EYES;
        }
        else
        {
            eyeIndex = (eyeIndex+MarineStatisticsPanel.EYES-1)%MarineStatisticsPanel.EYES;
        }
    }

    public void cycleEars( int direction )
    {
        if( direction > 0 )
        {
            earsIndex = (earsIndex+1)%MarineStatisticsPanel.EARS;
        }
        else
        {
            earsIndex = (earsIndex+MarineStatisticsPanel.EARS-1)%MarineStatisticsPanel.EARS;
        }
    }

    public void cycleNose( int direction )
    {
        if( direction > 0 )
        {
            noseIndex = (noseIndex+1)%MarineStatisticsPanel.NOSES;
        }
        else
        {
            noseIndex = (noseIndex+MarineStatisticsPanel.NOSES-1)%MarineStatisticsPanel.NOSES;
        }
    }
    
    public void cycleMouth( int direction )
    {
        if( direction > 0 )
        {
            mouthIndex = (mouthIndex+1)%MarineStatisticsPanel.MOUTHS;
        }
        else
        {
            mouthIndex = (mouthIndex+MarineStatisticsPanel.MOUTHS-1)%MarineStatisticsPanel.MOUTHS;
        }
    }

    public void cycleHead( int direction )
    {
        if( direction > 0 )
        {
            headIndex = (headIndex+1)%MarineStatisticsPanel.HEADS;
        }
        else
        {
            headIndex = (headIndex+MarineStatisticsPanel.HEADS-1)%MarineStatisticsPanel.HEADS;
        }
    }

    public boolean assignPointsToAttribute( int attribute )
    {
        if( freeLevels > 0 && attributeLevels[ attribute ] < 7 )
        {
            attributeLevels[ attribute ]++;
            freeLevels--;
            return true;
        }
        return false;
    }

    public boolean removePointsFromAttribute( int attribute )
    {
        if( attributeLevels[ attribute ] > 0 )
        {
            attributeLevels[ attribute ]--;
            freeLevels++;
            return true;
        }
        return false;
    }

    public int getFreeAttributeLevels()
    {
        return freeLevels;
    }
    
    public String getClassName()
    {
	return "Marine";
    }

    /**
     *  This method converts a CreatorMarine to normal Marine, so it can be used
     *  in the Game.
     *
     *  @param cMarine the CreatorMarine to be converted
     *  @return the converted Marine
     */
    public static Marine OLDconvert( CreatorMarine cMarine )
    {
        DataInputStream dis;
        DataOutputStream dos;
        PipedInputStream pipeIn;
        PipedOutputStream pipeOut;
        Marine marine = null;
        try
        {
            pipeOut = new PipedOutputStream();
            dos = new DataOutputStream( pipeOut );
            pipeIn = new PipedInputStream( pipeOut );	// Yes, using piped streams like this is dangerous
            dis = new DataInputStream( pipeIn );

            cMarine.save( dos );
            dos.flush();
            marine = Marine.load( dis );
            dis.close();
            dos.close();
            pipeIn.close();
            pipeOut.close();
        }
        catch( Exception e )
        {
            System.out.println( "Error converting CreatorMarine to Marine!" );
            e.printStackTrace();
        }
        return marine;
    }
    
    public static Marine createRandomGrunt()
    {
        CreatorMarine marine = new CreatorMarine();
        marine.randomizeAppearance();
        marine.randomizeAttributesForGrunt();
        marine.verifyAttributes();
        //return convert( marine );
	return (Marine)marine;
    }

    public static Marine createRandomSergeant()
    {
        CreatorMarine marine = new CreatorMarine();
        marine.randomizeAppearance();
        marine.randomizeAttributesForSergeant();
        marine.verifyAttributes();

        //return convert( marine );
	return (Marine)marine;
    }
    
    
    public void randomizeAppearance()
    {
        name = defaultNames[ (int)(Math.random()*defaultNames.length) ];
        headIndex = (int)(Math.random()*MarineStatisticsPanel.HEADS);
        earsIndex = (int)(Math.random()*MarineStatisticsPanel.EARS);
        noseIndex = (int)(Math.random()*MarineStatisticsPanel.NOSES);
        mouthIndex = (int)(Math.random()*MarineStatisticsPanel.MOUTHS);
        eyeIndex = (int)(Math.random()*MarineStatisticsPanel.EYES);
        hairIndex = (int)(Math.random()*MarineStatisticsPanel.HAIRS);
        if( Math.random() > 0.5 )
        {
            extrasIndex = (int)(Math.random()*MarineStatisticsPanel.EXTRAS);
        }
        else
        {
            extrasIndex = -1;
        }
        if( Math.random() > 0.65 )
        {
            browsIndex = (int)(Math.random()*MarineStatisticsPanel.BROWS);
        }
        else
        {
            browsIndex = -1;
        }
        if( Math.random() > 0.65 )
        {
            helmetIndex = (int)(Math.random()*MarineStatisticsPanel.HELMETS);
        }
        else
        {
            helmetIndex = -1;
        }
        mouthYOffset = (int)(Math.random()*3)-1;
        mouthXOffset = (int)(Math.random()*3)-1;
        eyeOffset = (int)(Math.random()*3)-1;
        browsOffset = (int)(Math.random()*3)-1;
    }

    public void randomizeAttributesForGrunt()
    {
        for( int i=0; i < ATTRIBUTES; i++ )
        {
            attributeLevels[i]=0;
            attributePoints[i]=0;
        }
        rank = 0;
        int points = 20;
        int attr;
        do
        {
            attr = Math.abs( (int)(Math.random()*ATTRIBUTES) );
            if( attributeLevels[ attr ] < 6 )
            {
                attributeLevels[attr]++;
                points--;
            }
        } while ( points > 0 );
        if( attributeLevels[ LEADERSHIP ] >= 2 )
            rank++;
        if( attributeLevels[ BASIC_MILITARY_TRAINING ] >= 2 )
            rank++;
        if( attributeLevels[ LUCK ] >= 2 )
            rank++;
        if( attributeLevels[ MORALE ] >= 2 )
            rank++;
        verifyAttributes();
    }

    public void randomizeAttributesForSergeant()
    {
        for( int i=0; i < ATTRIBUTES; i++ )
        {
            attributeLevels[i]=0;
            attributePoints[i]=0;
        }
        rank = 6;
        attributeLevels[ LEADERSHIP ] = 5;
        attributeLevels[ BASIC_MILITARY_TRAINING ] = 3;
        attributeLevels[ MORALE ] = 4;
        attributeLevels[ RIFLE ] = 2;
        attributeLevels[ HANDGUN ] = 2;
        int points = 10;
        int attr;
        do
        {
            attr = Math.abs( (int)(Math.random()*ATTRIBUTES) );
            if( attributeLevels[ attr ] < 6 )
            {
                attributeLevels[attr]++;
                points--;
            }
        } while ( points > 0 );
        verifyAttributes();
    }

    
    
}
