// ShotgunBrute.java
// Started on June 4th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class CyberBrute extends Monster implements Armed
{
    public static final int ROAMING = 0;
    public static final int SEEKING = 1;
    public static final int ATTACKING = 2;
    public static final int SHOOTING = 3;
    public static final int WAITING = 4;

    public static final int ROCKET = 0;
    public static final int LOSING_ROCKET = 1;
    public static final int PISTOL = 2;
    public static final int FINAL_DEATH = 3;

    private Character target = null;
    private boolean hasMoreActions = false;
    private int state = ROAMING;
    private int healthState = ROCKET;
    private Weapon weapon;
    private int skill = 0;

    public CyberBrute()
    {
        location = new Point( 0, 0 );
        direction = Map.NORTH;
        alive = true;
        lastAction = new WaitAction( this, location );
        maximumActionPoints = 5;
        currentActionPoints = 5;
        maximumHitPoints = 25;
        currentHitPoints = 25;
        if( Math.random() < 0.125 )
        {
            // Elite !
            maximumActionPoints = 5;
            currentActionPoints = 5;
            maximumHitPoints = 30;
            currentHitPoints = 30;
            skill = 4;
        }
        weapon = new AssaultRifle();
        weapon.setFiringMode( AssaultRifle.GRENADE );
    }

    public GameAction doAction( GameAction action )
    {
        if( action.getActor() != this )
        {
            System.out.println( "Illegal action passed to " + getClass().getName() );
            return null;
        }
        if( action instanceof MoveAction )
        {
            //System.out.println( getClass().getName() + " walking from " + location.toString() + " to " + ((MoveAction)action).getEndPoint().toString() );
            direction = Map.getDirection( new Point( location ), new Point( ((MoveAction)action).getEndPoint() ), direction );
            //System.out.println( "Direction: " + direction );
            location = ((MoveAction)action).getEndPoint();
            currentActionPoints-=((MoveAction)action).getCost();
        }
        if( action instanceof ShootAction )
        {
	    // We don't want to change direction when blowing up
	    if( healthState != LOSING_ROCKET && healthState != FINAL_DEATH )
	    {
		direction = Map.getDirection( new Point( location ), new Point( ((ShootAction)action).getEndPoint() ), direction );
	    }
            currentActionPoints-=((ShootAction)action).getWeapon().getActionsToFire();
        }
        if( action instanceof UseAction )
        {
            direction = Map.getDirection( new Point( location ), new Point( ((UseAction)action).getPoint() ), direction );
            currentActionPoints-= ((UseAction)action).getCost();
        }
	if( action instanceof MeleeAction )
	{
	    MeleeAction mAction = (MeleeAction)action;
	    direction = Map.getDirection( mAction.getAttacker().getLocation(), mAction.getDefender().getLocation(), direction );
	    currentActionPoints-=mAction.getCost();
	}
        lastAction = action;
        return action;
    }    
    
    public void setSpottedPlayer( Character character, boolean canSee )
    {
	super.setSpottedPlayer( character, canSee );
        if( canSee )
        {
            target = character;
            state = SEEKING;
            //System.out.println( "Brute spotted player: " + index );
        }
    }

    public void setState( int state )
    {
        this.state = state;
    }

    public Weapon getWeapon()
    {
        return weapon;
    }

    public void setWeapon( Weapon weapon )
    {}
    
    public GameAction playAction()
    {
        int x;
        int y;

        Point cell;
        Vector enemies = new Vector();
        Character check;

        hasMoreActions = true;

        if( healthState == LOSING_ROCKET )
        {
            healthState = PISTOL;
            weapon = new Pistol();
        }
        if( alive )
        {
            for( int i=0; i < activeGame.getPlayerCharacters().size(); i++ )
            {
                check = activeGame.getPlayerCharacter( i );
                if( activeGame.getMap().hasLineOfSight( check.getLocation(), location ) )
                    enemies.add( check );
            }
            //System.out.println( "CyberBrute: playAction" );
            if( target != null )
                if( !target.isAlive() )
                {
                    target = null;
                    state = ROAMING;
                }
                else
                {
                    state = SEEKING;
                }
            if( currentActionPoints <= 0 )
            {
                lastAction = new WaitAction( this, new Point( location ) );
                return lastAction;
            }
            // If next to an enemy, attack
            //System.out.println( "Brute checking neighbours" );
            for( int i=0; i < enemies.size(); i++ )
            {
                check = (Character)enemies.elementAt(i);
                if( check.isAlive() )
                {
                    cell = check.getLocation();
                    if( activeGame.getMap().isNeighbor( location, cell ) )
                    {
                        if( currentActionPoints >= getMeleeCost() )
                        {
                            state = ATTACKING;
                            direction = Map.getDirection( location, cell, direction );
                            lastAction = new MeleeAction( this, check );
                            return lastAction;
                        }
                        else
                        {
                            state = WAITING;
                            lastAction = new WaitAction( this, new Point( location ) );
                            hasMoreActions = false;
                            return lastAction;
                        }
                    }
                }
            }
            // If not next to an enemy, but can see one, shoot it
            if( enemies.size() > 0 )
            {
                target = activeGame.getClosestCharacter( this, enemies );
                if( activeGame.getMap().getLineOfFire( location, target.getLocation(), weapon.getBlockingSize() ).equals( target.getLocation() ) && ( location.distance( target.getLocation() ) <= 16 ) && ( location.distance( target.getLocation() ) > 2 ) && currentActionPoints > 2 )
                {
                    state = SHOOTING;
                    direction = Map.getDirection( location, target.getLocation(), direction );
                    lastAction = new ShootAction( this, weapon, target.getLocation(), true );
                    System.out.println( "CyberBrute shooting at " + target.toString() + ", action points: " + currentActionPoints );
                    return lastAction;
                }
                else
                {
                    state = SEEKING;
                }
            }
            if( state == SEEKING )
            {
                //System.out.println( "Brute seeking targets" );
                if( target != null )
                {
                    //System.out.println(" Brute seeking: " + target.toString() );
                    Vector path = activeGame.getMap().findShortestPath( this, target.getLocation(), true );
                    if( path.size() >= 2 )
                    {
                        cell = ((MapNode)path.elementAt( 1 )).getPoint();
                        if( activeGame.getMap().isClosedDoor( cell ) )
                        {
			    lastAction = new UseAction( this, cell );
			    return lastAction; 
                        }
                        direction = Map.getDirection( location, cell, direction );
                        lastAction = new MoveAction( this, new Point( location ), new Point( cell ) );
                        //System.out.println( "Ripper walking from " + location.toString() + " to " + cell.toString() );
                        location = new Point( cell );
                        return lastAction;
                    }
                    else
                    {
                        state = ROAMING;
                        target = null;
                    }
                }
            }
            if( state == ROAMING )
            {
                // Roam around mindlessly like a common Oid
                do
                {
                    x = location.x;
                    y = location.y;
                    x = x + (int)(Math.random()*3)-1;
                    y = y + (int)(Math.random()*3)-1;
                } while( !activeGame.getMap().isPassable( x, y, false ) );
		lastAction = new MoveAction( this, new Point( location ), new Point( x, y ) );
		//System.out.println( "Brute roaming" );
            }
        }
        return lastAction;
    }

        public GameAction getLastAction()
        {
            //System.out.println( "Get action from Brute " +index );
            return lastAction;
        }

        public void resetActionsPoints()
        {
            currentActionPoints = maximumActionPoints;
        }

        public boolean hasMoreActions()
        {
            if( currentActionPoints > 0 && hasMoreActions )
            {
                //System.out.println( "Brute has " + currentActionPoints + " action points left, attacking: " + attacking +", seeking: " + seeking );
                return true;
            }
            return false;
        }

        public int getWeaponSkill()
        {
            return skill;
        }

        public boolean isAlive()
        {
            return alive;
        }

        public boolean applyDamage( int damage )
        {
            if( damage > 100 )
            {
                finalDeathSequence();
                return false;
            }
            if( healthState == LOSING_ROCKET || healthState == FINAL_DEATH )
            {
                return alive;
            }
            currentHitPoints = currentHitPoints-damage;
            lastAction = new WoundAction( this, location );
            if( currentHitPoints <= 0 )
            {
                if( healthState == ROCKET )
                {
                    loseRocketSequence();
                    maximumHitPoints = 16;
                    currentHitPoints = 16;
                    return true;
                }
                if( healthState == PISTOL )
                {
                    finalDeathSequence();
                }
            }
            return alive;
        }

	public void addKill()
	{}
	
	public void registerHit()
	{}
	
        private void loseRocketSequence()
        {
            healthState = LOSING_ROCKET;
            int rocketDir = (direction+6)%8;
            Point point = Map.getNeighbor( location, rocketDir );
            activeGame.handleRangedAttack( this, point );
            currentActionPoints = maximumActionPoints;
        }
        
        private void finalDeathSequence()
        {
            Point point = new Point();
            weapon = new AssaultRifle();
            weapon.setFiringMode( AssaultRifle.GRENADE );
            lastAction = new WoundAction( this, location );
            healthState = FINAL_DEATH;
	    System.out.println( "CyberBrute dying at: " + location.x + ", " +location.y );
            for( int i = 0; i < 2+(int)(Math.random()*3); i++ )
            {
                currentActionPoints = weapon.getActionsToFire();
                point.x = location.x+(int)(Math.random()*5)-2;
                point.y = location.y+(int)(Math.random()*5)-2;
		System.out.println( "Going boom at: " + point.x + ", " + point.y );
                activeGame.handleRangedAttack( this, new Point( point ) );
            }
            currentActionPoints = weapon.getActionsToFire();
            activeGame.handleRangedAttack( this, location );
            lastAction = new DeadAction( this, new Point( location ) );
            maximumActionPoints = 0;
            currentActionPoints = 0;
            alive = false;
        }

        public int getHealthState()
        {
            return healthState;
        }
	
	public int getMeleeAbility()
	{
	    return 10;
	}
        
	public int getMeleeCost()
	{
	    return 2;   
	}
	
        public int getSize()
        {
            return 95;
        }
}