// DestroySceneryObjective.java
// Started on August 1st, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class DestroySceneryObjective extends Objective
{
    public static final String ADD_TARGET_KEY = "Add target";
    public static final String REMOVE_TARGET_AT_KEY = "Remove target";

    private Vector properties = new Vector();
    private Vector targetIds = new Vector();
    private Vector targets = new Vector();

    public DestroySceneryObjective()
    {
        properties.add( ADD_TARGET_KEY );
        properties.add( REMOVE_TARGET_AT_KEY );
    }
    
    public void setGame( Game game )
    {
        activeGame = game;
	MapObject mapObject;
	String id;
        for( int i = 0; i < targetIds.size(); i++ )
        {
            id = (String)targetIds.elementAt( i );
            mapObject = game.getMap().getObject( id );
            if( mapObject != null )
            {
                targets.add( mapObject );
            }
        }
    }
    
    public boolean isCompleted()
    {
	MapObject object;
	for( int i=0; i < targets.size(); i++ )
	{
	    object = (MapObject)targets.elementAt( i );
	    if( !object.isDestroyed() )
	    {
		return false;
	    }
	}
	return true;
    }
    
    public boolean isDestructible()
    {
	return true;
    }
    
    public boolean isSuccessful()
    {
	return isCompleted();
    }
    
    public boolean isShownOnMap()
    {
	return true;
    }
    
    public boolean isUsable()
    {
	return false;
    }

    // Returns true if destroyed
    public boolean handleDamage( Point point, int damage )
    {
	// The objects can handle getting damaged by themselves, so we don't
	// need to do anything here
	return false;
    }    
    
    public String [] getPropertyNames()
    {
        String [] propertyNames = new String [ properties.size() + targetIds.size() ];
        for( int i = 0; i < properties.size(); i++ )
        {
            propertyNames[ i ] = (String)properties.elementAt( i );
        }
	for( int i = 0; i < targetIds.size(); i++ )
	{
	    propertyNames[ properties.size()+i ] = "Target_"+ i;
	}
        return propertyNames;
    }

    public String getProperty( String key )
    {
        if( key.startsWith( "Target_" ) )
        {
            int index = Objective.intFromString( key.substring( 7 ) );
	    if( index >= 0 && index < targetIds.size() )
	    {
		return (String)targetIds.elementAt( index );
	    }
        }
        return null;
    }

    public Class getPropertyClass( String key )
    {
        if( key.equals( ADD_TARGET_KEY ) )
        {
            return MapObject.class;
        }
        if( key.equals( REMOVE_TARGET_AT_KEY ) )
        {
	    return Integer.class;
        }
        if( key.startsWith( "Target_" ) )
        {
            return MapObject.class;
        }
        return String.class;
    }    
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( ADD_TARGET_KEY ) )
        {
            if( value == null )
            {
                return;
            }
	    targetIds.add( value );
	}
        if( key.equals( REMOVE_TARGET_AT_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            int index = Objective.intFromString( key.substring( 7 ) );
	    if( index >= 0 && index < targetIds.size() )
	    {
		targetIds.removeElementAt( index );
	    }
        }
        if( key.startsWith( "Target_" ) )
        {
	    if( value == null )
            {
                return;
            }
	    int index = Objective.intFromString( key.substring( 7 ) );
	    if( index >= 0 && index < targetIds.size() )
	    {
		targetIds.set( index, value );
	    }
	    else
	    {	    
		targetIds.add( value );
	    }
        }
     }
    
    public boolean isLocatedIn( Point point )
    {
        MapObject check;
        for( int i = 0; i < targets.size(); i++ )
        {
            check = (MapObject)targets.elementAt( i );
            if( check.isLocatedIn( point ) && !check.isDestroyed() )
            {
                return true;
            }
        }
        return false;
    }

    public boolean isLocatedIn( int x, int y )
    {
        return isLocatedIn( new Point( x, y ) );
    }
    
}