// ExitAreaObjective.java
// Started on July 29th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.io.*;

public class ExitAreaObjective extends Objective
{
    protected Game activeGame;
    private Point topLeft = new Point( -1, -1 );
    private Point bottomRight = new Point( -1, -1 );
    private boolean returned = false;
    
    public static final String TOP_LEFT_KEY = "Top left";
    public static final String BOTTOM_RIGHT_KEY = "Bottom right";
    public static final String [] propertyNames = { TOP_LEFT_KEY, BOTTOM_RIGHT_KEY };
    
    public void setGame( Game game )
    {
	activeGame = game;
    }
    
    public int getNumberOfSprites()
    {
	return 0;
    }
    
    public Sprite getSprite( int x, int y )
    {
	return null;
    }
    
    public String [] getPropertyNames()
    {
        return propertyNames;
    }
    
    public Class getPropertyClass( String key )
    {
	if( key.equals( TOP_LEFT_KEY ) )
	{
	    return Point.class;
	}
	if( key.equals( BOTTOM_RIGHT_KEY ) )
	{
	    return Point.class;   
	}
        return String.class;
    }
    
    
    public String getProperty( String key )
    {
	if( key.equals( TOP_LEFT_KEY ) )
	{
	    return Objective.pointToString( topLeft );
	}
	if( key.equals( BOTTOM_RIGHT_KEY ) )
	{
	    return Objective.pointToString( bottomRight );   
	}
        return null;
    }
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
	if( key.equals( TOP_LEFT_KEY ) )
	{
	    topLeft = Objective.pointFromString( value );
	}
	if( key.equals( BOTTOM_RIGHT_KEY ) )
	{
	    bottomRight = Objective.pointFromString( value );
	}
	if( topLeft.x > -1 && bottomRight.x > -1 )
	{
	    topLeft.x = Math.min( topLeft.x, bottomRight.x );
	    topLeft.y = Math.min( topLeft.y, bottomRight.y );
	    bottomRight.x = Math.max( topLeft.x, bottomRight.x );
	    bottomRight.y = Math.max( topLeft.y, bottomRight.y );
	}
    }
        
    public boolean isCompleted()
    {
        returned = true;
        Character character;
        Point location;
        boolean characterReturned;
	
	int minX = Math.min( topLeft.x, bottomRight.x );
	int minY = Math.min( topLeft.y, bottomRight.y );
	int maxX = Math.max( topLeft.x, bottomRight.x );
	int maxY = Math.max( topLeft.y, bottomRight.y );
	
        for( int i = 0; i < activeGame.getPlayerCharacters().size(); i++ )
	{
            character = activeGame.getPlayerCharacter(i);
            characterReturned = false;
	    if( character.isAlive() )
	    {
                for( int j = 0; j < activeGame.getPlayerStartingPoints().size(); j++ )
                {
                    location = character.getLocation();
		    if( location.x >= minX && location.x <= maxX && location.y >= minY && location.y <= maxY )
		    {
                        characterReturned = true;
		    }
                }
	    }
            else
            {
                // Dead count as returned
                characterReturned=true;
            }
            returned = (returned && characterReturned );
        }
	return returned;
    }
    
    public boolean isLocatedIn( Point point )
    {
	return isLocatedIn( point.x, point.y );
    }
    
    public boolean isLocatedIn( int x, int y )
    {
	int minX = Math.min( topLeft.x, bottomRight.x );
	int minY = Math.min( topLeft.y, bottomRight.y );
	int maxX = Math.max( topLeft.x, bottomRight.x );
	int maxY = Math.max( topLeft.y, bottomRight.y );
	return ( x >= minX && x <= maxX && y >= minY && y<= maxY );
    }
    
    public boolean isDestructible()
    {
	return false;
    }
    
    public boolean isShownOnMap()
    {
        return true;
    }
    
    public boolean handleDamage( Point point, int damage )
    {
	return false;
    }
    
    public boolean handleAction( GameAction action )
    {
        return false;
    }
    
}