// Flamer.java
// Started on June 23rd, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.ImageIcon;
import java.util.Vector;

public class Flamer extends Weapon
{
    private String [] firingModes = { "Normal" };
    private static Image icon = null;
    public static final int COST = 9;

    
    private int baseRange = 8;
    private double radius = 25;
    private Point [] area = null;

    public Flamer()
    {
        if( icon == null )
        {
            icon = ResourceManager.loadImage( "WeaponGraphics/FlamerIcon.png" );
        }
    }
    
    public int getDamage()
    {
        return (int)(Math.random()*7)+3;
    }
    
    public int getBlockingSize()
    {
        return 100;
    }
    
    
    public int getActionsToFire()
    {
        return 2;
    }

    public int getCost()
    {
        return COST;
    }    
    
    public int getMovementCost()
    {
        return 2;
    }    

    public boolean hasAreaEffect()
    {
        return true;
    }

    public String getName()
    {
        return "Flamer";
    }

    public Point [] getArea( Game theGame, Point fireFrom, Point fireTo )
    {
        area = null;
        Vector points = new Vector();
        Point point;
        int x, y, width, height;
        double start, extent;

        if( !fireFrom.equals( fireTo ) )
        {
            // Uses Arc2D to save the trouble of calculating the angle between the points
            Arc2D.Double helper = new Arc2D.Double( fireFrom.x-1, fireFrom.y-1, 2, 2, 0.0, 0.0, Arc2D.OPEN );
            helper.setAngles( new Point( fireFrom.x, fireFrom.y-1 ), fireTo );
            double angle = helper.getAngleExtent()+90.0;
            System.out.println( "Firing flamer at angle: " + angle );
            x=fireFrom.x-baseRange;
            y=fireFrom.y-baseRange;
            width=2*baseRange;
            height=2*baseRange;
            start = angle-radius/2;
            extent = radius;
        }
        else
        {
            x=fireFrom.x-baseRange/4;
            y=fireFrom.y-baseRange/4;
            width=baseRange/2;
            height=baseRange/2;
            start = 0.0;
            extent = 360.0;
        }
        System.out.println( "Flamer area: " + x + ", " +y + " to " + (x+width) + ", " + (y+height) + " from " + fireFrom.x + ", " + fireFrom.y );
        Arc2D.Double template = new Arc2D.Double( x, y, width, height, start, extent, Arc2D.PIE );
        Rectangle2D.Double rect = (Rectangle2D.Double)template.getBounds2D();
        x=(int)rect.x;
        y=(int)rect.y;
        width=(int)rect.width;
        height=(int)rect.height;
        for( int cX=x; cX<=(x+width); cX++ )
        {
            for( int cY=y; cY<=(y+height); cY++ )
            {
                if( ( cX > 0 ) && ( cX < theGame.getMap().getHorizontalSize() ) && ( cY > 0 ) && ( cY < theGame.getMap().getVerticalSize() ) )
                {
                    point = new Point( cX, cY );
                    if( template.intersects( (double)cX-0.5, (double)cY-0.5, 0.6, 0.6 ) )
                    {
                        if( point.equals( theGame.getMap().getLineOfFire( fireFrom, point, 100 ) ) )
                        {
                            if( !point.equals( fireFrom ) )
                            {
                                points.add( point );
                            }
                        }
                    }
                }
            }
        }
        if( fireFrom.equals( fireTo ) )
        {
            points.add( fireFrom );
        }	
	Point nextPoint = Map.getNeighbor( fireFrom, Map.getDirection( fireFrom, fireTo, 0 ) );
	boolean found = false;
	for( int i=0; i < points.size() && !found; i++ )
	{
	    if( ((Point)points.elementAt( i )).equals( nextPoint ) )
	    {
		found = true;
	    }
	}
	if( !found )
	{
	    points.add( nextPoint );
	}
        System.out.println( "Flamer area: " + points.size() );
        area = new Point[ points.size() ];
        for( int i=0; i < points.size(); i++ )
        {
            area[ i ] = (Point)points.elementAt( i );
        }
        return area;
    }

    public Point [] getLastArea()
    {
        return area;
    }
    
    public String [] getFiringModeNames()
    {
        return firingModes;
    }

    public int getNumberOfFiringModes()
    {
        return 1;
    }

    public int getFiringMode()
    {
        return 0;
    }

    public void setFiringMode( int mode )
    {
    }

    public int getRequiredAttribute()
    {
        return Marine.HEAVY_WEAPONS;
    }

    public Image getIcon()
    {
        return icon;
    }
}