// HackComputerObjective.java
// Started on May 29th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;
import java.util.Vector;

public class HackComputerObjective extends Objective
{
    public static final String ADD_TARGET_KEY = "Add target";
    public static final String REMOVE_TARGET_AT_KEY = "Remove target at";
    public static final String DIRECTION_KEY = "Direction";
    public static final String FAIL_TARGET_KEY = "Fail target";
    public static final String FAIL_KEY_KEY = "Fail key";
    public static final String FAIL_VALUE_KEY = "Fail value";
    public static final String DIFFICULTY_KEY = "Difficulty";
    public static final String SUCCESSES_KEY = "Successes needed";
    public static final String COST_KEY = "Action cost";
    public static final String MAXIMUM_REWARD_KEY = "Maximum reward";
    public static final String SHOW_ON_MAP_KEY = "Show on map";
    public static final String SHOW_MESSAGE_KEY = "Show message";
    public static final String HACK_MESSAGE_KEY = "Hack message";

    private Vector targets = new Vector();
    private Vector targetObjectives = new Vector();
    private boolean cracked = false;
    private boolean primary = true;
    private boolean destroyed = false;
    private boolean showOnMap = true;
    private boolean showMessage = true;
    private ComputerSprite computerSprite;
    private int direction = 0;
    private int hitPoints = 4;
    private int successes = 0;
    private int successesNeeded = 3;
    private int maximumReward = 3;
    private int actionsCost = 3;
    private int difficulty = 9;
    private int failTarget = -1;
    private String failKey = "";
    private String failValue = "";
    private Point location = new Point( -1, -1 );
    private String hackMessage = "$RANKNAME$ has hacked the computer!";

    private String [] properties = { "Location", DIRECTION_KEY, DIFFICULTY_KEY, COST_KEY, HACK_MESSAGE_KEY, SUCCESSES_KEY, MAXIMUM_REWARD_KEY, SHOW_ON_MAP_KEY, SHOW_MESSAGE_KEY, ADD_TARGET_KEY, REMOVE_TARGET_AT_KEY, FAIL_TARGET_KEY, FAIL_KEY_KEY, FAIL_VALUE_KEY };
    
    public HackComputerObjective()
    {
        computerSprite = new ComputerSprite();
    }

    public void setGame( Game game )
    {
        activeGame = game;
        if( location.x < 0 || location.y < 0 )
        {
            Point point = null;
            while( point == null )
            {
                point = activeGame.getRandomPointForMonster();
                if( activeGame.getMap().isPassable( point, Map.NORTH ) )
                {
                    location = point;
                    direction = Map.NORTH;
                }
                else if( activeGame.getMap().isPassable( point, Map.EAST ) )
                {
                    location = point;
                    direction = Map.EAST;
                }
                else if( activeGame.getMap().isPassable( point, Map.SOUTH ) )
                {
                    location = point;
                    direction = Map.SOUTH;
                }
                else if( activeGame.getMap().isPassable( point, Map.WEST ) )
                {
                    location = point;
                    direction = Map.WEST;
                }
                else
                {
                    point = null;
                }
            }
        }
        computerSprite.setDirection( direction );
    }

    public int getNumberOfSprites()
    {
        return 1;
    }    
    
    public Sprite getSprite( int x, int y)
    {
        return (Sprite)computerSprite;
    }

    public String [] getPropertyNames()
    {
        int propertyCount = properties.length + (targets.size()*3);
        //System.out.println( "HackComputerObjective has " + propertyCount + " properties with " + targets.size() + " targets" );
        String [] propertyNames = new String[ propertyCount ];
        for( int i = 0; i < properties.length; i++ )
        {
            propertyNames[ i ] = properties[ i ];
        }
        int index = 0;
        for( int i = properties.length; i < propertyCount; i++ )
        {
            propertyNames[ i++ ] = "Target_"+index;
            propertyNames[ i++ ] = "Target Key_"+index;
            propertyNames[ i ] = "Target Value_"+index;
            index++;
        }
        return propertyNames;
    }

    private ObjectiveTarget getTarget( String id )
    {
	for( int i=0; i < targets.size(); i++ )
	{
	    ObjectiveTarget target = (ObjectiveTarget)targets.elementAt( i );
	    if( target.getIndex().equals( id ) )
	    {
		return target;
	    }
	}
	return null;
    }
    
    public Class getPropertyClass( String key )
    {
        if( key.equals( Objective.LOCATION_KEY ) )
        {
            return Point.class;
        }
        if( key.equals( DIRECTION_KEY ) )
        {
            return Integer.class;
        }
        if( key.equals( DIFFICULTY_KEY ) )
        {
            return Integer.class;
        }
	if( key.equals( SHOW_ON_MAP_KEY ) )
	{
	    return Boolean.class;
	}
	return String.class;
    }
    
    public String getProperty( String key )
    {
        if( key.equals( Objective.LOCATION_KEY ) )
        {
            return Objective.pointToString( location );
        }
        if( key.equals( DIRECTION_KEY ) )
        {
            return String.valueOf( direction );
        }
        if( key.equals( DIFFICULTY_KEY ) )
        {
            return String.valueOf( difficulty );
        }
	if( key.equals( SHOW_ON_MAP_KEY ) )
        {
            if( showOnMap )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }
        if( key.equals( SHOW_MESSAGE_KEY ) )
        {
            return Objective.booleanToString( showMessage );
        }
        if( key.equals( SUCCESSES_KEY ) )
        {
            return String.valueOf( successesNeeded );
        }
        if( key.equals( COST_KEY ) )
        {
            return String.valueOf( actionsCost );
        }
	if( key.equals( HACK_MESSAGE_KEY ) )
	{
	    return hackMessage;
	}
        if( key.equals( MAXIMUM_REWARD_KEY ) )
        {
            return String.valueOf( maximumReward );
        }	
        if( key.equals( FAIL_TARGET_KEY ) )
        {
            return String.valueOf( failTarget );
        }
        if( key.equals( FAIL_KEY_KEY ) )
        {
            return failKey;
        }
        if( key.equals( FAIL_VALUE_KEY ) )
        {
            return failValue;
        }
        if( key.startsWith( "Target_" ) )
        {
            int index = Integer.parseInt( key.substring( 7 ) );
            return ((ObjectiveTarget)targets.elementAt( index )).getIndex();
        }
        if( key.startsWith( "Target Key_" ) )
        {
            int index = Integer.parseInt( key.substring( 11 ) );
            return ((ObjectiveTarget)targets.elementAt( index )).getKey();
        }
        if( key.startsWith( "Target Value_" ) )
        {
            int index = Integer.parseInt( key.substring( 13 ) );
            return ((ObjectiveTarget)targets.elementAt( index )).getValue();
        }
        return null;
    }

    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( Objective.LOCATION_KEY ) )
        {
            location = new Point( Objective.pointFromString( value ) );
        }
        if( key.equals( DIRECTION_KEY ) )
        {
            direction = Objective.intFromString( value );
            computerSprite.setDirection( direction );
        }
        if( key.equals( DIFFICULTY_KEY ) )
        {
            difficulty = Objective.intFromString( value );
        }
        if( key.equals( SUCCESSES_KEY ) )
        {
            successesNeeded = Objective.intFromString( value );
        }
        if( key.equals( SHOW_ON_MAP_KEY ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                showOnMap = true;
            }
            else
            {
                showOnMap = false;
            }
        }
        if( key.equals( SHOW_MESSAGE_KEY ) )
        {
            showMessage = Objective.booleanFromString( value );
        }
        if( key.equals( COST_KEY ) )
        {
            actionsCost = Objective.intFromString( value );
        }
	if( key.equals( HACK_MESSAGE_KEY ) )
	{
	    hackMessage = value;
	}
        if( key.equals( MAXIMUM_REWARD_KEY ) )
        {
            maximumReward = Objective.intFromString( value );
        }
        if( key.equals( ADD_TARGET_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            String id = value;
            ObjectiveTarget newTarget = new ObjectiveTarget( id );
            targets.add( newTarget );
        }
        if( key.equals( REMOVE_TARGET_AT_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            int index = Objective.intFromString( value );
	    targets.removeElementAt( index );
	}
        if( key.equals( FAIL_TARGET_KEY ) )
        {
            failTarget = Objective.intFromString( value );
        }
        if( key.equals( FAIL_KEY_KEY ) )
        {
            failKey = value;
        }
        if( key.equals( FAIL_VALUE_KEY ) )
        {
            failValue = value;
        }
        if( key.startsWith( "Target_" ) )
        {
            int index = Integer.parseInt( key.substring( 7 ) );
            // Check whether target exists
            if( targets.size() > index )
            {
                ((ObjectiveTarget)targets.elementAt( index )).setIndex( value );
            }
            else
            {
                // If it doesn't exist, add it
                setProperty( ADD_TARGET_KEY, value );
            }
        }
        if( key.startsWith( "Target Key_" ) )
        {
            int index = Integer.parseInt( key.substring( 11 ) );
            if( targets.size() > index )
            {
                ((ObjectiveTarget)targets.elementAt( index )).setKey( value );
            }
        }
        if( key.startsWith( "Target Value_" ) )
        {
            int index = Integer.parseInt( key.substring( 13 ) );
            if( targets.size() > index )
            {
                ((ObjectiveTarget)targets.elementAt( index )).setValue( value );
            }
        }
    }    
    
    public boolean isCompleted()
    {
        return ( cracked || destroyed );
    }

    public boolean isSuccessful()
    {
	return cracked;
    }
    
    public int getSize()
    {
        return 20;
    }

    public boolean isLocatedIn( Point point )
    {
        if( location.x == point.x && location.y == point.y )
            return true;
        return false;
    }

    public boolean isLocatedIn( int x, int y )
    {
        if( location.x == x && location.y == y )
            return true;
        return false;
    }

    public boolean isDestructible()
    {
        if( destroyed )
            return false;
        return true;
    }

    public boolean isShownOnMap()
    {
        if( destroyed )
            return false;
        return showOnMap;
    }
    
    public boolean isUsable()
    {
	return !destroyed;
    }

    public boolean handleDamage( Point point, int damage )
    {
        if( isLocatedIn( point ) && !destroyed )
        {
            hitPoints -= damage;
            if( hitPoints <= 0 )
            {
                destroyed = true;
                computerSprite.setDestroyed();
                if( !cracked )
                {
                    System.out.println( "Computer broken" );
                    activeGame.handleMessage( new Message( null, "The computer is broken!" ) );
                    if( failTarget > -1 )
                    {
                        try
                        {
                            Objective objective = activeGame.getObjective( failTarget-1 );
                            objective.setProperty( failKey, failValue );
                        }
                        catch( IllegalArgumentException iae )
                        {}
                    }
                }
                return true;
            }
        }
        return false;
    }

    public boolean handleAction( GameAction action )
    {
        boolean handled = false;
        Marine user;
        if( action instanceof UseAction )
        {
            if( ((UseAction)action).getActor() instanceof Marine )
            {
                user = (Marine)((UseAction)action).getActor();
                if( location.equals( ((UseAction)action).getPoint() ) )
                {
		    String messageString = "";
                    if( user.getCurrentActionPoints() >= actionsCost )
                    {
                        handled = true;
			((UseAction)action).setConsumed( true );
                        ((UseAction)action).setCost( actionsCost );
                        if( user.getAttribute( Marine.ENGINEERING ) + (int)(Math.random()*10) >= difficulty )
                        {
                            successes++;
			    messageString = Marine.ranks[ user.getRank() ] + " " + user.getName()+" is working on the computer, but isn't ready yet. Try again.";
                        }
			else
			{
			    messageString = Marine.ranks[ user.getRank() ] + " " + user.getName()+" failed to crack the computer. Try again.";
			}
                        if( successes >= successesNeeded )
                        {
			    if( !cracked )
			    {
				System.out.println( "Computer cracked!" );
				cracked = true;
				int reward = (int)( Math.random()*maximumReward );
				user.grantExperienceTo( Marine.ENGINEERING, reward );
			    }
			    successes = 0;
			    String rankName = Marine.ranks[ user.getRank() ] + " " + user.getName();
			    messageString = hackMessage.replaceAll( "\\$RANKNAME\\$", rankName );
                            // Activate targets
                            for( int i = 0; i < targets.size(); i++ )
                            {
                                try
                                {
                                    //System.out.println( "Activating target " + (i+1) );
                                    ObjectiveTarget target = (ObjectiveTarget)targets.elementAt( i );
                                    Objective objective = activeGame.getObjectiveById( target.getIndex() );
                                    objective.setProperty( target.getKey(), target.getValue() );
				    //System.out.println( objective.getClass().toString() + ": " + target.getKey() + " = " + target.getValue() ); 
                                }
                                catch( Exception e )
                                {
                                    System.out.println( e.toString() );
                                }
                            }
                        }
			if( showMessage )
			{
			    activeGame.handleMessage( new Message( (Character)user, messageString ) );
			}
                    }
		    else
		    {
			activeGame.handleOutOfAP( user );
		    }
                }
            }
        }
        return handled;
    }
}