// LavaFlowObjective.java
// Started on December 22nd, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.util.Vector;
import java.io.*;

public class LavaFlowObjective extends Objective
{
    public static final int LAVA_DAMAGE = 1000;
    public static final int MAXIMUM_LAVA_FLOW_SPEED = 5;

    protected Game activeGame;
    private Point location = new Point(-1,-1);
    private Sprite lavaSprite;
    private Vector flowFront;
    private boolean [][] flooded;
    private int flowSpeed = 1;

    private boolean started = false;

    public LavaFlowObjective()
    {
        lavaSprite = new LavaSprite( );
    }

    public void setGame( Game game )
    {
        activeGame = game;
        flooded = new boolean[ game.getMap().getHorizontalSize() ][ game.getMap().getVerticalSize() ];
        for( int i = 0; i < game.getMap().getHorizontalSize(); i++ )
            for( int j = 0; j < game.getMap().getVerticalSize(); j++ )
                flooded[i][j] = false;
    }

    public void startFlooding()
    {
        if( !started )
        {
            flowFront = new Vector();
            flowFront.add( location );
            flooded[ location.x ][ location.y ] = true;
            activeGame.getMap().setLight( location.x, location.y, (byte)0);
            started = true;
        }
    }

    public void flood()
    {
        Vector newFront = new Vector();
        Point point;
        for( int i=0; i < flowFront.size(); i++ )
        {
            point = (Point)flowFront.elementAt( i );
            // Check the neighbours and flood them if necessary
            if( activeGame.getMap().isViewable( point.x, point.y-1, true ) && !flooded[ point.x ][ point.y-1 ] && !activeGame.isPlayerStartingPoint( new Point( point.x, point.y-1 ) ) )
            {
                newFront.add( new Point( point.x, point.y-1 ) );
                flooded[ point.x ][ point.y-1 ] = true;
                activeGame.getMap().setLight( point.x, point.y-1, (byte)0);
            }
            if( activeGame.getMap().isViewable( point.x+1, point.y, true ) && !flooded[ point.x+1 ][ point.y ] && !activeGame.isPlayerStartingPoint( new Point( point.x+1, point.y ) ) )
            {
                newFront.add( new Point( point.x+1, point.y ) );
                flooded[ point.x+1 ][ point.y ] = true;
                activeGame.getMap().setLight( point.x+1, point.y, (byte)0);
            }
            if( activeGame.getMap().isViewable( point.x, point.y+1, true ) && !flooded[ point.x ][ point.y+1 ] && !activeGame.isPlayerStartingPoint( new Point( point.x, point.y+1 ) ) )
            {
                newFront.add( new Point( point.x, point.y+1 ) );
                flooded[ point.x ][ point.y+1 ] = true;
                activeGame.getMap().setLight( point.x, point.y+1, (byte)0);
            }
            if( activeGame.getMap().isViewable( point.x-1, point.y, true ) && !flooded[ point.x-1 ][ point.y ] && !activeGame.isPlayerStartingPoint( new Point( point.x-1, point.y ) ) )
            {
                newFront.add( new Point( point.x-1, point.y ) );
                flooded[ point.x-1 ][ point.y ] = true;
                activeGame.getMap().setLight( point.x-1, point.y, (byte)0);
            }
        }
        flowFront = newFront;
    }

    public int getNumberOfSprites()
    {
        if( started )
            return 1;
        return 0;
    }

    public Sprite getSprite( int x, int y )
    {
        return lavaSprite;
    }

    public String [] getPropertyNames()
    {
        return null;
    }

    public String getProperty( String key )
    {
        return null;
    }
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {}

    public void setLocation( Point point )
    {
        location = point;
    }

    public Point getLocation()
    {
        return location;
    }

    public void setLocation( int x, int y )
    {
        setLocation( new Point( x, y ) );
    }

    public boolean isCompleted()
    {
        Character character;
        MapObject mapObject;
        if( started )
        {
            for( int i = 0; i < flowSpeed; i++ )
            {
                flood();
            }
            if( flowSpeed < MAXIMUM_LAVA_FLOW_SPEED )
                flowSpeed++;
        }
        for( int i = 0; i < activeGame.getPlayerCharacters().size(); i++ )
        {
            character = activeGame.getPlayerCharacter( i );
            if( flooded[ character.getLocation().x ][ character.getLocation().y ] )
            {
                if( character.isAlive() )
                {
                    character.applyDamage( LAVA_DAMAGE );
                }
            }
        }
        for( int i = 0; i < activeGame.getMonsters().size(); i++ )
        {
            character = activeGame.getMonster( i );
            Point point = character.getLocation();
            if( point.x > 0 && point.y > 0 )
            {
                if( flooded[ point.x ][ point.y ] )
                {
                    if( character.isAlive() )
                    {
                        character.applyDamage( LAVA_DAMAGE );
                    }
                }
            }
        }	
	
	Vector damagedObjects = new Vector();
        for( int i=0; i < activeGame.getMap().getNumberOfObjects(); i++ )
        {
            mapObject = activeGame.getMap().getObject( i );
	    MapObjectCell [] cells = mapObject.getCells();
	    for( int j=0; j < cells.length; j++ )
	    {
		if( flooded[ cells[j].getX() ][ cells[j].getY() ] )
		{
		    if( (!mapObject.isDestroyed()) && (!damagedObjects.contains( mapObject )) )
		    {
			damagedObjects.add( mapObject );
			mapObject.applyDamage( LAVA_DAMAGE );
		    }
		}
            }
        }
        return true;
    }

    public boolean isLocatedIn( Point point )
    {
        if( point.x < 0 || point.y < 0 )
        {
            return false;
        }
        if( started )
        {
            if( flooded[ point.x ][ point.y ] )
                return true;
        }
        return false;
    }

    public boolean isLocatedIn( int x, int y )
    {
        if( x < 0 || y < 0 )
        {
            return false;
        }
        if( started )
        {
            if( flooded[ x ][ y ] )
                return true;
        }
        return false;
    }

    public boolean isDestructible()
    {
        return false;
    }

    public boolean isShownOnMap()
    {
        if( started )
            return true;
        return false;
    }

    public boolean handleDamage( Point point, int damage )
    {
        return false;
    }

    public boolean handleAction( GameAction action )
    {
        MoveAction mAction;
        if( action instanceof MoveAction )
        {
            mAction = (MoveAction)action;
            if( isLocatedIn( mAction.getEndPoint() ) )
            {
                mAction.getActor().applyDamage( LAVA_DAMAGE );
                return true;
            }
        }
        return false;
    }

    
}