// LockedDoorObjective.java
// Started on May 28th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class LockedDoorObjective extends Objective
{
    public static final String LOCK_DOOR_KEY = "Lock door";
    public static final String UNLOCK_DOOR_KEY = "Unlock door";
    public static final String OPEN_DOOR_KEY = "Open door";
    public static final String CLOSE_DOOR_KEY = "Close door";
    public static final String LOCKED_DOOR_MESSAGE_KEY = "Locked Message";
    public static final String SHOW_LOCKED_MESSAGE_ONCE_KEY = "Show message once";

    private Vector properties = new Vector();
    private Vector lockedDoors = new Vector();
    private String lockedMessage = "It seems to be locked";
    private boolean messageOnce = false;
    private boolean messageIsShown = false;

    public LockedDoorObjective()
    {
        properties.add( LOCK_DOOR_KEY );
        properties.add( UNLOCK_DOOR_KEY );
        properties.add( OPEN_DOOR_KEY );
	properties.add( CLOSE_DOOR_KEY );
        properties.add( LOCKED_DOOR_MESSAGE_KEY );
        properties.add( SHOW_LOCKED_MESSAGE_ONCE_KEY );
    }

    private void lockDoor( Door door, boolean locked )
    {
        if( activeGame != null )
        {
            door.setLocked( locked );
            Point otherPoint = Map.getNeighbor( new Point( door.getX(), door.getY() ), door.getDirection() );
            Door otherHalf = activeGame.getMap().getDoor( otherPoint.x, otherPoint.y );
            if( otherHalf != null )
            {
                if( !lockedDoors.contains( otherPoint ) && locked )
                {
                    lockedDoors.add( otherPoint );
                }
                otherHalf.setLocked( locked );
                if( !locked && lockedDoors.contains( otherPoint ) )
                {
                    lockedDoors.remove( otherPoint );
                }
            }
        }
    }
    
    public void setGame( Game game )
    {
        activeGame = game;
        Door door;
        Point point;
        for( int i = 0; i < lockedDoors.size(); i++ )
        {
            point = (Point)lockedDoors.elementAt( i );
            door = game.getMap().getDoor( point.x, point.y );
            if( door != null )
            {
                lockDoor( door, true );
            }
        }
    }

    public boolean handleAction( GameAction action )
    {
        Character character = action.getActor();
        if( character instanceof Marine )
        {
            if( action instanceof UseAction )
            {
                Point point = ((UseAction)action).getPoint();
                if( isLocatedIn( point ) && Map.isNeighbor( point, character.getLocation() ) )
                {
                    if( !messageIsShown || !messageOnce )
                    {
                        messageIsShown = true;
                        activeGame.handleMessage( new Message( character, lockedMessage ) );
                    }
                    return true;
                }
            }
        }
        return false;
    }
    
    public boolean isUsable()
    {
	return true;
    }
    

    public String [] getPropertyNames()
    {
        String [] propertyNames = new String [ properties.size() + lockedDoors.size() ];
        for( int i = 0; i < properties.size(); i++ )
        {
            propertyNames[ i ] = (String)properties.elementAt( i );
        }
        for( int i = properties.size(); i < properties.size() + lockedDoors.size(); i++ )
        {
            propertyNames[ i ] = "Door_" + String.valueOf( i-properties.size() );
        }
        return propertyNames;
    }

    public String getProperty( String key )
    {
        if( key.equals( LOCKED_DOOR_MESSAGE_KEY ) )
        {
            return lockedMessage;
        }
        if( key.equals( SHOW_LOCKED_MESSAGE_ONCE_KEY ) )
        {
            if( messageOnce )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }
        if( key.startsWith( "Door_" ) )
        {
            int index = Objective.intFromString( key.substring( 5 ) );
            return Objective.pointToString( (Point)lockedDoors.elementAt( index ) );
        }
        return null;
    }

    public Class getPropertyClass( String key )
    {
        if( key.equals( LOCK_DOOR_KEY ) )
        {
            return Door.class;
        }
        if( key.equals( UNLOCK_DOOR_KEY ) )
        {
	    return Door.class;
        }
        if( key.startsWith( "Door_" ) )
        {
            return Door.class;
        }
        return String.class;
    }    
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( LOCK_DOOR_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            Point point = Objective.pointFromString( value );
            if( !lockedDoors.contains( point ) )
            {
                lockedDoors.add( point );
                if( activeGame != null )
                {
                    Door theDoor = activeGame.getMap().getDoor( point.x, point.y );
                    if( theDoor != null )
                    {
                        lockDoor( theDoor, true );
                    }
                }
            }
        }
        if( key.equals( UNLOCK_DOOR_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            Point point = Objective.pointFromString( value );
            if( lockedDoors.contains( point ) )
            {
                lockedDoors.removeElement( point );
                if( activeGame != null )
                {
                    Door theDoor = activeGame.getMap().getDoor( point.x, point.y );
                    if( theDoor != null )
                    {
                        lockDoor( theDoor, false );
                    }
                }
            }
        }
        if( key.equals( OPEN_DOOR_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            Point point = Objective.pointFromString( value );
            if( activeGame != null )
            {
                activeGame.openDoor( null, point );
            }
        }
	if( key.equals( CLOSE_DOOR_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            Point point = Objective.pointFromString( value );
            if( activeGame != null )
            {
                activeGame.closeDoor( null, point );
            }
        }	
        if( key.equals( LOCKED_DOOR_MESSAGE_KEY ) )
        {
            lockedMessage = new String( value );
        }
        if( key.equals( SHOW_LOCKED_MESSAGE_ONCE_KEY ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                messageOnce = true;
            }
            else
            {
                messageOnce = false;
            }
        }
        if( key.startsWith( "Door_" ) )
        {
            setProperty( LOCK_DOOR_KEY, value );
        }
    }
    
    public boolean isLocatedIn( Point point )
    {
        Point check;
        for( int i = 0; i < lockedDoors.size(); i++ )
        {
            check = (Point)lockedDoors.elementAt( i );
            if( point.equals( check ) )
            {
                return true;
            }
        }
        return false;
    }

    public boolean isLocatedIn( int x, int y )
    {
        return isLocatedIn( new Point( x, y ) );
    }
    
}