//
//  MapNode.java
//
//  Created by Virtu Halttunen on Sat Dec 30 2000.
//  Copyright (c) 2000 Virtu Halttunen. All rights reserved.
//
// June 30th, 2001: Changed for TheGame
// April 17th, 2002: Added methods used for using the MapNode simply to store a cell and
// a direction

package TheGame;

import java.awt.Point;

public class MapNode 
{
    private int neighbours;
    private boolean random;
    private boolean [] checked;
    private int numChecked;
    private int lastChecked;
    private boolean visited;
    private MapNode next = null;
    int x;
    int y;

    /**
     * Use this constructor if you want to store a location and a direction
     * for a cell. 
     *
     * @param point	the location
     * @param direction	the direction
     */
    public MapNode( Point point, int direction )
    {
        neighbours = 0;
        lastChecked = direction;
        x = point.x;
        y = point.y;
        random = false;
    }

    /**
     * Use this constructor if you want to store a location and a direction
     * for a cell.
     *
     * @param x		the x coordinate for the location
     * @param y		the y coordinate for the location
     * @param direction	the direction
     */
    public MapNode( int x, int y, int direction )
    {
        neighbours = 0;
        lastChecked = direction;
        this.x = x;
        this.y = y;
        random = false;
    }
    
    public MapNode( Point point, int neighbours, boolean random)
    {
	x = point.x;
	y = point.y;
	this.neighbours = neighbours;
	this.random = random;
	checked = new boolean [ 8 ];
	numChecked = 0;
	lastChecked = -1;
	visited = false;
	for( int i = 0; i < checked.length; i++ )
	    checked[i]=false;
	if( neighbours == 4 )
	{
	    for( int i = 1; i < checked.length; i+=2 )
		checked[i]=true;
	    numChecked = 4;
	}
    }
    
    public void setNext( MapNode node )
    {
	next = node;
    }
    
    public MapNode getNext()
    {
	return next;
    }
   
    public void setChecked( int direction )
    {
	if( direction >= 0 && direction < checked.length )
	{
	    checked[direction]=true;
	    numChecked++;
	}
	else
	    System.out.println( "Illegal direction to be checked!" + direction );
    }
        
    public Point getPoint()
    {
	Point me = new Point( x, y );
	return me;
    }
       
    public int getX()
    {
	return x;
    }
    
    public int getY()
    {
	return y;
    }
    public boolean checkedAll()
    {
	if( numChecked >= 8 )
	    return true;
	else
	    return false;
    }
    
    public boolean checked( int direction )
    {	
	return checked[ direction ];
    }

    public int getDirection()
    {
        return lastChecked;
    }
    
    public int nextDirection()
    {
	int dir;
	if( !random )
	{
	    if( neighbours == 8 )
            {
                // Should check against directions that are setChecked, but...
		return ++lastChecked;
            }
	    if( neighbours == 4 )
	    {
		if( lastChecked == -1 )
		    lastChecked = -2;
		if( checkedAll() )
		    return -1;
		do
		{
		    lastChecked += 2;
		} while ( checked[ lastChecked ] );
		return lastChecked;
	    }
            return lastChecked;
	}
	else
	{
	    do dir = (int)(Math.random()*neighbours);
	    while( checked[ dir ] );
	    lastChecked=dir;
	    return dir;
	}
    }
}
