// Marine.java
// Started on June 20th, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.io.*;
import java.util.Vector;
import org.w3c.dom.*;

public class Marine extends Character implements Armed
{
    public static final int ATTRIBUTES = 13;
    public static final int AWARENESS = 0;
    public static final int BASIC_MILITARY_TRAINING = 1;
    public static final int ENDURANCE = 2;
    public static final int ENGINEERING = 3;
    public static final int EXPLOSIVES = 4;
    public static final int FIRST_AID = 5;
    public static final int HANDGUN = 6;
    public static final int HEAVY_WEAPONS = 7;
    public static final int LEADERSHIP = 8;
    public static final int LUCK = 9;
    public static final int MELEE = 10;
    public static final int MORALE = 11;
    public static final int RIFLE = 12;

    public static final int PRIVATE = 0;
    public static final int PFC = 2;
    public static final int CORPORAL = 3;
    public static final int SERGEANT = 5;
    public static final int LIEUTENANT = 7;
    public static final int CAPTAIN = 8;
    public static final int MAJOR = 9;
    
    public static final String [] defaultNames = { "Anderson", "Apone", "Barker", "Beckett", "Billy", 
    "Bishop", "Blain", "Brekken", "Brickley", "Bunsen", "Burch", "Campbell", "Carless", "Carrot",
    "Cole", "Cowboy", "Crowe", "Crusher", "Dietrich", "Dillon", "Diz", "Dodger", "Drake", "Dredd",
    "Dutch", "Fargo", "Ferro", "Fielding", "Flores", "Frost", "Garibaldi", "Giant", "Goldstock", "Gorman",
    "Grey", "Griffin", "Harris", "Hart", "Hawkins", "Hershey", "Hicks", "Hudson", "Hyers", "Ibanez",
    "Inglis", "Joker", "Jones", "Jonlan", "Keifer", "Kitten", "Klavs", "Koskela", "Kusanagi", "Lanier",
    "Levy", "Logan", "McGormick", "McMurphy", "McTavish", "Mears", "Monroe", "Nemique", "Nobbs",
    "O'Brien", "Ocks", "Ordellen", "Ozman", "Parker", "Pepper", "Picard", "Pryde", "Pucci", "Pyle",
    "Raczek", "Rahikainen", "Ramirez", "Ravenwing", "Reice", "Rico", "Ripley", "Rokka", "Shellan",
    "Sheridan", "Short", "Simpson", "Sinclair", "Silverstein", "Solo", "Spunkmeyer", "Steinburg",
    "Stone", "Summers", "Tenzer", "Turner", "Vasquez", "Vimes", "Watkins", "Watters", "Wierzbowski",
    "Winters", "Wycziezbsky", "Zim" };

    public static final String [] attributeNames = { "Awareness", "Basic military training", "Endurance", "Engineering", "Explosives", "First aid", "Handgun", "Heavy weapons", "Leadership", "Luck", "Melee", "Morale", "Rifle" };

    public static final String [] ranks = { "Private", "Private", "PFC", "Corporal", "Corporal", "Sergeant", "Sergeant", 
    "Lieutenant", "Captain", "Major" };

    public static final double HEALING_LIMIT = 0.7;

    protected int headIndex;
    protected int earsIndex;
    protected int noseIndex;
    protected int mouthIndex;
    protected int eyeIndex;
    protected int browsIndex;
    protected int extrasIndex;
    protected int hairIndex;
    protected int helmetIndex;
    protected int mouthXOffset;
    protected int mouthYOffset;
    protected int eyeOffset;
    protected int browsOffset;
    
    protected int missions = 0;
    protected int lastMissionKills = 0;
    protected int totalKills = 0;
    protected int rank = 0;
    protected int freeAttributePoints = 0;			// Extra "experience" that can be used for any attribute
    protected int [] attributeLevels = new int [ ATTRIBUTES ];	// The actual attribute level (0-10)
    protected int [] attributePoints = new int [ ATTRIBUTES ];	// "Experience points" for the attribute (next level*10 to raise a level)

    protected Weapon weapon = null;
    protected int hits = 0;
    protected int lastCommandValue = 0;
    protected boolean followedThisTurn = false;
    protected int leaderTurns = 0;
    protected boolean hasNotification = false;
    protected String rankPromotionString = "";
    protected Vector raisedAttributes = new Vector();

    public Marine()
    {
	location = new Point( 0, 0 );
	direction = Map.NORTH;
	alive = true;
	maximumActionPoints = 7;
	currentActionPoints = 7;
	maximumHitPoints = 10;
	currentHitPoints = 10;
	lastAction = new MoveAction( this, location, location );
    }

    public void verifyAttributes()
    {
        switch( attributeLevels[ ENDURANCE ] )
        {
            case 4:
            case 5:
                maximumHitPoints = 11;
                currentHitPoints = 11;
                break;
            case 6:
                maximumHitPoints = 12;
                currentHitPoints = 12;
                break;
            case 7:
                maximumHitPoints = 13;
                currentHitPoints = 13;
                break;
            case 8:
                maximumHitPoints = 15;
                currentHitPoints = 15;
                break;
            case 9:
                maximumHitPoints = 17;
                currentHitPoints = 17;
                break;
        }
        if( attributeLevels[ ENDURANCE ] >= 10 )
        {
            maximumHitPoints = 20;
            currentHitPoints = 20;
        }
    }
    
    public void setGame( Game game )
    {
        super.setGame( game );
        lastMissionKills = 0;
    }
    
    public GameAction doAction( GameAction action )
    {
        lastAction = action;
	if( action instanceof MoveAction )
	{
	    //System.out.println( "Soldier walking from " + location.toString() + " to " + target.toString() );
	    direction = Map.getDirection( new Point( location ), new Point( ((MoveAction)action).getEndPoint() ), direction );
	    //System.out.println( "Direction: " + direction );
	    location = ((MoveAction)action).getEndPoint();
            currentActionPoints-=((MoveAction)action).getCost();
	}
	if( action instanceof ShootAction )
	{
	    direction = Map.getDirection( new Point( location ), new Point( ((ShootAction)action).getEndPoint() ), direction );
            currentActionPoints-=((ShootAction)action).getCost();
	}
	if( action instanceof UseAction )
	{
	    direction = Map.getDirection( new Point( location ), new Point( ((UseAction)action).getPoint() ), direction );
            currentActionPoints-= ((UseAction)action).getCost();
	}
	if( action instanceof MeleeAction )
	{
	    MeleeAction mAction = (MeleeAction)action;
	    if( mAction.getAttacker() == this )
	    {
		currentActionPoints-=mAction.getCost();
	    }
	}
        return lastAction;
    }
    
    public boolean applyDamage( int damage )
    {
        lastAction = new WoundAction( this, location );
        int oldHitPoints = currentHitPoints;
	currentHitPoints = currentHitPoints-damage;
	if( currentHitPoints < 0 )
	{
            if( ( attributeLevels[ LUCK ] > 0 ) && ( currentHitPoints > -attributeLevels[ ENDURANCE ] ) )
            {
                attributeLevels[ LUCK ]--;
                currentHitPoints = oldHitPoints;
		activeGame.handleMessage( new Message( this, ranks[ rank ] + " " + name + " saved from death by sheer luck!" ) );
                System.out.println( ranks[ rank ] + " " + name + " saved from death by sheer luck!" );
            }
            else
            {
                alive = false;
                lastAction = new DeadAction( this, location );
		activeGame.handleMessage( new Message( this, ranks[ rank ] + " " + name + " is killed!" ) );
                System.out.println( ranks[ rank ] + " " + name + " is killed!" );
            }
	}
	return alive;
    }
    
    public void addKill()
    {
	lastMissionKills++;
	totalKills++;
    }

    public int getWeaponSkill()
    {
        return attributeLevels[ weapon.getRequiredAttribute() ];
    }

    public boolean useExperienceOn( int attribute, int wish )
    {
	int amount = wish;
        if( freeAttributePoints > 0 )
        {
	    if( freeAttributePoints < amount )
	    {
		amount = freeAttributePoints;
	    }
            attributePoints[ attribute ]+=amount;
            freeAttributePoints-=amount;
            return true;
        }
        return false;
    }

    public void grantExperienceTo( int attribute, int amount )
    {
        attributePoints[ attribute ]+=amount;
    }

    public void healDamage( int hitPoints )
    {
        int tempHitPoints = currentHitPoints;
        currentHitPoints+=hitPoints;
        if( currentHitPoints > (int)(maximumHitPoints*HEALING_LIMIT) )
        {
            //System.out.println( name + " can't be healed more" );
            currentHitPoints = Math.max( tempHitPoints, (int)(maximumHitPoints*0.7) );
        }
        /*
        else
        {
            System.out.println( name + " was healed" );
        }
         */
    }

    public boolean hasNotification()
    {
	return hasNotification;
    }
    
    public boolean leavesCorpse()
    {
        return false;
    }
    
    public void markAsLeader()
    {
        if( !followedThisTurn )
        {
            leaderTurns++;
            followedThisTurn = true;
        }
    }
    
    public void checkActionPoints()
    {
        int commandValue = 0;
        int testCommand = 0;
        Marine marine;
        Marine myLeader = null;
        followedThisTurn = false;
        if( currentHitPoints <= 6 )
        {
            // Wounded characters have less action points
            currentActionPoints-=2;
        }
        if( currentHitPoints <= 3 )
        {
            // And badly wounded have even less
            currentActionPoints = 3;
        }
        for( int i = 0;  i < activeGame.getPlayerCharacters().size(); i++ )
        {
            marine = (Marine)activeGame.getPlayerCharacter( i );
            if( marine != this )
            {
                if( activeGame.isCellVisibleToCharacter( this, marine.getLocation() ) )
                {
                    testCommand = marine.getAttribute( LEADERSHIP );
                    if( testCommand > commandValue && marine.getRank() >= rank )
                    {
                        commandValue = testCommand;
                        myLeader = marine;
                    }
                }
            }
        }
        if( myLeader != null )
        {
            myLeader.markAsLeader();
            System.out.println( name + " follows " + myLeader.getName() );
        }
        int tempCommand = commandValue;
        if( lastCommandValue > commandValue )
        {
            // Previous orders stay in memory for a while
            commandValue = lastCommandValue;
        }
        if( commandValue < 5 )
        {
            // Low leadership drops action points
            currentActionPoints--;
            if( commandValue < 3 )
            {
                // And really low drops the even further
                currentActionPoints--;
            }
            if( attributeLevels[ MORALE ] >= 4 )
            {
                // But good morale can raise them
                currentActionPoints++;
            }
        }
        if( attributeLevels[ MORALE ] >= 9 )
        {
            // ... Even giving bonus
            currentActionPoints++;
        }
        if( commandValue > 9 )
        {
            // Really good leadership will give further bonus
            currentActionPoints++;
        }
        if( attributeLevels[ BASIC_MILITARY_TRAINING ] >= 9 )
        {
            // As will good training
            currentActionPoints++;
        }
        // Weapons affect the action points, too
        currentActionPoints-=weapon.getMovementCost();
        // Minimum action points is still 3, though
        if( currentActionPoints < 3 )
        {
            currentActionPoints = 3;
        }
        lastCommandValue = tempCommand;
    }

    public void checkPromotions()
    {
        int oldRank = rank;
	rankPromotionString = "";
        if( rank == CAPTAIN )
        {
            if( attributeLevels[ LEADERSHIP ] >= 10 && attributeLevels[ MORALE ] >= 9 && attributeLevels[ LUCK ] >= 8 )
            {
                if( freeAttributePoints >= 20 && attributeLevels[ LUCK ] > (int)(Math.random()*20) )
                {
                    rank = MAJOR;
                    freeAttributePoints-=20;
                }
            }
        }
        if( rank >= LIEUTENANT && rank < CAPTAIN )
        {
            if( attributeLevels[ LEADERSHIP ] >= 9 && attributeLevels[ MORALE ] >= 8 && attributeLevels[ HANDGUN ] >= 7 )
            {
                if( freeAttributePoints >= 10 && attributeLevels[ LUCK ] > (int)(Math.random()*17) )
                {
                    rank = CAPTAIN;
                    freeAttributePoints-=10;
                }
            }
        }
        if( rank >= PFC && rank < LIEUTENANT )
        {
            if( attributeLevels[ MORALE ] >= 6 )
            {
                if( attributeLevels[ LEADERSHIP ] >= 7 || attributeLevels[ FIRST_AID ] >= 9 || attributeLevels[ ENGINEERING ] >= 9 )
                {
                    if( freeAttributePoints >= 10 && attributeLevels[ LUCK ] > (int)(Math.random()*15) )
                    {
                        rank = LIEUTENANT;
                        freeAttributePoints-=10;
                    }
                }

            }
            
        }
        if( rank >= CORPORAL && rank < SERGEANT )
        {
            if( attributeLevels[ LEADERSHIP ] >= 5 && attributeLevels[ BASIC_MILITARY_TRAINING ] >= 3 && attributeLevels[ MORALE ] >= 2 )
            {
                if( freeAttributePoints >= 10 && attributeLevels[ LUCK ] > (int)(Math.random()*12) )
                {
                    rank = SERGEANT;
                    freeAttributePoints-=10;
                }
            }
        }
        if( rank == PFC )
        {
            if( attributeLevels[ BASIC_MILITARY_TRAINING ] >= 3 && attributeLevels[ LEADERSHIP ] >= 4 )
            {
                if( freeAttributePoints >= 7 && attributeLevels[ LUCK ] > (int)(Math.random()*10) )
                {
                    rank = CORPORAL;
                    freeAttributePoints-=7;
                }
            }
        }
        if( rank < PFC )
        {
            if( attributeLevels[ BASIC_MILITARY_TRAINING ] >= 2 &&  attributeLevels[ MORALE ] >= 2 )
            {
                if( attributeLevels[ FIRST_AID ] >= 3 || attributeLevels[ RIFLE ] >= 2 || attributeLevels[ HEAVY_WEAPONS ] >= 2 )
                {
                    if( freeAttributePoints >= 5 && attributeLevels[ LUCK ] > (int)(Math.random()*10) )
                    {
                        rank = PFC;
                        freeAttributePoints -= 5;
                    }
                }
            }
        }
        if( rank > oldRank )
        {
	    hasNotification = true;
	    rankPromotionString = ranks[ oldRank ] + " " + name + " has been promoted to " + ranks[ rank ] + ".";
            System.out.println( ranks[ oldRank ] + " " + name + " has been promoted to " + ranks[ rank ] + "." );
        }
    }
    
    public void endMission( boolean success, int maxBonus )
    {
	hasNotification = false;
	raisedAttributes.clear();
	int bonusExperience = 0;
        if( alive )
        {
            missions++;
	    freeAttributePoints+=lastMissionKills/10;
            if( success )
            {
                int leadershipReward = leaderTurns/10;
                if( leadershipReward > 3 )
                {
                    leadershipReward = 3;
                }
                attributePoints[ LEADERSHIP ] += leadershipReward;
                if( currentHitPoints == maximumHitPoints )
                {
                    bonusExperience+=2;
                    attributePoints[ MORALE ] += 1+(int)(Math.random()*3);
                    attributePoints[ LUCK ] += 2;
                }
            }
            if( currentHitPoints > 9 )
            {
		bonusExperience+=2;
                attributePoints[ BASIC_MILITARY_TRAINING ] += 2;
            }
            if( currentHitPoints > 6 && success )
            {
		bonusExperience+=2;
                attributePoints[ LUCK ] += 1;
            }
            if( currentHitPoints < 4 )
            {
                attributePoints[ ENDURANCE ] += 2;
            }
            if( hits >= 10 )
            {
                attributePoints[ weapon.getRequiredAttribute() ]+=1;
            }
            if( hits >= 25 && success )
            {
                attributePoints[ weapon.getRequiredAttribute() ]+=3;
            }
	    if( success && bonusExperience < maxBonus )
	    {
		bonusExperience = maxBonus;
	    }
	    if( !success )
	    {
		bonusExperience = Math.min( bonusExperience, maxBonus );
	    }
	    freeAttributePoints += bonusExperience;
	}
    }
	
    public void checkExperience()
    {
	// Check for automatic raises
	for( int i=0; i < ATTRIBUTES; i++ )
	{
	    while( attributePoints[ i ] >= (attributeLevels[ i ]+1)*10 )
	    {
		hasNotification = true;
		attributePoints[ i ]-=(attributeLevels[ i ]+1)*10;
		attributeLevels[ i ]++;
		raisedAttributes.add( attributeNames[ i ] );
		System.out.println( ranks[ rank ] + " " + name + " advanced in " + attributeNames[ i ] );
	    }
	}
	// Check for promotions
	if( freeAttributePoints >= 5 )
	{
	    checkPromotions();
	}
        verifyAttributes();
    }

    public void registerHit()
    {
        hits++;
    }

    public void resetActionsPoints()
    {
        currentActionPoints = maximumActionPoints;
    }

    public void resetAmmunition()
    {
        if( weapon != null )
        {
            weapon.resetAmmunition();
        }
    }
    
    public static Marine load( DataInputStream in ) throws IOException
    {
        Marine marine = new Marine();
        String className = in.readUTF();
        if( className.equals( "Marine" ) )
        {
            int version = in.readInt();
            marine.name = in.readUTF();
            marine.headIndex = in.readInt();
            marine.earsIndex = in.readInt();
            marine.noseIndex = in.readInt();
            marine.mouthIndex = in.readInt();
            marine.eyeIndex = in.readInt();
            marine.browsIndex = in.readInt();
            marine.extrasIndex = in.readInt();
            marine.hairIndex = in.readInt();
            marine.helmetIndex = in.readInt();
            marine.mouthXOffset = in.readInt();
            marine.mouthYOffset = in.readInt();
            marine.eyeOffset = in.readInt();
            marine.browsOffset = in.readInt();
            marine.missions = in.readInt();
            marine.lastMissionKills = in.readInt();
            marine.totalKills = in.readInt();
            marine.rank = in.readInt();
            marine.freeAttributePoints = in.readInt();
            marine.attributeLevels[ AWARENESS ] = in.readInt();
            marine.attributePoints[ AWARENESS ] = in.readInt();
            marine.attributeLevels[ BASIC_MILITARY_TRAINING ] = in.readInt();
            marine.attributePoints[ BASIC_MILITARY_TRAINING ] = in.readInt();
            marine.attributeLevels[ ENDURANCE ] = in.readInt();
            marine.attributePoints[ ENDURANCE ] = in.readInt();
            marine.attributeLevels[ ENGINEERING ] = in.readInt();
            marine.attributePoints[ ENGINEERING ] = in.readInt();
            marine.attributeLevels[ EXPLOSIVES ] = in.readInt();
            marine.attributePoints[ EXPLOSIVES ] = in.readInt();
            marine.attributeLevels[ FIRST_AID ] = in.readInt();
            marine.attributePoints[ FIRST_AID ] = in.readInt();
            marine.attributeLevels[ HANDGUN ] = in.readInt();
            marine.attributePoints[ HANDGUN ] = in.readInt();
            marine.attributeLevels[ HEAVY_WEAPONS ] = in.readInt();
            marine.attributePoints[ HEAVY_WEAPONS ] = in.readInt();
            marine.attributeLevels[ LEADERSHIP ] = in.readInt();
            marine.attributePoints[ LEADERSHIP ] = in.readInt();
            marine.attributeLevels[ LUCK ] = in.readInt();
            marine.attributePoints[ LUCK ] = in.readInt();
            marine.attributeLevels[ MELEE ] = in.readInt();
            marine.attributePoints[ MELEE ] = in.readInt();
            marine.attributeLevels[ MORALE ] = in.readInt();
            marine.attributePoints[ MORALE ] = in.readInt();
            marine.attributeLevels[ RIFLE ] = in.readInt();
            marine.attributePoints[ RIFLE ] = in.readInt();
        }
        else
        {
            marine = null;
        }
        marine.verifyAttributes();
        return marine;
    }

    public static Marine decodeFromElement( Element marineElement ) throws Exception
    {
	Marine marine = new Marine();
	if( marineElement.getNodeName().equals( "marine" ) )
	{
	    marine.name = marineElement.getAttribute( "name" );
	    marine.missions = XMLUtils.getIntFromAttribute( marineElement, "missions" );
	    marine.lastMissionKills = XMLUtils.getIntFromAttribute( marineElement, "lastmissionkills" );
	    marine.totalKills = XMLUtils.getIntFromAttribute( marineElement, "totalkills" );
	    marine.rank = XMLUtils.getIntFromAttribute( marineElement, "rank" );
	    marine.freeAttributePoints = XMLUtils.getIntFromAttribute( marineElement, "freeattributepoints" );
	    Element appearanceElement = XMLUtils.getNamedChild( marineElement, "appearance" );
	    marine.headIndex = XMLUtils.getIntFromAttribute( appearanceElement, "headindex" );
	    marine.earsIndex = XMLUtils.getIntFromAttribute( appearanceElement, "earsindex" );
	    marine.noseIndex = XMLUtils.getIntFromAttribute( appearanceElement, "noseindex" );
	    marine.mouthIndex = XMLUtils.getIntFromAttribute( appearanceElement, "mouthindex" );
	    marine.eyeIndex = XMLUtils.getIntFromAttribute( appearanceElement, "eyeindex" );
	    marine.browsIndex = XMLUtils.getIntFromAttribute( appearanceElement, "browsindex" );
	    marine.extrasIndex = XMLUtils.getIntFromAttribute( appearanceElement, "extrasindex" );
	    marine.hairIndex = XMLUtils.getIntFromAttribute( appearanceElement, "hairindex" );
	    marine.helmetIndex = XMLUtils.getIntFromAttribute( appearanceElement, "helmetindex" );
	    marine.mouthXOffset = XMLUtils.getIntFromAttribute( appearanceElement, "mouthxoffset" );
	    marine.mouthYOffset = XMLUtils.getIntFromAttribute( appearanceElement, "mouthyoffset" );
	    marine.eyeOffset = XMLUtils.getIntFromAttribute( appearanceElement, "eyeoffset" );
	    marine.browsOffset = XMLUtils.getIntFromAttribute( appearanceElement, "browsoffset" );
	    Element [] attributeElements = XMLUtils.getNamedChildren( marineElement, "attribute" );
	    for( int i=0; i < attributeElements.length; i++ )
	    {
		String attrName = attributeElements[ i ].getAttribute( "name" );
		int level = XMLUtils.getIntFromAttribute( attributeElements[i], "level" );
		int points = XMLUtils.getIntFromAttribute( attributeElements[i], "points" );
		boolean found = false;
		for( int j=0; j < attributeNames.length && !found; j++ )
		{
		    if( attrName.equals( attributeNames[j] ) )
		    {
			found = true;
			marine.attributeLevels[j] = level;
			marine.attributePoints[j] = points;
		    }
		}
	    }
	}
	marine.verifyAttributes();
	return marine;
    }
    
    public Element encodeForDocument( Document document )
    {
	Element marineElement = document.createElement( "marine" );
	marineElement.setAttribute( "name", name );
	marineElement.setAttribute( "missions", String.valueOf( missions ) );
	marineElement.setAttribute( "lastmissionkills", String.valueOf( lastMissionKills ) );
	marineElement.setAttribute( "totalkills", String.valueOf( totalKills ) );
	marineElement.setAttribute( "rank", String.valueOf( rank ) );
	marineElement.setAttribute( "freeattributepoints", String.valueOf( freeAttributePoints ) );
	Element appearanceElement = document.createElement( "appearance" );
	appearanceElement.setAttribute( "headindex", String.valueOf( headIndex ) );
	appearanceElement.setAttribute( "earsindex", String.valueOf( earsIndex ) );
	appearanceElement.setAttribute( "noseindex", String.valueOf( noseIndex ) );
	appearanceElement.setAttribute( "mouthindex", String.valueOf( mouthIndex ) );
	appearanceElement.setAttribute( "eyeindex", String.valueOf( eyeIndex ) );
	appearanceElement.setAttribute( "browsindex", String.valueOf( browsIndex ) );
	appearanceElement.setAttribute( "extrasindex", String.valueOf( extrasIndex ) );
	appearanceElement.setAttribute( "hairindex", String.valueOf( hairIndex ) );
	appearanceElement.setAttribute( "helmetindex", String.valueOf( helmetIndex ) );
	appearanceElement.setAttribute( "mouthxoffset", String.valueOf( mouthXOffset ) );
	appearanceElement.setAttribute( "mouthyoffset", String.valueOf( mouthYOffset ) );
	appearanceElement.setAttribute( "eyeoffset", String.valueOf( eyeOffset ) );
	appearanceElement.setAttribute( "browsoffset", String.valueOf( browsOffset ) );
	marineElement.appendChild( document.createTextNode( "\n" ) );
	marineElement.appendChild( appearanceElement );
	for( int i=0; i < attributeLevels.length; i++ )
	{
	    Element attributeElement = document.createElement( "attribute" );
	    attributeElement.setAttribute( "name", attributeNames[i] );
	    attributeElement.setAttribute( "level", String.valueOf( attributeLevels[i] ) );
	    attributeElement.setAttribute( "points", String.valueOf( attributePoints[i] ) );
	    marineElement.appendChild( document.createTextNode( "\n" ) );
	    marineElement.appendChild( attributeElement );
	}
	return marineElement;
    }
    
    public void save( DataOutputStream out ) throws IOException
    {
        // Write the class identifier
        out.writeUTF( "Marine" );
        // Then, the version number
        out.writeInt( 1 );
        // Now the actual fun starts...
        out.writeUTF( name );
        out.writeInt( headIndex );
        out.writeInt( earsIndex );
        out.writeInt( noseIndex );
        out.writeInt( mouthIndex );
        out.writeInt( eyeIndex );
        out.writeInt( browsIndex );
        out.writeInt( extrasIndex );
        out.writeInt( hairIndex );
        out.writeInt( helmetIndex );
        out.writeInt( mouthXOffset );
        out.writeInt( mouthYOffset );
        out.writeInt( eyeOffset );
        out.writeInt( browsOffset );
        out.writeInt( missions );
        out.writeInt( lastMissionKills );
        out.writeInt( totalKills );
        out.writeInt( rank );
        out.writeInt( freeAttributePoints );
        out.writeInt( attributeLevels[ AWARENESS ] );
        out.writeInt( attributePoints[ AWARENESS ] );
        out.writeInt( attributeLevels[ BASIC_MILITARY_TRAINING ] );
        out.writeInt( attributePoints[ BASIC_MILITARY_TRAINING ] );
        out.writeInt( attributeLevels[ ENDURANCE ] );
        out.writeInt( attributePoints[ ENDURANCE ] );
        out.writeInt( attributeLevels[ ENGINEERING ] );
        out.writeInt( attributePoints[ ENGINEERING ] );
        out.writeInt( attributeLevels[ EXPLOSIVES ] );
        out.writeInt( attributePoints[ EXPLOSIVES ] );
        out.writeInt( attributeLevels[ FIRST_AID ] );
        out.writeInt( attributePoints[ FIRST_AID ] );
        out.writeInt( attributeLevels[ HANDGUN ] );
        out.writeInt( attributePoints[ HANDGUN ] );
        out.writeInt( attributeLevels[ HEAVY_WEAPONS ] );
        out.writeInt( attributePoints[ HEAVY_WEAPONS ] );
        out.writeInt( attributeLevels[ LEADERSHIP ] );
        out.writeInt( attributePoints[ LEADERSHIP ] );
        out.writeInt( attributeLevels[ LUCK ] );
        out.writeInt( attributePoints[ LUCK ] );
        out.writeInt( attributeLevels[ MELEE ] );
        out.writeInt( attributePoints[ MELEE ] );
        out.writeInt( attributeLevels[ MORALE ] );
        out.writeInt( attributePoints[ MORALE ] );
        out.writeInt( attributeLevels[ RIFLE ] );
        out.writeInt( attributePoints[ RIFLE ] );
        
    }

    public String toString()
    {
        String theString = ranks[ rank ];
        theString = theString + " " + name + " (";
        for( int i=0; i < ATTRIBUTES; i++ )
        {
            theString = theString + attributeNames[ i ] + ": " + attributeLevels[ i ] +" ";
        }
        theString = theString + ")";
        return theString;
    }

    public Weapon getWeapon()
    {
        return weapon;
    }

    public void setWeapon( Weapon weapon )
    {
        this.weapon = weapon;
    }
    
    public int getLastMissionKills()
    {
	return lastMissionKills;
    }
    
    public int getTotalKills()
    {
        return totalKills;
    }

    public int getFreeAttributePoints()
    {
        return freeAttributePoints;
    }

    public int getMeleeAbility()
    {
	return attributeLevels[ MELEE ];
    }
    
    public int getMissions()
    {
        return missions;
    }	
    
    public int getCurrentActionPoints()
    {
	return currentActionPoints;
    }
    
    public String getName()
    {
	return name;
    }


    public int getAttribute( int attribute )
    {
        return attributeLevels[ attribute ];
    }

    public int getAttributePoints( int attribute )
    {
        return attributePoints[ attribute ];
    }
    
    public String getAttributeAdvanceDescription()
    {
	String desc = "";
	if( raisedAttributes.size() == 1 )
	{
	    desc = ranks[ rank ] + " " + name + " has advanced in ";
	    desc += (String)raisedAttributes.elementAt( 0 );
	}
	if( raisedAttributes.size() > 1 )
	{
	    desc = ranks[ rank ] + " " + name + " has advanced in ";
	    for( int i = 0; i < raisedAttributes.size(); i++ )
	    {
		desc += (String)raisedAttributes.elementAt( i );
		if( i < (raisedAttributes.size() -1 ) )
		{
		    if( i < (raisedAttributes.size() - 2 ) )
		    {
			desc += ", ";
		    }   
		    else
		    {
			desc += " and ";
		    }
		}
	    }
	}
	return desc;
    }
    
    public int getRank()
    {
        return rank;
    }
    
    public String getRankPromotionDescription()
    {
	return rankPromotionString;
    }
    
    public int getEyeIndex()
    {
	return eyeIndex;
    }
    
    public int getNoseIndex()
    {
	return noseIndex;
    }
    
    public int getHeadIndex()
    {
	return headIndex;
    }
    
    public int getEarsIndex()
    {
	return earsIndex;
    }
    
    public int getMouthIndex()
    {
	return mouthIndex;
    }

    public int getExtrasIndex()
    {
	return extrasIndex;
    }

    public int getHairIndex()
    {
	return hairIndex;
    }

    public int getBrowsIndex()
    {
	return browsIndex;
    }


    public int getHelmetIndex()
    {
	return helmetIndex;
    }
    
    public int getMouthXOffset()
    {
	return mouthXOffset;
    }
    
    public int getMouthYOffset()
    {
	return mouthYOffset;
    }

    public int getEyeOffset()
    {
	return eyeOffset;
    }
     
    public int getBrowsOffset()
    {
	return browsOffset;
    }
   
    public int getSize()
    {
	return 75;
    }

}