// MarineCreationDialog.java
// Started on Augist 2nd, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class MarineCreationDialog extends JDialog implements MouseListener, ActionListener, DocumentListener
{
    private MarineStatisticsPanel marinePanel;
    private CreatorMarine marine;
    private Marine theMarine = null;
    private JTable attributesTable;
    private JTextField nameField;
    private JTextField pointField;
    private IconPanel leftIcons;
    private IconPanel rightIcons;
    private JButton okButton;
    private JButton cancelButton;

    public MarineCreationDialog( Frame owner )
    {
        super( owner, "New soldier", true );
        marine = new CreatorMarine();
        Box top = Box.createVerticalBox();
        Box contentBox = Box.createHorizontalBox();
        // Create the main attributes table
        TableModel tableModel = new AbstractTableModel() {
            public int getColumnCount() { return 4; }
            public int getRowCount() { return Marine.ATTRIBUTES; };
            public Object getValueAt( int row, int col )
            {
                if( col == 0 )
                {
                    return Marine.attributeNames[ row ];
                }
                if( col == 1 )
                {
                    if( marine != null )
                    {
                        return new Integer( marine.getAttribute( row ) );
                    }
                    else
                    {
                        return new Integer( 0 );
                    }
                }
                if( col == 2 )
                {
                    return "+";
                }
                if( col == 3 )
                {
                    return "-";
                }
                return "Undefined";
            }
        };
        attributesTable = new JTable( tableModel );
        // Set up the column widths
        TableColumn column;
        column = attributesTable.getColumnModel().getColumn( 0 );
        Component temp;
        // Get the renderer for the columns, so we can get the proper font metrics
        temp = attributesTable.getDefaultRenderer( attributesTable.getColumnClass(0) ).getTableCellRendererComponent( attributesTable, "Basic military training   ", false, false, 0, 0 );
        FontMetrics fm = temp.getFontMetrics( temp.getFont() );
        // Calculate the width for (probably) the longest attribute, with some extra padding
        int cellWidth = SwingUtilities.computeStringWidth( fm, "Basic military training   " );
        column.setMinWidth( cellWidth );
        column.setMaxWidth( cellWidth );
        column.setPreferredWidth( cellWidth );
        column = attributesTable.getColumnModel().getColumn( 1 );	// Attribute levels
        cellWidth = SwingUtilities.computeStringWidth( fm, "000" );
        column.setMinWidth( cellWidth );
        column.setMaxWidth( cellWidth );
        column.setPreferredWidth( cellWidth );
        column = attributesTable.getColumnModel().getColumn( 2 );	
        cellWidth = SwingUtilities.computeStringWidth( fm, " + " );
        column.setMinWidth( cellWidth );
        column.setMaxWidth( cellWidth );
        column.setPreferredWidth( cellWidth );
        column = attributesTable.getColumnModel().getColumn( 3 );	
        cellWidth = SwingUtilities.computeStringWidth( fm, " - " );
        column.setMinWidth( cellWidth );
        column.setMaxWidth( cellWidth );
        column.setPreferredWidth( cellWidth );
        
        attributesTable.removeEditor();
        attributesTable.setEnabled( false );
        attributesTable.addMouseListener( this );
        attributesTable.setCursor( Cursor.getPredefinedCursor( Cursor.HAND_CURSOR ) );
        attributesTable.setToolTipText( "Click + and - to add or remove levels to attribute" );

        contentBox.add( attributesTable );
        Box right = Box.createVerticalBox();
        JPanel tempPanel = new JPanel();
        JLabel nameLabel = new JLabel( "Name" );
        tempPanel.add( nameLabel );
        nameField = new JTextField( "", 12 );
        nameField.getDocument().addDocumentListener( this );
        tempPanel.add( nameField );
        right.add( tempPanel );

        Box faceBox = Box.createHorizontalBox();
        Image leftButtons = ResourceManager.loadImage( "FaceIconsLeft.png" );
        leftIcons = new IconPanel( leftButtons, 60, 165 );
        leftIcons.addMouseListener( this );
        faceBox.add( leftIcons );
        marinePanel = new MarineStatisticsPanel();
        marinePanel.setDrawFaceOnly( true );
        marinePanel.setMarine( (Marine)marine );
        faceBox.add( marinePanel );
        Image rightButtons = ResourceManager.loadImage( "FaceIconsRight.png" );
        rightIcons = new IconPanel( rightButtons, 25, 165 );
        rightIcons.addMouseListener( this );
        faceBox.add( rightIcons );
        right.add( faceBox );

        tempPanel = new JPanel();
        tempPanel.add( new JLabel( "Free points" ) );
        pointField = new JTextField( String.valueOf( marine.getFreeAttributeLevels() ),  5 );
        pointField.setEditable( false );
        tempPanel.add( pointField );
        right.add( tempPanel );
        
        contentBox.add( right );
        top.add( contentBox );
        Box buttonsPanel = Box.createHorizontalBox();
        buttonsPanel.add( Box.createHorizontalGlue() );
        cancelButton = new JButton( "Cancel" );
        cancelButton.addActionListener( this );
        buttonsPanel.add( cancelButton );
        okButton = new JButton( "OK" );
        okButton.setEnabled( false );
        okButton.addActionListener( this );
        buttonsPanel.add( okButton );
        top.add( buttonsPanel );

        getRootPane().setDefaultButton( okButton );
        getContentPane().add( top );
        pack();
    }

    public Marine getMarine()
    {
        return theMarine;
    }

    public void showNew()
    {
        okButton.setEnabled( false );
        theMarine = null;
        marine = new CreatorMarine();
        marinePanel.setMarine( (Marine)marine );
        nameField.setText( "" );
        pointField.setText( String.valueOf( marine.getFreeAttributeLevels() ) );
        setVisible( true );
    }
    
    public void actionPerformed( ActionEvent e )
    {
        if( e.getSource() == cancelButton )
        {
            setVisible( false );
        }
        if( e.getSource() == okButton )
        {
            marine.setName( nameField.getText().trim() );
            //theMarine = CreatorMarine.convert( marine );
	    theMarine = (Marine)marine;
            setVisible( false );
        }
    }
    
    public void mouseClicked( MouseEvent e )
    {
        if( e.getSource() == attributesTable )
        {
            if( marine != null )
            {
                int attribute = attributesTable.rowAtPoint( e.getPoint() );
                int column = attributesTable.columnAtPoint( e.getPoint() );
                System.out.println( "Clicked on " + Marine.attributeNames[ attribute ] );
                if( SwingUtilities.isRightMouseButton( e ) || column == 3 )
                {
                    marine.removePointsFromAttribute( attribute );
                }
                else
                {
                    marine.assignPointsToAttribute( attribute );
                }
                pointField.setText( String.valueOf( marine.getFreeAttributeLevels() ) );
                repaint();
            }
        }
        if( e.getSource() == leftIcons )
        {
	    System.out.println( "Cycle backwards" );
            Point point = e.getPoint();
            if( marine != null && point.x < 25 )
            {
                int y = point.y;
                if( y > 6 && y < 23 )
                {
                    marine.cycleHelmet( -1 );
                }
                if( y > 29 && y < 44 )
                {
                    marine.cycleHair( -1 );
                }
                if( y > 46 && y < 59 )
                {
                    marine.cycleBrows( -1 );
                }
                if( y > 60 && y < 74 )
                {
                    marine.cycleExtras( -1 );
                }
                if( y > 75 && y < 90 )
                {
                    marine.cycleEyes( -1 );
                }
                if( y > 91 && y < 106 )
                {
                    marine.cycleEars( -1 );
                }
                if( y > 107 && y < 122 )
                {
                    marine.cycleNose( -1 );
                }
                if( y > 123 && y < 137 )
                {
                    marine.cycleMouth( -1 );
                }
                if( y > 139 && y < 155 )
                {
                    marine.cycleHead( -1 );
                }
                marinePanel.repaint();
            }
        }
        if( e.getSource() == rightIcons )
        {
	    System.out.println( "Cycle forward" );
            Point point = e.getPoint();
            if( marine != null )
            {
                int y = point.y;
                if( y > 6 && y < 23 )
                {
                    marine.cycleHelmet( 1 );
                }
                if( y > 29 && y < 44 )
                {
                    marine.cycleHair( 1 );
                }
                if( y > 46 && y < 59 )
                {
                    marine.cycleBrows( 1 );
                }
                if( y > 60 && y < 74 )
                {
                    marine.cycleExtras( 1 );
                }
                if( y > 75 && y < 90 )
                {
                    marine.cycleEyes( 1 );
                }
                if( y > 91 && y < 106 )
                {
                    marine.cycleEars( 1 );
                }
                if( y > 107 && y < 122 )
                {
                    marine.cycleNose( 1 );
                }
                if( y > 123 && y < 137 )
                {
                    marine.cycleMouth( 1 );
                }
                if( y > 139 && y < 155 )
                {
                    marine.cycleHead( 1 );
                }
                marinePanel.repaint();
            }
            
        }
    }

    public void changedUpdate( DocumentEvent e )
    {
        if( nameField.getText().length() > 1 )
        {
            okButton.setEnabled( true );
        }
        else
        {
            okButton.setEnabled( false );
        }
    }

    public void insertUpdate( DocumentEvent e )
    {
        if( nameField.getText().length() > 1 )
        {
            okButton.setEnabled( true );
        }
        else
        {
            okButton.setEnabled( false );
        }
        
    }

    public void removeUpdate( DocumentEvent e )
    {
        if( nameField.getText().length() > 1 )
        {
            okButton.setEnabled( true );
        }
        else
        {
            okButton.setEnabled( false );
        }
        
    }

    public void mousePressed( MouseEvent e )
    {}

    public void mouseReleased( MouseEvent e )
    {}

    public void mouseEntered( MouseEvent e )
    {}

    public void mouseExited( MouseEvent e )
    {}

}

class IconPanel extends JPanel
{
    private Image image;
    private Dimension mySize;

    public IconPanel( Image image, int x, int y )
    {
        this.image = image;
        mySize = new Dimension( x, y );
        setMinimumSize( mySize );
        setMaximumSize( mySize );
        setPreferredSize( mySize );
    }

    public void paintComponent( Graphics g )
    {
        g.setColor( Color.darkGray );
        g.fillRect( 0, 0, mySize.width, mySize.height );
        g.drawImage( image, 0, 0, this );
    }
}