// MarineInfoPanel.java
// Started on June 15th, 2002 
// Copyright (c) Virtu Halttunen

package TheGame;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;

public class MarineInfoPanel extends JPanel implements MouseListener
{
    private MarineStatisticsPanel marinePanel;
    private Marine marine;
    private JTable attributesTable;
    private JTable statsTable;
    
    public MarineInfoPanel()
    {
        Box top = Box.createHorizontalBox();
        // Create the main attributes table
        TableModel tableModel = new AbstractTableModel() {
            public int getColumnCount() { return 4; }
            public int getRowCount() { return Marine.ATTRIBUTES; };
	    public String getColumnName( int col )
	    {
		if( col == 0 )
                {
                    return "Attribute";
                }
                if( col == 1 )
                {
		    return "L";
                }
                if( col == 2 )
                {
		    return "Exp";
                }
                if( col == 3 )
                {
                    return "";
                }
                return "";
	    }
	    
            public Object getValueAt( int row, int col )
            {
                if( col == 0 )
                {
                    return Marine.attributeNames[ row ];
                }
                if( col == 1 )
                {
                    if( marine != null )
                    {
                        return new Integer( marine.getAttribute( row ) );
                    }
                    else
                    {
                        return new Integer( 0 );
                    }
                }
                if( col == 2 )
                {
                    if( marine != null )
                    {
                        return new Integer( marine.getAttributePoints( row ) );
                    }
                    else
                    {
                        return new Integer( 0 );
                    }
                }
                if( col == 3 )
                {
                    return "+";
                }
                return "Undefined";
            }
        };
        attributesTable = new JTable( tableModel );
        // Set up the column widths
        TableColumn column;
        column = attributesTable.getColumnModel().getColumn( 0 );
        Component temp;
        // Get the renderer for the columns, so we can get the proper font metrics
        temp = attributesTable.getDefaultRenderer( attributesTable.getColumnClass(0) ).getTableCellRendererComponent( attributesTable, "Basic military training   ", false, false, 0, 0 );
        FontMetrics fm = temp.getFontMetrics( temp.getFont() );
        // Calculate the width for (probably) the longest attribute
        int cellWidth = SwingUtilities.computeStringWidth( fm, "Basic military training   " );
        column.setMinWidth( cellWidth );
        column.setMaxWidth( cellWidth );
        column.setPreferredWidth( cellWidth );
        column = attributesTable.getColumnModel().getColumn( 1 );	// Attribute levels
        cellWidth = SwingUtilities.computeStringWidth( fm, "000" );
        column.setMinWidth( cellWidth );
        column.setMaxWidth( cellWidth );
        column.setPreferredWidth( cellWidth );
        column = attributesTable.getColumnModel().getColumn( 2 );	// Experience points
        cellWidth = SwingUtilities.computeStringWidth( fm, "0000" );
        column.setMinWidth( cellWidth );
        column.setMaxWidth( cellWidth );
        column.setPreferredWidth( cellWidth );
        column = attributesTable.getColumnModel().getColumn( 3 );	// Plus sign
        cellWidth = SwingUtilities.computeStringWidth( fm, " + " );
        column.setMinWidth( cellWidth );
        column.setMaxWidth( cellWidth );
        column.setPreferredWidth( cellWidth );
        attributesTable.removeEditor();
        attributesTable.setEnabled( false );
        attributesTable.addMouseListener( this );
        attributesTable.setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
        attributesTable.setToolTipText( "Click to add experience to attribute" );
	attributesTable.getTableHeader().setReorderingAllowed( false );
	Box tableBox = Box.createVerticalBox();
	tableBox.add( attributesTable.getTableHeader() );
	tableBox.add( attributesTable );
        top.add( tableBox );
        Box right = Box.createVerticalBox();
        marinePanel = new MarineStatisticsPanel();
        right.add( marinePanel );
        // Create the smaller statistics table
        tableModel = new AbstractTableModel() {
            private String [] rows = { "Mission kills", "Total kills", "Missions", "Experience" };
            public int getColumnCount() { return 2; }
            public int getRowCount() { return 4; };
            public Object getValueAt( int row, int col )
            {
                if( col == 0 )
                {
                    return rows[ row ];
                }
                if( col == 1 )
                {
                    if( marine != null )
                    {
                        int value = 0;
                        switch( row )
                        {
                            case 0: value = marine.getLastMissionKills(); break;
                            case 1: value = marine.getTotalKills(); break;
                            case 2: value = marine.getMissions(); break;
                            case 3: value = marine.getFreeAttributePoints(); break;
                        }
                        return new Integer( value );
                    }
                    else
                    {
                        return new Integer( 0 );
                    }
                }
                return "Undefined";
            }
        };
        statsTable = new JTable( tableModel );
        column = statsTable.getColumnModel().getColumn( 0 );
        temp = statsTable.getDefaultRenderer( statsTable.getColumnClass(0) ).getTableCellRendererComponent( statsTable, "Mission kills   ", false, false, 0, 0 );
        fm = temp.getFontMetrics( temp.getFont() );
        cellWidth = SwingUtilities.computeStringWidth( fm, "Mission kills   " );
        column.setMinWidth( cellWidth );
        column.setMaxWidth( cellWidth );
        column.setPreferredWidth( cellWidth );
        column = statsTable.getColumnModel().getColumn( 1 );
        cellWidth = SwingUtilities.computeStringWidth( fm, "00000" );
        column.setMinWidth( cellWidth );
        column.setMaxWidth( cellWidth );
        column.setPreferredWidth( cellWidth );
        statsTable.removeEditor();
        statsTable.setEnabled( false );
        right.add( statsTable );
        top.add( right );
        add( top );
    }

    public void setMarine( Marine marine )
    {
        this.marine = marine;
        marinePanel.setMarine( marine );
	statsTable.setToolTipText( "Total kills: " + marine.getTotalKills() );
        repaint();
    }

    public Marine getMarine()
    {
        return marine;
    }

    public void mouseClicked( MouseEvent e )
    {
        if( e.getSource() == attributesTable )
        {
            if( marine != null )
            {
                int attribute = attributesTable.rowAtPoint( e.getPoint() );
                System.out.println( "Clicked on " + Marine.attributeNames[ attribute ] );
		int amount = 1;
		if( e.isShiftDown() )
		{
		    amount = 10;
		}
                marine.useExperienceOn( attribute, amount );
                repaint();
            }
        }
    }

    public void mousePressed( MouseEvent e )
    {}

    public void mouseReleased( MouseEvent e )
    {}

    public void mouseEntered( MouseEvent e )
    {}

    public void mouseExited( MouseEvent e )
    {}
    
}