// MineBehaviorObjective.java
// Started on July 6th, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;
import java.util.Vector;

public class MineBehaviorObjective extends Objective
{
    protected Game activeGame;
    
    public MineBehaviorObjective()
    {
    }

    public void setGame( Game activeGame )
    {
        this.activeGame = activeGame;
    }

    public int getNumberOfSprites()
    {
        return 0;
    }    
    
    public Sprite getSprite( int x, int y)
    {
        return null;
    }

    public String [] getPropertyNames()
    {
        return null;
    }

    public String getProperty( String key )
    {
        return null;
    }

    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
    }    
    
    public boolean isCompleted()
    {
        return true;
    }

    public int getSize()
    {
        return 0;
    }

    public boolean isLocatedIn( Point point )
    {
        int mines = activeGame.getMonsters().size();
        Monster monster;
        for( int i=0; i < mines; i++ )
        {
            monster = activeGame.getMonster( i );
            if( monster instanceof Mine )
            {
                if( monster.getLocation().equals( point ) )
                {
                    return true;
                }
            }
        }
        return false;
    }

    public boolean isLocatedIn( int x, int y )
    {
        return isLocatedIn( new Point( x, y ) );
    }

    public boolean isDestructible()
    {
        return false;
    }

    public boolean isShownOnMap()
    {
        return false;
    }
    
    public boolean isUsable()
    {
	return true;
    }

    public boolean handleDamage( Point point, int damage )
    {
        return false;
    }

    public boolean handleAction( GameAction action )
    {
        boolean handled = false;
	boolean messaged = false;
        Character character = action.getActor();
	if( character instanceof Mine )
	{
	    return false;
	}
        // Check move actions
        if( action instanceof MoveAction )
        {
            Point point = ((MoveAction)action).getEndPoint();
            Mine mine = null;
            // Does the move end next to a mine
            int mines = activeGame.getMonsters().size();
            Monster monster;
            for( int i=0; i < mines; i++ )
            {
                monster = activeGame.getMonster( i );
                if( monster instanceof Mine && monster.isAlive() )
                {
                    // Ok, we found a mine. 
                    mine = (Mine)monster;
                    //System.out.println( "Mine at " + mine.getLocation().toString() );
                    boolean setByPlayer = mine.isSetByPlayer();
		    Character check = activeGame.getPlayerCharacter( 0 );
                    //boolean steppedByPlayer = ( activeGame.isPlayerCharacter( character ) );
                    boolean steppedByPlayer = ( !character.isFoe( check ) );

                    //Now check whether it will be triggered by the poor character
                    if( mine.isArmed() )
                    {
                        if( setByPlayer && !steppedByPlayer )
                        {
                            if( activeGame.getMap().isNeighbor( mine.getLocation(), point ) )
                            {
                                // Ignore mines, obviously
                                if( ! ( character instanceof Mine ) )
                                {
                                    // It goes boom
                                    handled = true;
                                    mine.setGoingBoom();
                                }
                            }
                        }
                        if( !setByPlayer && steppedByPlayer && ! (character instanceof Mine ) )
                        {
			    if( mine.getLocation().equals( point ) )
			    {
				mine.setDiscovered( true );
				mine.setGoingBoom();
				System.out.println( character.getType() + " stepped on mine" );
				handled = true;
			    }
                            if( ! handled && activeGame.getMap().isNeighbor( mine.getLocation(), point ) )
                            {
                                if( activeGame.isPlayerCharacter( character ) )
                                {
                                    Marine marine = (Marine)character;
                                    //System.out.println( marine.getName() + " stepped on mine!" );
                                    if( ( marine.getAttribute( Marine.LUCK ) > (int)(Math.random()*15) ) || ( marine.getAttribute( Marine.EXPLOSIVES ) > (int)(Math.random()*10) ) )
                                    {
					activeGame.handleMessage( new Message( marine, marine.getName() + " evaded mine!" ) );
                                        System.out.println( marine.getName() + " evaded mine!" );
					messaged = true;
                                    }
                                    else
                                    {
                                        // It goes boom
                                        handled = true;
					System.out.println( character.getType() + " stepped on mine" );
                                        mine.setGoingBoom();
                                    }
                                }
                            }
                        }
                    }
                    // Ok, how about seeing it?
                    if( character instanceof Marine && !setByPlayer )
                    {
                        Marine marine = (Marine)character;
                        if( activeGame.getMap().hasLineOfSight( mine.getLocation(), point ) && !mine.isDiscovered() )
                        {
                            // Can see the spot, but is it close enough?
                            int distance = (int)mine.getLocation().distance( point );
                            //if( distance <= 10 )
                            //{
                                //System.out.println( marine.getName() + " is close to a mine" );
                            //}
                            int skill = Math.max( marine.getAttribute( Marine.AWARENESS ), marine.getAttribute( Marine.BASIC_MILITARY_TRAINING )/2 );
                            if( skill+(int)(Math.random()*10) >= distance+6 )
                            {
				Room room = activeGame.getMap().getRoomForPoint( mine.getLocation() );
				String message = marine.getName() + " spotted a mine";
				if( room != null )
				{
				    message = message + " in " + room.getName();
				}
				message = message + "!";
				if( !messaged )
				{
				    activeGame.handleMessage( new Message( marine, message ) );
				    int reward = Math.max( 0, 2-(int)(Math.random()*4) );
				    marine.grantExperienceTo( Marine.AWARENESS, reward );
				    messaged = true;
				}
                                System.out.println( message );
                                handled = true;
                                mine.setDiscovered( true );
			    }
                        }
                    }
                }
            }
        }
        if( action instanceof UseAction )
        {
            Point point = ((UseAction)action).getPoint();
            int mines = activeGame.getMonsters().size();
            Monster monster;
            Mine mine = null;
            for( int i=0; i < mines; i++ )
            {
                monster = activeGame.getMonster( i );
                if( monster instanceof Mine && monster.isAlive() )
                {
                    // Ok, we found a mine.
                    mine = (Mine)monster;
                    boolean armed = mine.isArmed();
                    if( mine.getLocation().equals( point ) )
                    {
                        if( character instanceof Marine )
                        {
                            Marine marine = (Marine)character;
                            if( marine.getCurrentActionPoints() >= 3 )
                            {
                                ((UseAction)action).setCost( 3 );
				((UseAction)action).setConsumed( true );
                                handled = true;
                                int skill = Math.max( marine.getAttribute( Marine.EXPLOSIVES ), marine.getAttribute( Marine.BASIC_MILITARY_TRAINING )/2 );
                                if( skill+(int)(Math.random()*10) >= 10 )
                                {
                                    String desc = "armed";
                                    if( armed )
                                    {
                                        desc = "disarmed";
                                    }
				    activeGame.handleMessage( new Message( marine, marine.getName() + " " + desc + " a mine!" ) );
                                    System.out.println( marine.getName() + " " + desc + " a mine!" );
                                    mine.setArmed( !armed );
                                    mine.setSetByPlayer( true );
                                    mine.setDiscovered( true );
                                    int reward = Math.max( 0, 2-(int)(Math.random()*4) );
                                    marine.grantExperienceTo( Marine.EXPLOSIVES, reward );
                                }
				else
				{
				    String desc = "arm";
                                    if( armed )
                                    {
                                        desc = "disarm";
                                    }
				    activeGame.handleMessage( new Message( marine, marine.getName() + " failed to " + desc + " a mine" ) );				    
				}
                            }
			    else
			    {
				activeGame.handleOutOfAP( marine );
			    }
                        }
                    }
                }
            }
                    
        }
        return handled;
    }
}