// Mission.java
// Started on July 27th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.awt.Point;
import javax.xml.parsers.*;
import org.w3c.dom.*;

public class Mission
{
    public static final String [] missionDebriefNames = { "Complete victory", "Victory", "Failure", "Complete failure" };

    public static final String [] baseStart = { "Neptune", "Lunar", "Acheron", "Centauri", "Pluto", "Martian", "Europa", "Remote", "Abandoned", "Important", "Katratzi" };
    public static final String [] baseMiddle = { "research base", "Observatory", "outpost", "Colony", "refinery station", "mining colony", "rehabilitation center", "moonbase", "factory complex", "detention block", "space dock" };
    public static final String [] baseEnd = { "1A", "2B", "3D", "III", "IV", "51", "52", "7", "Alpha", "Beta", "Gamma", "LV-426", "TLH-858", "TK-421", "THX-1138" };
    
    public static final int COMPLETE_VICTORY = 0;
    public static final int VICTORY = 1;
    public static final int FAILURE = 2;
    public static final int COMPLETE_FAILURE = 3;
    
    private Vector playerStartPoints;
    private Vector monsters;
    private Vector objectives;
    private Objective primaryObjective = null;
    private Objective secondaryObjective = null;
    private String missionName;
    private String longMissionDescription;
    private String [] missionDebriefDescriptions = { "Complete victory", "Victory", "Failure", "Complete failure" };
    private String [] nextMissionFileNames = { null, null, null, null };
    private boolean [] canChangeMissionTeamAfter = { true, true, true, true };
    private int [] extraExperienceAfter = { 9, 6, 4, 2 };
    private boolean hasPrequisite = false;
    private boolean endsAtBase = false;
    private String missionFileName = null;
    
    private Map map = null;
    
    public Mission()
    {
	playerStartPoints = new Vector();
	monsters = new Vector();
	objectives = new Vector();
    }
    
    public void addMonster( Character monster )
    {
	monsters.add( monster );
    }
    
    public String addObjective( Objective objective )
    {
	String id = null;
	if( objective.getId() == null )
	{
	    objective.setId( String.valueOf( objectives.size() ) );
	}
	objectives.add( objective );
	return objective.getId();
    }
        
    private void decodeCondition( Element victoryElement )
    {
	String conditionName = victoryElement.getAttribute( "condition" );
	int condition = 0;
	for( int i = 0; i < missionDebriefNames.length; i++ )
	{
	    if( conditionName.equals( missionDebriefNames[ i ] ) )
	    {
		condition = i;
	    }
	}
	String nextMissionName = victoryElement.getAttribute( "nextmission" );
	if( nextMissionName.equalsIgnoreCase( "Null" ) || nextMissionName.trim().length() < 1 )
	{
	    nextMissionName = null;
	}
	nextMissionFileNames[ condition ] = nextMissionName;
	canChangeMissionTeamAfter[ condition ] = new Boolean( victoryElement.getAttribute( "canchangeteamafter" ) ).booleanValue();
	extraExperienceAfter[ condition ] = Integer.parseInt( victoryElement.getAttribute( "extraexperienceafter" ) );
	
	String debrief = victoryElement.getAttribute( "debrief" );
	missionDebriefDescriptions[ condition ] = debrief;
    }
    
    public boolean [] getCanChangeMissionTeamAfter()
    {
	return canChangeMissionTeamAfter;
    }
    
    public boolean getEndsAtBase()
    {
	return endsAtBase;
    }
    
    public boolean getHasPrequisite()
    {
	return hasPrequisite;
    }
    
    public Vector getPlayerStartPoints()
    {
	return playerStartPoints;
    }
    
    public String [] getDebriefMessages()
    {
	return missionDebriefDescriptions;
    }
    
    public int [] getMissionBonusPoints()
    {
	return extraExperienceAfter;
    }
    
    public String getDescription()
    {
	return longMissionDescription;
    }
    
    public String getMissionDescription()
    {
        String missionDesc = longMissionDescription;
        if( missionDesc.indexOf( "$CLIP-" ) > -1 )
        {
	    //System.out.println( "Clipping: " + missionDesc );
	    int clipStart = missionDesc.indexOf( "$CLIP-" );
	    int clipEnd = missionDesc.indexOf( "$", clipStart+1 );
	    //System.out.println( "Range: " + clipStart + " to " + clipEnd );
	    String id = missionDesc.substring( clipStart+6, clipEnd );
	    //System.out.println( "Trying to clip id: " + id );
	    String clipContent = "";
	    Objective objective = getObjectiveById( id );
	    if( objective != null )
	    {
		clipContent = objective.getDescriptionClip();
		//System.out.println( "Clip content: " + clipContent );
	    }
	    //String clipString = "$CLIP-"+id+"$";
            //missionDesc = missionDesc.substring( 0, clipStart ) + clipContent + missionDesc.substring( clipEnd+1 );
	    missionDesc = missionDesc.replaceAll( "\\$CLIP-"+id+"\\$", clipContent );
        }
        return missionDesc;
    }
    
    public String getMissionFileName()
    {
	return missionFileName;
    }
    
    public Map getMap()
    {
	return map;
    }
    
    public String [] getNextMissionFileNames()
    {
	return nextMissionFileNames;
    }
    
    public Vector getMonsters()
    {
	return monsters;
    }
    
    public String getName()
    {
	return missionName;
    }
    
    public Vector getObjectives()
    {
	return objectives;
    }
    
    public Objective getObjectiveById( String id )
    {
	if( id.equalsIgnoreCase( "Primary" ) )
	{
	    return primaryObjective;
	}
	if( id.equalsIgnoreCase( "Secondary" ) )
	{
	    return secondaryObjective;
	}
	try
	{
	    for( int i=0; i < objectives.size(); i++ )
	    {
		String theId = ((Objective)objectives.elementAt( i )).getId();
		if( theId != null && theId.equals( id ) )
		{
		    return (Objective)objectives.elementAt( i );
		}
	    }
	}
	catch( NumberFormatException nfe )
	{}
	System.err.println( "No match for objective id: " + id );
	return null;
    }    
    
    public Objective getPrimaryObjective()
    {
	return primaryObjective;
    }
    
    public Objective getSecondaryObjective()
    {
	return secondaryObjective;
    }
    
    public boolean hasFollowUp()
    {
	boolean followUp = false;
	for( int i=0; i < nextMissionFileNames.length; i++ )
	{
	    if( nextMissionFileNames[ i ] != null )
	    {
		if( !( nextMissionFileNames[ i ].equalsIgnoreCase( "End" ) || nextMissionFileNames[ i ].equalsIgnoreCase( "Null" ) ) )
		{
		    System.out.println( "Mission: " + missionName + " has follow-up: " + nextMissionFileNames[i] ); 
		    followUp = true;
		}
	    }
	}
	return followUp;
    }
    
    public static Mission openMissionFromFile( File file ) throws Exception
    {
	Mission mission = new Mission();
	if( file.exists() && file.canRead() )
	{
	    DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
	    DOMImplementation builder = documentBuilder.getDOMImplementation();
	    
	    FileInputStream fis = new FileInputStream( file );
	    GZIPInputStream gis = new GZIPInputStream( fis );
	    
	    System.out.println( "Parsing mission file: " + file.getName() );
	    mission.missionFileName = file.getName();
	    Document document = documentBuilder.parse( gis );
	    gis.close();
	    fis.close();
	    
	    Element root = document.getDocumentElement();
	    
	    mission.missionName = root.getAttribute( "name" );
	    mission.hasPrequisite = new Boolean( root.getAttribute( "hasprequisite" ) ).booleanValue();
	    mission.endsAtBase = new Boolean( root.getAttribute( "endsatbase" ) ).booleanValue();

	    String mapFileName = root.getAttribute( "mapfile" );
	    try
	    {
		File mapFile = new File( file.getParent(), mapFileName );
		mission.map = Map.load( mapFile );
	    }
	    catch( FileNotFoundException e )
	    {
		System.out.println( "Map file not found!" );
		return null;
	    }
	    catch( IOException e )
	    {
		System.out.println( e.toString() );
		return null;
	    }
	    catch( Exception e )
	    {
		e.printStackTrace();
		return null;
	    }
	    // Load player start points
	    Element playersElement = XMLUtils.getNamedChild( root, "players" );
	    if( playersElement != null )
	    {
		Element [] playerElements = XMLUtils.getNamedChildren( playersElement, "startpoint" );
		int x, y, dir;
		for( int i = 0; i < playerElements.length; i++ )
		{
		    x = Integer.parseInt( playerElements[i].getAttribute( "x" ) );
		    y = Integer.parseInt( playerElements[i].getAttribute( "y" ) );
		    dir = Integer.parseInt( playerElements[i].getAttribute( "direction" ) );
		    mission.playerStartPoints.add( new MapNode( x, y, dir ) );
		}
	    }
	    // Load monsters
	    Element monstersElement = XMLUtils.getNamedChild( root, "monsters" );
	    Monster monster;
	    if( monstersElement != null )
	    {
		/// Load actual monsters
		String monsterClass;
		int x, y, dir;
		Element [] monsterElements = XMLUtils.getNamedChildren( monstersElement, "monster" );
		for( int i = 0; i < monsterElements.length; i++ )
		{
		    monsterClass = monsterElements[i].getAttribute( "class" );
		    x = Integer.parseInt( monsterElements[i].getAttribute( "x" ) );
		    y = Integer.parseInt( monsterElements[i].getAttribute( "y" ) );
		    dir = Integer.parseInt( monsterElements[i].getAttribute( "direction" ) );
		    if( !monsterClass.startsWith( "TheGame." ) )
		    {
			monsterClass = "TheGame."+monsterClass;
		    }
		    if( monsterClass.endsWith( GenericMonster.GENERIC_MONSTER_KEY ) )
		    {
			monster = GenericMonster.create( monsterClass.substring( 8, monsterClass.indexOf( "(" ) ) );
		    }
		    else
		    {
			monster = (Monster)Class.forName( monsterClass ).newInstance();
		    }
		    monster.setLocation( new Point( x, y ) );
		    monster.setDirection( dir );
		    String monsterName = monsterElements[i].getAttribute( "name" );
		    if( monsterName != null )
		    {
			monster.setName( monsterName );
		    }
		    String id = monsterElements[i].getAttribute( "id" );
		    if( id == null || id.trim().length() < 1 )
		    {
			id = String.valueOf( i );
		    }
		    monster.setId( id );
		    
		    String monsterCustomHP = monsterElements[i].getAttribute( "customhitpoints" );
		    if( monsterCustomHP != null )
		    {
			monster.setCustomHitPoints( monsterCustomHP );
		    }
		    String monsterCustomAP = monsterElements[i].getAttribute( "customactionpoints" );
		    if( monsterCustomAP != null )
		    {
			monster.setCustomActionPoints( monsterCustomAP );
		    }
		    monster.setDormant( new Boolean( monsterElements[i].getAttribute( "dormant" ) ).booleanValue() );

		    mission.addMonster( monster );
		}
		/// Load random bounds
		Element [] randomBoundsElements = XMLUtils.getNamedChildren( monstersElement, "randommonsterbounds" );
		for( int i=0; i < randomBoundsElements.length; i++ )
		{
		    monsterClass = randomBoundsElements[i].getAttribute( "class" );
		    int min = Integer.parseInt( randomBoundsElements[i].getAttribute( "minimum" ) );
		    int max = Integer.parseInt( randomBoundsElements[i].getAttribute( "maximum" ) );
		    if( monsterClass.equals( "Marine" ) )
                    {
                        mission.playerStartPoints.add( new MapNode( -1, -1, 0 ) );
                    }
		    else
		    {
			if( !monsterClass.startsWith( "TheGame." ) )
			{
			    monsterClass = "TheGame."+monsterClass;
			}
			for( int m = min; m <= max; m++ )
			{
			    if( monsterClass.endsWith( GenericMonster.GENERIC_MONSTER_KEY ) )
			    {
				monster = GenericMonster.create( monsterClass.substring( 8, monsterClass.indexOf( "(" ) ) );
			    }
			    else
			    {
				monster = (Monster)Class.forName( monsterClass ).newInstance();
			    }
			    monster.setLocation( new Point( -1, -1 ) );
			    monster.setDirection( Map.NORTH );
			    mission.addMonster( monster );
			}
		    }
		}
	    }
	    // Load the Objectives
	    Element objectivesElement = XMLUtils.getNamedChild( root, "objectives" );
	    if( objectivesElement != null )
	    {
		Element primaryObjectiveElement = XMLUtils.getNamedChild( objectivesElement, "primaryobjective" );
		if( primaryObjectiveElement != null )
		{
		    mission.primaryObjective = Objective.decodeFromElement( XMLUtils.getNamedChild( primaryObjectiveElement, "objective" ) );
		}
		Element secondaryObjectiveElement = XMLUtils.getNamedChild( objectivesElement, "secondaryobjective" );
		if( secondaryObjectiveElement != null )
		{
		    mission.secondaryObjective = Objective.decodeFromElement( XMLUtils.getNamedChild( secondaryObjectiveElement, "objective" ) );
		}

		Element [] objectiveElements = XMLUtils.getNamedChildren( objectivesElement, "objective" );
		for( int i=0; i < objectiveElements.length; i++ )
		{
		    Objective objective = Objective.decodeFromElement( objectiveElements[i] );
		    mission.addObjective( objective );
		}
	    }
	    // Load mission end conditions and descriptions
	    Element longDescriptionElement = XMLUtils.getNamedChild( root, "description" );
	    if( longDescriptionElement != null )
	    {
		mission.longMissionDescription = longDescriptionElement.getAttribute( "description" );
	    }
	    Element conditionsElement = XMLUtils.getNamedChild( root, "endconditions" );
	    if( conditionsElement != null )
	    {
		Element [] endConditionElements = XMLUtils.getNamedChildren( conditionsElement, "victorycondition" );
		for( int i=0; i < endConditionElements.length; i++ )
		{
		    mission.decodeCondition( endConditionElements[i] );
		}
	    }
	}
	return mission;
    }
    
    public void randomizeObjectives()
    {
        Objective objective;
	String id;
	boolean hasReturn = false;
	boolean hasMines = false;
        int missionType = (int)(Math.random()*23);
	//missionType = 21;
        switch( missionType )
        {
            case 0:
                primaryObjective = new SecureAreaObjective();
		longMissionDescription = "Secure the area. Kill everything that moves. \nHell, kill everything that doesn't, just to be sure.\n";
		missionDebriefDescriptions[ VICTORY ] = "Area secured.";
		missionDebriefDescriptions[ FAILURE ] = "You failed to secure the area.";
                System.out.println( "Secure" );
            case 1:
                primaryObjective = new RecoverObjective();
		primaryObjective.setProperty( RecoverObjective.IS_SHOWN_ON_MAP_KEY, "False" );
		longMissionDescription = "A very important cybernetically preserved brain has been lost\nin the alien infiltration of " + getRandomBaseName() + ".\nYou must recover the brain intact.\n";
 		missionDebriefDescriptions[ VICTORY ] = "The brain has been rescued.";
		missionDebriefDescriptions[ FAILURE ] = "You failed to rescue the brain.";
		System.out.println( "Recover" );
                break;
            case 2:
            case 3:
                primaryObjective = new SabotageCommObjective();
		longMissionDescription= "The aliens have set up a communications device in " + getRandomBaseName() + ".\nYou must sabotage the device.\nDon't let the aliens alarm reinforcements.\n";
		missionDebriefDescriptions[ VICTORY ] = "The communications device has been destroyed.\n";
		missionDebriefDescriptions[ FAILURE ] = "The aliens managed to call reinforcements.\nMission failed.\n";
		extraExperienceAfter[ VICTORY ] = 10;
		endsAtBase = true;
		hasReturn = true;
		System.out.println( "Sabotage" );
                break;
            case 4:
            case 5:
                primaryObjective = new RecoverObjective();
		primaryObjective.setProperty( RecoverObjective.IS_SHOWN_ON_MAP_KEY, "False" );
 		missionDebriefDescriptions[ VICTORY ] = "The brain has been rescued.";
		missionDebriefDescriptions[ FAILURE ] = "You have failed to rescue the brain.";
		objective = new RipperSpawnObjective();
		id = addObjective( objective );
		String type = "Ripper";
		int switcher = (int)(Math.random()*4);
		switch( switcher )
		{
		    case 0:
			type = "Brute";
			break;
		    case 1:
			type = "Slasher";
			break;
		}
		objective.setProperty( RipperSpawnObjective.MONSTER_CLASSNAME_KEY, type );
		longMissionDescription = "A very important cybernetically preserved brain has been lost\nin the alien infiltration of " + getRandomBaseName() + ".\nYou must recover the brain intact.\nThe $CLIP-" + id + "$s have found a secret way into the complex.\nDon't waste time.\n";
                endsAtBase = true;
		hasReturn = true;
                System.out.println( "Recover from RipperSpawn" );
                break;
            case 6:
            case 7:
                primaryObjective = new ReleaseLavaObjective();
		longMissionDescription = "The aliens have captured " + getRandomBaseName() + ".\nLocate the control valve of the volcanic energy converter and release lava into the base.\nBeware! You will lose all power, including lights and door controls.\n";
		endsAtBase = true;
		hasReturn = true;
		missionDebriefDescriptions[ COMPLETE_VICTORY ] = "The base has been purified.\n$CLIP-Secondary$";
		missionDebriefDescriptions[ VICTORY ] = "The base has been purified.\nBut evacuating the base first would have been a good idea.\n$CLIP-Secondary$";
		missionDebriefDescriptions[ FAILURE ] = "You have failed to purify the base.";
		missionDebriefDescriptions[ COMPLETE_FAILURE ] = "You have failed to purify the base.\nAnd the civilians are dead, too.";
		objective = new RescueCiviliansObjective();
                objective.setProperty( "Bobs", "10" );
                objective.setProperty( "Must save", "5" );
                objective.setProperty( "Show bobs", "False" );
                secondaryObjective = objective;
                System.out.println( "Purify" );
                break;
            case 8:
            case 9:
                primaryObjective = new RescueCiviliansObjective();
		longMissionDescription = "There are civilians trapped in " + getRandomBaseName() + ".\nRescue them\n";
		missionDebriefDescriptions[ VICTORY ] = "$CLIP-Primary$";
		missionDebriefDescriptions[ FAILURE ] = "$CLIP-Primary$";
		extraExperienceAfter[ VICTORY ] = 10;
                primaryObjective.setProperty( "Bobs", "20" );
                primaryObjective.setProperty( "Must save", "5" );
		primaryObjective.setProperty( "Show bobs", "True" );
                System.out.println( "Rescue civilians" );
                break;
            case 10:
            case 11:
                primaryObjective = new BobBQueObjective();
		longMissionDescription = "There are $CLIP-Primary$ masquerading as civilians in " + getRandomBaseName() + ".\nTerminate them.\n";
		missionDebriefDescriptions[ VICTORY ] = "The $CLIP-Primary$ have been eliminated.";
		missionDebriefDescriptions[ FAILURE ] = "You have failed to eliminate all $CLIP-Primary$.";		
                System.out.println( "Bob hunting" );
                break;
            case 12:
            case 13:
                primaryObjective = new RecoverDataObjective();
		longMissionDescription = "Locate the computer terminal in the complex and\nretrieve the $CLIP-Primary$ before the aliens get them.\nDestroy the terminal after you get the data.\n";
		missionDebriefDescriptions[ VICTORY ] = "You have recovered the $CLIP-Primary$.";
		missionDebriefDescriptions[ FAILURE ] = "You have failed to secure the $CLIP-Primary$.";	
                System.out.println( "Data rescue" );
                endsAtBase = true;
		hasReturn = true;
		break;
            case 14:
            case 15:
                primaryObjective = new DisarmMinesObjective();
		longMissionDescription = "Disarm or destroy all the mines in " + getRandomBaseName() + "\n";
		missionDebriefDescriptions[ VICTORY ] = "All the mines have been cleared.";
		missionDebriefDescriptions[ FAILURE ] = "You have failed to clear all the mines.";
                System.out.println( "Disarm mines" );
                endsAtBase = true;
		hasReturn = true;
		hasMines = true;
                break;
            case 16:
                primaryObjective = new SurvivalObjective();
		longMissionDescription = "The base is lost! Hold your position until reinforcements arrive.\n";
		missionDebriefDescriptions[ VICTORY ] = "The cavalry has arrived!";
		missionDebriefDescriptions[ FAILURE ] = "Mission failed.";
		extraExperienceAfter[ VICTORY ] = 10;
		hasReturn = true;
                System.out.println( "Survival" );
                break;
	    case 17:
	    case 18:
		primaryObjective = new CollectTribblesObjective();
		int tribbleCount = 10+(int)(Math.random()*30);
		primaryObjective.setProperty( CollectTribblesObjective.TRIBBLES_KEY, String.valueOf( tribbleCount) );
		longMissionDescription = Mission.getRandomBaseName() + " has been infected with strange furry vermins.\nGo in and capture at least five specimens of the critters for further studies.";
		missionDebriefDescriptions[ VICTORY ] = "You have collected $CLIP-Primary$ critters.\nGood job!";
		missionDebriefDescriptions[ FAILURE ] = "You only got $CLIP-Primary$ critters.\nMission failed.";
		System.out.println( "Tribbles" );
		break;
	    case 19:
	    case 20:
		primaryObjective = new VipidiotObjective();
		endsAtBase=true;
		hasReturn=true;
		longMissionDescription = "Military consultant " + primaryObjective.getDescriptionClip() + " wants to examine " + getRandomBaseName() + ".\nProceed along and protect his safety.";
		missionDebriefDescriptions[ VICTORY ] = "The base has been examined.";
		missionDebriefDescriptions[ FAILURE ] = "$CLIP-Primary$ has died.\nMission failed.";
		System.out.println( "Protect vipidiot" );
		break;
	    case 21:
		objective = new VipidiotObjective();
		objective.setProperty( VipidiotObjective.START_POINT_KEY, "(-1, -1)" );
		objective.setProperty( VipidiotObjective.END_POINT_KEY, "(-1, -1)" );
		objective.setProperty( VipidiotObjective.TRAITOR_KEY, "True" );
		int points = 3+(int)(Math.random()*4);
		for( int i=0; i < points; i++ )
		{
		    objective.setProperty( VipidiotObjective.ADD_POINT_KEY, "(-1, -1)" );
		}
		id = addObjective( objective );
		primaryObjective = new AssassinateObjective();
		primaryObjective.setProperty( AssassinateObjective.TARGET_ID_KEY, "VIP-"+id );
		longMissionDescription = "Intelligence reports say that $CLIP-" + id + "$,\nrecently uncovered enemy spy, is hiding in " + getRandomBaseName() + ".\nLocate and eliminate him before he escapes.";
		missionDebriefDescriptions[ VICTORY ] = "You have eliminated $CLIP-" + id + "$.";
		missionDebriefDescriptions[ FAILURE ] = "$CLIP-"+id+"$ has escaped.\nMission failed.";
		System.out.println( "Assassinate vipidiot" );
		break;
            default:
                primaryObjective = new SecureAreaObjective();
		longMissionDescription = "Secure " + getRandomBaseName() + ". Kill everything that moves. \nHell, kill everything that doesn't, just to be sure.\n";
                missionDebriefDescriptions[ VICTORY ] = "Area secured\n";
		missionDebriefDescriptions[ FAILURE ] = "You have failed to secure the area.";
		System.out.println( "Secure" );
                break;
        }
        if( Math.random() < 0.3 )
        {
	    if( !hasMines )
            {
                objective = new AddSomeMinesObjective();
                objectives.add( objective );
		longMissionDescription += "\nThere are some mines spread around the complex.\nExercise caution.";
            }
        }        
        if( Math.random() < 0.5 )
        {
            objective = new BomberBruteParty();
            objectives.add( objective );
        }
        if( Math.random() < 0.5 )
        {
            if( !hasReturn )
            {
                endsAtBase = true;
		longMissionDescription += "\nReturn to base.";
                System.out.println( "Return" );
            }
        }   
    }
    
    public String toString()
    {
	return missionName;
    }
    
    public static String getRandomBaseName()
    {
	int start = (int)(Math.random()*baseStart.length);
        int middle = (int)(Math.random()*baseMiddle.length);
        int end = (int)(Math.random()*baseEnd.length);
        String baseName = baseStart[ start ] + " " + baseMiddle[ middle ] + " " + baseEnd[ end ];
	return baseName;
    }
}