// Monster.java
// Started on June 20th, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;

public abstract class Monster extends Character
{
    protected boolean dormant = false;
    
    public boolean applyDamage( int damage )
    {
        currentHitPoints = currentHitPoints-damage;
        lastAction = new WoundAction( this, location );
        if( currentHitPoints <= 0 )
        {
            alive = false;
            lastAction = new DeadAction( this, new Point( location ) );
            maximumActionPoints = 0;
            currentActionPoints = 0;
        }
        return alive;
    }    
    
    public boolean ignoresWalls()
    {
        return false;
    }

    // Monsters generally hate marines. Override for friendly ones
    public boolean isFoe( Character character )
    {
	if( character.getType().equals( "Marine" ) )
	    return true;
	return false;
    }    
    
    public abstract GameAction playAction();

    public GameAction doAction( GameAction action )
    {
        if( action.getActor() != this )
        {
            System.out.println( "Illegal action passed to " + getClass().getName() );
            return null;
        }
        if( action instanceof MoveAction )
        {
            //System.out.println( getClass().getName() + " walking from " + location.toString() + " to " + ((MoveAction)action).getEndPoint().toString() );
            direction = Map.getDirection( new Point( location ), new Point( ((MoveAction)action).getEndPoint() ), direction );
            //System.out.println( "Direction: " + direction );
            location = ((MoveAction)action).getEndPoint();
            currentActionPoints-=((MoveAction)action).getCost();
        }
        if( action instanceof ShootAction )
        {
            direction = Map.getDirection( new Point( location ), new Point( ((ShootAction)action).getEndPoint() ), direction );
            currentActionPoints-=((ShootAction)action).getWeapon().getActionsToFire();
        }
        if( action instanceof UseAction )
        {
            direction = Map.getDirection( new Point( location ), new Point( ((UseAction)action).getPoint() ), direction );
            currentActionPoints-= ((UseAction)action).getCost();
        }
	if( action instanceof MeleeAction )
	{
	    MeleeAction mAction = (MeleeAction)action;
	    direction = Map.getDirection( mAction.getAttacker().getLocation(), mAction.getDefender().getLocation(), direction );
	    currentActionPoints-=mAction.getCost();
	}
        lastAction = action;
        return action;
    }
    
    
    public boolean hasMoreActions()
    {
	return false;
    }
    
    public boolean isDormant()
    {
	return dormant;
    }
    
    public void preAction()
    {
    }
    
    public void postAction()
    {
    }
    
    public void setSpottedPlayer( Character character, boolean canSee )
    {
	if( canSee )
	{
	    dormant = false;
	}
    }
    
    public void setCustomActionPoints( String range )
    {
	int newActionPoints = maximumActionPoints;
	try
	{
	    if( range.indexOf( "-" ) < 0 )
	    {
		newActionPoints = Integer.parseInt( range );
	    }
	    else
	    {
		int minimum = Integer.parseInt( range.substring( 0, range.indexOf( "-" ) ) );
		int maximum = Integer.parseInt( range.substring( 1+range.indexOf( "-" ) ) );
		newActionPoints = Math.min( minimum, maximum ) + 1 + (int)( Math.random()*Math.abs( maximum-minimum ) );
	    }
	}
	catch( NumberFormatException nfe )
	{
	}
	maximumActionPoints = newActionPoints;
	currentActionPoints = newActionPoints;
    }
    
    
    public void setCustomHitPoints( String range )
    {
	int newHitPoints = maximumHitPoints;
	try
	{
	    if( range.indexOf( "-" ) < 0 )
	    {
		newHitPoints = Integer.parseInt( range );
	    }
	    else
	    {
		int minimum = Integer.parseInt( range.substring( 0, range.indexOf( "-" ) ) );
		int maximum = Integer.parseInt( range.substring( 1+range.indexOf( "-" ) ) );
		newHitPoints = Math.min( minimum, maximum ) + 1 + (int)( Math.random()*Math.abs( maximum-minimum ) );
	    }
	}
	catch( NumberFormatException nfe )
	{
	}
	maximumHitPoints = newHitPoints;
	currentHitPoints = newHitPoints;
    }
    
    public void setDormant( boolean dormant )
    {
	this.dormant = dormant;
    }

} 