// PressureSwitchObjective.java
// Started on August 2nd, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;
import java.util.Vector;

public class PressureSwitchObjective extends Objective
{
    public static final String ADD_ON_TARGET_KEY = "Add on target";
    public static final String REMOVE_ON_TARGET_AT_KEY = "Remove on target at";
    public static final String ADD_OFF_TARGET_KEY = "Add off target";
    public static final String REMOVE_OFF_TARGET_AT_KEY = "Remove off target at";
    public static final String SHOW_ON_MAP_KEY = "Show on map";
    public static final String TOP_LEFT_KEY = "Top left";
    public static final String BOTTOM_RIGHT_KEY = "Bottom right";
    public static final String ON_TARGET_KEY = "On target ";
    public static final String ON_TARGET_KEY_KEY = "On target key ";
    public static final String ON_TARGET_VALUE_KEY = "On target value ";
    public static final String OFF_TARGET_KEY = "Off target ";
    public static final String OFF_TARGET_KEY_KEY = "Off target key ";
    public static final String OFF_TARGET_VALUE_KEY = "Off target value ";
    public static final String TRIGGER_THRESHOLD_KEY = "Trigger threshold";
    
    private Vector targets = new Vector();
    private Vector onTargets = new Vector();
    private Vector offTargets = new Vector();
    private boolean showOnMap = true;
    private Point topLeft = new Point( -1, -1 );
    private Point bottomRight = new Point( -1, -1 );
    private boolean triggered = false;
    private int triggerThreshold = 60;

    private String [] properties = { TOP_LEFT_KEY, BOTTOM_RIGHT_KEY, TRIGGER_THRESHOLD_KEY, SHOW_ON_MAP_KEY, ADD_ON_TARGET_KEY, REMOVE_ON_TARGET_AT_KEY, ADD_OFF_TARGET_KEY, REMOVE_OFF_TARGET_AT_KEY };
    
    public PressureSwitchObjective()
    {
    }

    public void setGame( Game game )
    {
        activeGame = game;
	topLeft.x = Math.min( topLeft.x, bottomRight.x );
	topLeft.y = Math.min( topLeft.y, bottomRight.y );
	bottomRight.x = Math.max( topLeft.x, bottomRight.x );
	bottomRight.y = Math.max( topLeft.y, bottomRight.y );
    }

    public int getNumberOfSprites()
    {
        return 0;
    }    
    
    public String [] getPropertyNames()
    {
        int propertyCount = properties.length + (onTargets.size()*3) + (offTargets.size()*3);
        //System.out.println( "PressureSwitchObjective has " + propertyCount + " properties with " + targets.size() + " targets" );
        String [] propertyNames = new String[ propertyCount ];
	int i, index;
        for( i = 0; i < properties.length; i++ )
        {
            propertyNames[ i ] = properties[ i ];
        }
        index = i;
        for( i = 0; i < onTargets.size(); i++ )
        {
            propertyNames[ index++ ] = ON_TARGET_KEY+i;
            propertyNames[ index++ ] = ON_TARGET_KEY_KEY+i;
            propertyNames[ index++ ] = ON_TARGET_VALUE_KEY+i;
        }
        for( i = 0; i < offTargets.size(); i++ )
        {
            propertyNames[ index++ ] = OFF_TARGET_KEY+i;
            propertyNames[ index++ ] = OFF_TARGET_KEY_KEY+i;
            propertyNames[ index++ ] = OFF_TARGET_VALUE_KEY+i;
        }
        return propertyNames;
    }

    private ObjectiveTarget getOnTarget( String id )
    {
	for( int i=0; i < targets.size(); i++ )
	{
	    ObjectiveTarget target = (ObjectiveTarget)onTargets.elementAt( i );
	    if( target.getIndex().equals( id ) )
	    {
		return target;
	    }
	}
	return null;
    }
    
    private ObjectiveTarget getOffTarget( String id )
    {
	for( int i=0; i < targets.size(); i++ )
	{
	    ObjectiveTarget target = (ObjectiveTarget)offTargets.elementAt( i );
	    if( target.getIndex().equals( id ) )
	    {
		return target;
	    }
	}
	return null;
    }
    
    public Class getPropertyClass( String key )
    {
        if( key.equals( TOP_LEFT_KEY ) || key.equals( BOTTOM_RIGHT_KEY ) )
        {
            return Point.class;
        }
	if( key.equals( TRIGGER_THRESHOLD_KEY ) )
	{
	    return Integer.class;
	}
	if( key.equals( SHOW_ON_MAP_KEY ) )
	{
	    return Boolean.class;
	}
	return String.class;
    }
    
    public String getProperty( String key )
    {
        if( key.equals( TOP_LEFT_KEY ) )
        {
            return Objective.pointToString( topLeft );
        }
        if( key.equals( BOTTOM_RIGHT_KEY ) )
        {
            return Objective.pointToString( bottomRight );
        }
 	if( key.equals( SHOW_ON_MAP_KEY ) )
        {
            return String.valueOf( showOnMap );
	}
	if( key.equals( TRIGGER_THRESHOLD_KEY ) )
	{
	    return String.valueOf( triggerThreshold );   
	}
        if( key.startsWith( ON_TARGET_KEY_KEY) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_KEY_KEY.length()-1 ).trim() );
            return ((ObjectiveTarget)onTargets.elementAt( index )).getKey();
        }
        if( key.startsWith( ON_TARGET_VALUE_KEY ) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_VALUE_KEY.length()-1 ).trim() );
            return ((ObjectiveTarget)onTargets.elementAt( index )).getValue();
        }
	if( key.startsWith( ON_TARGET_KEY ) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_KEY.length()-1 ).trim() );
            return ((ObjectiveTarget)onTargets.elementAt( index )).getIndex();
        }
	
        if( key.startsWith( OFF_TARGET_KEY_KEY ) )
        {
            int index = Integer.parseInt( key.substring( OFF_TARGET_KEY_KEY.length()-1 ).trim() );
            return ((ObjectiveTarget)offTargets.elementAt( index )).getKey();
        }
        if( key.startsWith( OFF_TARGET_VALUE_KEY ) )
        {
            int index = Integer.parseInt( key.substring( OFF_TARGET_VALUE_KEY.length()-1 ).trim() );
            return ((ObjectiveTarget)offTargets.elementAt( index )).getValue();
        }
	if( key.startsWith( OFF_TARGET_KEY ) )
        {
            int index = Integer.parseInt( key.substring( OFF_TARGET_KEY.length()-1 ).trim() );
            return ((ObjectiveTarget)offTargets.elementAt( index )).getIndex();
        }
	
        return null;
    }

    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( TOP_LEFT_KEY ) )
        {
            topLeft = new Point( Objective.pointFromString( value ) );
        }
        if( key.equals( BOTTOM_RIGHT_KEY) )
        {
            bottomRight = new Point( Objective.pointFromString( value ) );
        }
        if( key.equals( SHOW_ON_MAP_KEY ) )
        {
	    showOnMap = new Boolean( value ).booleanValue();
	}
	if( key.equals( TRIGGER_THRESHOLD_KEY ) )
	{
	    triggerThreshold = Objective.intFromString( value );
	}
        if( key.equals( ADD_ON_TARGET_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            String id = value;
	    //System.out.println( "Adding ON target: " + id );
            ObjectiveTarget newTarget = new ObjectiveTarget( id );
            onTargets.add( newTarget );
        }
        if( key.equals( REMOVE_ON_TARGET_AT_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            int index = Objective.intFromString( value );
	    onTargets.removeElementAt( index );
	}
	if( key.equals( ADD_OFF_TARGET_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            String id = value;
            ObjectiveTarget newTarget = new ObjectiveTarget( id );
            offTargets.add( newTarget );
        }
        if( key.equals( REMOVE_OFF_TARGET_AT_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            int index = Objective.intFromString( value );
	    offTargets.removeElementAt( index );
	}
	
        if( key.startsWith( ON_TARGET_KEY_KEY ) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_KEY_KEY.length()-1 ).trim() );
            if( onTargets.size() > index )
            {
                ((ObjectiveTarget)onTargets.elementAt( index )).setKey( value );
            }
	    return;
        }
	if( key.startsWith( ON_TARGET_VALUE_KEY ) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_VALUE_KEY.length()-1 ).trim() );
            if( onTargets.size() > index )
            {
                ((ObjectiveTarget)onTargets.elementAt( index )).setValue( value );
            }
	    return;
        }
	if( key.startsWith( ON_TARGET_KEY ) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_KEY.length()-1 ).trim() );
	    //System.out.println( "Setting ON target: " + index );
            // Check whether target exists
            if( onTargets.size() > index )
            {
                ((ObjectiveTarget)onTargets.elementAt( index )).setIndex( value );
            }
            else
            {
                // If it doesn't exist, add it
                setProperty( ADD_ON_TARGET_KEY, value );
            }
	    //System.out.println( onTargets.size() + " ON targets" );
        }
	
	
        if( key.startsWith( OFF_TARGET_KEY_KEY ) )
        {
            int index = Integer.parseInt( key.substring( OFF_TARGET_KEY_KEY.length()-1 ).trim() );
            if( offTargets.size() > index )
            {
                ((ObjectiveTarget)offTargets.elementAt( index )).setKey( value );
            }
	    return;
        }
        if( key.startsWith( OFF_TARGET_VALUE_KEY ) )
        {
            int index = Integer.parseInt( key.substring( OFF_TARGET_VALUE_KEY.length()-1 ).trim() );
            if( offTargets.size() > index )
            {
                ((ObjectiveTarget)offTargets.elementAt( index )).setValue( value );
            }
	    return;
        }
	if( key.startsWith( OFF_TARGET_KEY ) )
        {
            int index = Integer.parseInt( key.substring( OFF_TARGET_KEY.length()-1 ).trim() );
            // Check whether target exists
            if( offTargets.size() > index )
            {
                ((ObjectiveTarget)offTargets.elementAt( index )).setIndex( value );
            }
            else
            {
                // If it doesn't exist, add it
                setProperty( ADD_OFF_TARGET_KEY, value );
            }
        }
	
    }    
    
    public boolean isCompleted()
    {
        return true;
    }

    public boolean isSuccessful()
    {
	return true;
    }
    
    public int getSize()
    {
        return 0;
    }

    public boolean isLocatedIn( Point point )
    {
        if( ( point.x >= topLeft.x ) && ( point.y >= topLeft.y ) && ( point.x <= bottomRight.x ) && ( point.y <= bottomRight.y ) )
	{
            return true;
	}
        return false;
    }

    public boolean isLocatedIn( int x, int y )
    {
        return isLocatedIn( new Point( x, y ) );
    }

    public boolean isDestructible()
    {
        return false;
    }

    public boolean isShownOnMap()
    {
        return showOnMap;
    }
    
    public boolean isUsable()
    {
	return false;
    }

    public boolean handleAction( GameAction action )
    {
	// Run through the monsters and player characters to see if someone is standing on the switch
	Character character;
	int weightOn = 0;
	
	for( int i=0; i < activeGame.getPlayerCharacters().size(); i++ )
	{
	    character = activeGame.getPlayerCharacter( i );
	    if( character.isAlive() && isLocatedIn( character.getLocation() ) )
	    {
		weightOn += character.getSize();
	    }
	}
	for( int i=0; i < activeGame.getMonsters().size(); i++ )
	{
	    character = activeGame.getMonster( i );
	    if( character.isAlive() && isLocatedIn( character.getLocation() ) )
	    {
		weightOn += character.getSize();
	    }
	}
	Vector objectsOnTop = new Vector();
	for( int x=topLeft.x; x <= bottomRight.x; x++ )
	{
	    for( int y=topLeft.y; y <= bottomRight.y; y++ )
	    {
		MapObject mapObject = activeGame.getMap().getObjectForPoint( new Point( x, y ) );
		if( mapObject != null && !objectsOnTop.contains( mapObject ) )
		{
		    int weight = Math.max( 0, mapObject.getCostToMove() );
		    weightOn += weight*10;
		    objectsOnTop.add( mapObject );
		}
	    }
	}
	if( weightOn >= triggerThreshold && !triggered )
	{
	    triggered = true;
	    // Activate on targets
	    for( int i = 0; i < onTargets.size(); i++ )
	    {
		try
		{
		    //System.out.println( "Activating target " + (i+1) );
		    ObjectiveTarget target = (ObjectiveTarget)onTargets.elementAt( i );
		    Objective objective = activeGame.getObjectiveById( target.getIndex() );
		    objective.setProperty( target.getKey(), target.getValue() );
		    //System.out.println( objective.getClass().toString() + ": " + target.getKey() + " = " + target.getValue() ); 
		}
		catch( Exception e )
		{
		    System.out.println( e.toString() );
		}
	    }
	}
	if( weightOn < triggerThreshold && triggered )
	{
	    triggered = false;
	    // Activate off targets
	    for( int i = 0; i < offTargets.size(); i++ )
	    {
		try
		{
		    //System.out.println( "Activating target " + (i+1) );
		    ObjectiveTarget target = (ObjectiveTarget)offTargets.elementAt( i );
		    Objective objective = activeGame.getObjectiveById( target.getIndex() );
		    objective.setProperty( target.getKey(), target.getValue() );
		    //System.out.println( objective.getClass().toString() + ": " + target.getKey() + " = " + target.getValue() ); 
		}
		catch( Exception e )
		{
		    System.out.println( e.toString() );
		}
	    }
	}
        return false;
    }
}
