// RescueCiviliansObjective.java
// Started on March 1st, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;

public class RescueCiviliansObjective extends Objective
{
    protected Game activeGame;
    private int INITIAL_BOBS = 20;
    private final Point SAFE_POINT = new Point( -10, -10 );
    private int alive = 0;
    private int bobCount = -1;
    Monster [] bobs = new Monster[0];
    private boolean complete = true;
    private boolean showBobs = true;
    private int successLimit = 1;
    private boolean failed = false;
    private String [] propertyNames = { "Bobs", "Must save", "Show bobs" };

    public RescueCiviliansObjective()
    {
    }

    public void setGame( Game game )
    {
        activeGame = game;
        Bob bob;
        Point bStart;
        if( INITIAL_BOBS > 0 )
        {
            bobs = new Monster[ INITIAL_BOBS ];
            for( int b = 0; b < INITIAL_BOBS; b++ )
            {
                bob = new Bob();
                bob.setGame( activeGame );
                bob.setBehavior( Bob.ESCAPE );
                bStart = activeGame.getRandomPointForMonster();
                bob.setLocation( bStart );
                activeGame.addMonster( bob );
                bobs[ b ] = bob;
            }
        }
    }

    public int getNumberOfSprites()
    {
        return 0;
    }

    public Sprite getSprite( int x, int y )
    {
        return null;
    }

    public String [] getPropertyNames()
    {
        return propertyNames;
    }

    public String getProperty( String key )
    {
        // Ok, a HashMap might be a better idea in the long run...
        if( key.equals( "Bobs" ) )
        {
            return String.valueOf( INITIAL_BOBS );
        }
        if( key.equals( "Must save" ) )
        {
            return String.valueOf( successLimit );
        }
	if( key.equals( "Show bobs" ) )
        {
            if( showBobs )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }
        return null;
    }
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        boolean fubar = true;
        if( key.equals( "Bobs" ) )
        {
            try
            {
                int newBobs = Integer.parseInt( value );
                INITIAL_BOBS = newBobs;
                fubar = false;
            }
            catch( NumberFormatException e )
            {
                throw new IllegalArgumentException( "Not a number: " + value );
            }
        }
        if( key.equals( "Must save" ) )
        {
            try
            {
                int newLimit = Integer.parseInt( value );
                successLimit = newLimit;
                fubar = false;
            }
            catch( NumberFormatException e )
            {
                throw new IllegalArgumentException( "Not a number: " + value );
            }
        }
        if( key.equals( "Show bobs" ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                showBobs = true;
            }
            else
            {
                showBobs = false;
            }
            fubar = false;
        }
        if( fubar )
        {
            throw new IllegalArgumentException( "Key not found!" );
        }
    }

    public String getDescriptionClip()
    {
        String message;
        if( alive < 1 )
        {
            message = "All civilians are dead\n";
        }
        message = alive + " out of " + bobs.length + " civilians have been rescued\n";
        if( failed )
        {
            message = message + "You let too many civilians die.";
        }
        return message;
    }

    private boolean hasExited( Character character, Point point )
    {
        if( (Math.abs( character.getLocation().x-point.x ) <= 1) && (Math.abs( character.getLocation().y-point.y ) <= 1) )
            return true;
        else
            if( character.getLocation().equals( SAFE_POINT ) )
                return true;
        return false;
    }
    
    public boolean isCompleted()
    {
        if( bobCount == -1 )
        {
            bobCount = 0;
            Monster character;
            for( int i = 0; i < activeGame.getMonsters().size(); i++ )
            {
                character = activeGame.getMonster( i );
                if( character instanceof Bob )
                {
                    bobCount++;
                    if( ((Bob)character).getBehavior() == Bob.FOLLOW )
                        ((Bob)character).setBehavior( Bob.ESCAPE );
                }
            }
            bobs = new Monster[ bobCount ];
            int b = 0;
            for( int i = 0; i < activeGame.getMonsters().size(); i++ )
            {
                character = activeGame.getMonster( i );
                if( character instanceof Bob )
                {
                    bobs[ b++ ] = character;
                }
            }
        }
	complete = true;
        alive = 0;
        boolean characterReturned = false;
        Monster character;
        Point location;
        for( int b = 0; b < bobs.length; b++ )
        {
            character = bobs[ b ];
            characterReturned = false;
            if( character.isAlive() )
            {
                for( int j = 0; j < activeGame.getPlayerStartingPoints().size() && !characterReturned; j++ )
                {
                    location = ((MapNode)activeGame.getPlayerStartingPoints().elementAt(j)).getPoint();
                    if( hasExited( character, location ) )
                    {
                        characterReturned = true;
			character.setLocation( SAFE_POINT );
			alive++;
			character.setDormant( true );
                    }
                }
            }
            else
            {
                // Dead count as returned
                characterReturned=true;
            }
            complete = (complete && characterReturned );
        }
	return complete;
    }
    
    public boolean isSuccessful()
    {
	return ( alive >= successLimit );
    }

    public boolean isLocatedIn( Point point )
    {
        if( bobs == null )
        {
            return false;
        }
        Character character;
        for( int b = 0; b < bobs.length; b++ )
        {
            character = bobs[ b ];
            if( character.isAlive() && character.getLocation().equals( point ) )
                return true;
        }
        return false;
    }

    public boolean isLocatedIn( int x, int y )
    {
        if( bobs == null )
        {
            return false;
        }
        Character character;
        for( int b = 0; b < bobs.length; b++ )
        {
            character = bobs[ b ];
            if( character.isAlive() && character.getLocation().equals( new Point( x, y ) ) )
                return true;
        }
        return false;
    }

    public boolean isDestructible()
    {
        return false;
    }

    public boolean isShownOnMap()
    {
        return showBobs;
    }

    public boolean handleDamage( Point point, int damage )
    {
        return false;
    }

    public boolean handleAction( GameAction action )
    {
        return false;
    }

    
}