// Room.java
// Started on July 14th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.util.*;
import java.awt.Point;

public class Room
{
    private static int lastId = -1;
    
    private String name = "Empty room";
    private String id = "";
    private Point topLeft = new Point( -1, -1 );
    private Point bottomRight = new Point( -1, -1 );
    private Point hotSpot = new Point( -1, -1 );
    private Vector linkedRoomIds = null;
    
    public Room()
    {
	id = "Room"+(++lastId);
	linkedRoomIds = new Vector();
    }
    
    public Room( String id, String name, int minX, int minY, int maxX, int maxY )
    {
	this.id = id;
	this.name = name;
	topLeft = new Point( minX, minY );
	bottomRight = new Point( maxX, maxY );
	linkedRoomIds = new Vector();
	
	int idNumber = Integer.parseInt( id.substring( 4 ) );
	if( lastId <= idNumber )
	{
	    lastId = idNumber;
	}
    }
    
    public Point getHotSpot()
    {
	return new Point( hotSpot );
    }
    
    public void setHotSpot( Point hotSpot )
    {
	this.hotSpot = new Point( hotSpot );
    }
    
    public Point getTopLeft()
    {
	return topLeft;
    }
    
    public void setTopLeft( Point topLeft )
    {
	this.topLeft = new Point( topLeft );
    }
    
    public Point getBottomRight()
    {
	return bottomRight;
    }
    
    public void setBottomRight( Point bottomRight )
    {
	this.bottomRight = new Point( bottomRight );
    }
    
    public String getName()
    {
	return name;
    }
    
    public void setName( String name )
    {
	this.name = name;
    }
    
    public String getId()
    {
	return id;
    }
    
    public boolean hasLinked( Room room )
    {
	return( linkedRoomIds.contains( room.getId() ) );
    }
    
    public void addLink( String target )
    {
	if( !linkedRoomIds.contains( target ) )
	{
	    linkedRoomIds.add( target );
	}
    }
    
    public void addLinkedRoom( Room room )
    {
	if( !linkedRoomIds.contains( room.getId() ) )
	{
	    linkedRoomIds.add( room.getId() );
	    System.out.println( "Linked " + name + " (" + id + ") to " + room.getName() + " (" + room.getId() + ")" );
	    if( !room.hasLinked( this ) )
	    {
		room.addLinkedRoom( this );
	    }
	}
    }
    
    public void removeLinkedRoom( Room room )
    {
	if( linkedRoomIds.contains( room.getId() ) )
	{
	    linkedRoomIds.remove( room.getId() );
	    if( room.hasLinked( this ) )
	    {
		room.removeLinkedRoom( this );
	    }
	}
    }
        
    public String [] getLinkedRoomIds()
    {
	String [] theRoomIds = new String[ linkedRoomIds.size() ];
	for( int i=0; i < linkedRoomIds.size(); i++ )
	{
	    theRoomIds[i] = (String)linkedRoomIds.elementAt( i );
	}
	return theRoomIds;
    }
    
    public String toString()
    {
	return ( name + " (" + id + ")" );
    }
}

