// SabotageCommObjective.java
// Started on November 24th, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;

public class SabotageCommObjective extends Objective
{
    protected Game activeGame;
    private Point location = new Point(-1,-1);
    private Sprite liveSprite;
    private Sprite deadSprite;
    private boolean dead = false;
    private boolean alarmed = false;
    private int hitPoints = 5;
    private int sentries = -1;

    private String [] propertyNames = { "Location", "HitPoints", "Sentries" };
 
    public SabotageCommObjective()
    {
         liveSprite = new CommSprite( true );
         deadSprite = new CommSprite( false );
    }
 
    public void setGame( Game game )
    {
	activeGame = game;
        if( location.x < 0 )
        {
            location = activeGame.getRandomPointForMonster();
        }
        // Add CommSentries
        Point sStart, sEnd;
        CommSentry sentry;
        int numSentries = sentries;
        if( sentries < 0 )
        {
            numSentries = (int)(Math.random()*5)+2;
        }
        for( int i=0; i < numSentries; i++ )
        {
            sentry = new CommSentry();
            sentry.setGame( activeGame );
            sStart = activeGame.getRandomPointForMonster();
            sentry.setLocation( sStart );
            do
            {
                sEnd = activeGame.getRandomPointForMonster();
            }
            while( sStart.x == sEnd.x && sStart.y == sEnd.y );
            sentry.setPath( sStart, sEnd );
            sentry.setObjective( location );
            activeGame.addMonster( sentry );
        }
    }
          
    public int getNumberOfSprites()
    {
 	return 1;
    }
    
    public Sprite getSprite( int x, int y )
    {
        if( !dead )
            return liveSprite;
        return deadSprite;
    }

    public String [] getPropertyNames()
    {
        return propertyNames;
    }

    public String getProperty( String key )
    {
        if( key.equals( "Location" ) )
        {
            return Objective.pointToString( getLocation() );
        }
        if( key.equals( "Sentries" ) )
        {
            if( sentries < 0 )
            {
                return "Random";
            }
            else
            {
                return String.valueOf( sentries );
            }
        }
        if( key.equals( "HitPoints" ) )
        {
            return String.valueOf( hitPoints );
        }
        return null;
    }
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( "Location" ) )
        {
            Point newLocation = Objective.pointFromString( value );
            setLocation( newLocation ); 
        }
        if( key.equals( "Sentries" ) )
        {
            if( value.equalsIgnoreCase( "Random" ) )
            {
                sentries = -1;
            }
            else
            {
                sentries = Objective.intFromString( value );
            }
        }
        if( key.equals( "HitPoints" ) )
        {
            hitPoints = Objective.intFromString( value );
        }
    }

    public void setLocation( Point point )
    {
        location = point;
    }
    
    public Point getLocation()
    {
        return location;
    }
    
    public void setLocation( int x, int y )
    {
	setLocation( new Point( x, y ) );
    }
    
    public void setAlarmed( boolean alarmed )
    {
        if( !dead )
        {
            this.alarmed = alarmed;
            System.out.println( "Alarm set! Mission failed." );
            activeGame.handleMessage( new Message( null, "Alarm set! Mission failed.\nNow get the hell out of there." ) );
            Objective objective;
            for( int i = 0; i < 3; i++ )
            {
                objective = new RipperSpawnObjective();
                objective.setGame( activeGame );
                activeGame.addObjective( objective );
            }
        }
    }
    
    public boolean isCompleted()
    {
        if( dead || alarmed )
            return true;
        return false;
    }
    
    public boolean isSuccessful()
    {
        if( dead && !alarmed )
            return true;
        return false;
    }
    
    public int getSize()
    {
        if( dead )
            return 20;
        return 85;
    }
    
    public boolean isLocatedIn( Point point )
    {
        if( location.x == point.x && location.y == point.y )
            return true;
	return false;
    }
 
    public boolean isLocatedIn( int x, int y )
    {
        if( location.x == x && location.y == y )
            return true;
	return false;
    }
   
    public boolean isDestructible()
    {
        if( dead )
            return false;
	return true;
    }
    
    public boolean isShownOnMap()
    {
        if( dead )
            return false;
        return true;
    }
    
    public boolean handleDamage( Point point, int damage )
    {
        if( isLocatedIn( point ) )
        {
            hitPoints -= damage;
            if( hitPoints <= 0 )
            {
                dead = true;
                return true;
            }
        }
	return false;
    }

    public boolean handleAction( GameAction action )
    {
        if( action instanceof MoveAction )
        {
            if( ((MoveAction)action).getActor() instanceof CommSentry )
            {
                CommSentry sentry = (CommSentry)((MoveAction)action).getActor();
                if( Map.isNeighbor( location, sentry.getLocation() ) && sentry.isAlarmed() )
                {
                    if( !alarmed )
                    {
                        setAlarmed( true );
                        return true;
                    }
                }
            }
        }
        return false;
    }

}