// SeekerAI.java
// Started on June 22nd, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.util.Vector;
import java.awt.Point;

public class SeekerAI
{
    private static int MAX_TRIES = 10; // How many times will attempt to find a spot at maximum
    
    public static Point getSafePointForCharacter( Character character )
    {
	Game game = character.getGame();
	Character [] visibleEnemies = game.getVisibleEnemies( character );
	return getSafePointForCharacter( character, visibleEnemies );
    }
    
    public static Point getSafePointForCharacter( Character character, Character [] enemies )
    {
	Game game = character.getGame();
	Point point = null;
	if( enemies.length > 0 )
	{
	    Room room = game.getMap().getClosestRoomForPoint( character.getLocation() );
	    if( room == null )
	    {
		// Just cheat. Anywhere is better than here
		int tries = 0;
		boolean found = true;
		do
		{
		    point = game.getRandomPointForMonster();
		    for( int i=0; i < enemies.length; i++ )
		    {
			if( game.isCellVisibleToCharacter( enemies[i], point ) )
			{
			    found = false;
			}
		    }
		} while( !found && tries++ < MAX_TRIES );
	    }
	    else
	    {
		String [] neighborIds = room.getLinkedRoomIds();
		boolean found = false;
		Point testPoint = null;
		for( int i=0; i < neighborIds.length && !found; i++ )
		{
		    Room otherRoom = game.getMap().getRoom( neighborIds[i] );
		    if( otherRoom != null )
		    {
			testPoint = otherRoom.getHotSpot();
			boolean willDo = true;
			for( int e=0; e < enemies.length; e++ )
			{
			    if( game.isCellVisibleToCharacter( enemies[e], testPoint ) )
			    {
				willDo = false;
			    }
			}
			found = willDo;
		    }
		}
		if( found )
		{
		    point = testPoint;
		}
		else
		{
		    // All neighbours can be seen by enemies, so we're screwed anyway. Just pick one.
		    found = false;
		    int tries = 0;
		    if( neighborIds.length > 0 )
		    {
			do
			{
			    int index = (int)(Math.random()*neighborIds.length);
			    Room otherRoom = game.getMap().getRoom( neighborIds[ index ] );
			    if( otherRoom != null )
			    {
				found = true;
				point = otherRoom.getHotSpot();
			    }
			} while( !found && tries++ < MAX_TRIES );
		    }
		    else
		    {
			System.out.println( "SeekerAI: No neighbours for room! Picking a random point for escape" );
			point = game.getRandomPointForMonster();
		    }
		}
	    }
	}
	return point;
    }
        
    public static GameAction seek( Character seeker, Point target )
    {
	if( target == null )
	{
	    return null;   
	}
        Game game = seeker.getGame();
        Point location = seeker.getLocation();
        int direction = seeker.getDirection();
        
        GameAction action;
        Point cell;
        boolean rightHanded = (Math.random() < 0.5 );

        //System.out.println(" SeekerAI seeking: " + target.toString() );
        Vector path = game.getMap().findShortestPath( seeker, target, seeker.getMaximumActionPoints(), true );
        if( path.size() >= 2 )
        {
            cell = ((MapNode)path.elementAt( 1 )).getPoint();
            if( game.getMap().isClosedDoor( cell ) )
            {
		System.out.println( "Trying to open door" );
		action = new UseAction( seeker, cell );
		return action; 
	    }
            if( game.canCharacterMove( seeker, cell, true ) )
            {
                direction = Map.getDirection( location, cell, direction );
                action = new MoveAction( seeker, new Point( location ), new Point( cell ) );
                return action;
            }
            else
            {
                int newDirection = Map.getDirection( location, cell, direction );
                if( rightHanded )
                {
                    newDirection = (newDirection+1)%8;
                    if( !game.canCharacterMove( seeker, newDirection, false ) )
                    {
                        newDirection = Map.getDirection( location, cell, direction );
                        newDirection = (newDirection+7)%8;
                    }
                }
                else
                {
                    newDirection = (newDirection+7)%8;
                    if( !game.canCharacterMove( seeker, newDirection, false ) )
                    {
                        newDirection = Map.getDirection( location, cell, direction );
                        newDirection = (newDirection+1)%8;
                    }

                }
                if( game.canCharacterMove( seeker, newDirection, false ) )
                {
                    direction = newDirection;
                    cell = game.getMap().getNeighbor( location, newDirection );
                    action = new MoveAction( seeker, new Point( location ), new Point( cell ) );
                    return action;
                }
                System.out.println( "SeekerAI seek blocked" );
            }
        }	
        return null;        
    }
    
}