// Shotgun.java
// Started on March 17th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.util.Vector;

public class Shotgun extends Weapon
{
    private String [] firingModes = { "Normal" };
    private static Image icon = null;
    public static final int COST = 1;

    
    private int theRange = 0;

    public Shotgun()
    {
        if( icon == null )
        {
            icon = ResourceManager.loadImage( "WeaponGraphics/ShotgunIcon.png" );
        }
    }
    
    private int getBaseDamage()
    {
        return (int)(Math.random()*6)+2;
    }
    
    public int getDamage()
    {
        return getDamage( theRange, false );
    }

    public int getDamage( boolean critical )
    {
        if( critical )
            return 7;
        return getDamage();
    }

    public int getDamage( int range, boolean critical )
    {
        int damage;
        if( critical )
            damage = 7;
        else
            damage = getBaseDamage();
        if( range > 8 )
        {
            damage -= getBaseDamage()/2;
        }
        if( range > 16 )
        {
            damage = 1;
        }
        if( damage <= 0 )
            damage = 1;
        //System.out.println( "Shotgun damage: " + damage );
        return damage;
    }

    public int getActionsToFire()
    {
        return 2;
    }

    public int getCost()
    {
        return COST;
    }
    
    public int getMovementCost()
    {
        return 1;
    }
        
    public boolean hasAreaEffect()
    {
        return true;
    }

    public String getName()
    {
        return "Shotgun";
    }

    public Point [] getArea( Game theGame, Point fireFrom, Point fireTo )
    {
        Point [] area = null;
        theRange = (int)fireFrom.distance( fireTo );
        if( theRange < 8 )
        {
            area = new Point[1];
            area[0] = fireTo;
        }
        else
        {
            Vector points = new Vector();
            points.add( fireTo );
            Point point;
            int direction = Map.getDirection( fireFrom, fireTo, Map.NORTH );
            switch( direction )
            {
                case Map.NORTH:
                case Map.SOUTH:
                    point = new Point( fireTo.x-1, fireTo.y );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    point = new Point( fireTo.x+1, fireTo.y );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    break;
                case Map.EAST:
                case Map.WEST:
                    point = new Point( fireTo.x, fireTo.y-1 );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    point = new Point( fireTo.x, fireTo.y+1 );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    break;
                case Map.NORTH_WEST:
                    point = new Point( fireTo.x, fireTo.y-1 );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    point = new Point( fireTo.x-1, fireTo.y );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    break;
                case Map.NORTH_EAST:
                    point = new Point( fireTo.x, fireTo.y-1 );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    point = new Point( fireTo.x+1, fireTo.y );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    break;
                case Map.SOUTH_WEST:
                    point = new Point( fireTo.x-1, fireTo.y );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    point = new Point( fireTo.x, fireTo.y+1 );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    break;
                case Map.SOUTH_EAST:
                    point = new Point( fireTo.x+1, fireTo.y );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    point = new Point( fireTo.x, fireTo.y+1 );
                    if( theGame.getMap().hasLineOfSight( fireFrom, point ) )
                    {
                        points.add( point );
                    }
                    break;
            }
            area = new Point[ points.size() ];
            for( int i=0; i < points.size(); i++ )
            {
                area[ i ] = (Point)points.elementAt( i );
            }
        }
        return area;
    }

    public String [] getFiringModeNames()
    {
        return firingModes;
    }

    public int getNumberOfFiringModes()
    {
        return 1;
    }

    public int getFiringMode()
    {
        return 0;
    }

    public void setFiringMode( int mode )
    {
    }

    public int getRequiredAttribute()
    {
        return Marine.RIFLE;
    }

    public Image getIcon()
    {
        return icon;
    }
}