// TeamManagerWindow.java
// Started on June 13th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.BadLocationException;
import javax.swing.border.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;


/**
 * This class creates the interface for the team manager.
 * It also handles mission selection.
 *
 * @author	Virtu Halttunen
 * @version	0.1
 */
public class TeamManagerWindow extends JFrame implements ActionListener, MouseListener, WindowListener
{
    private final String MISSION_FILE_PATH_KEY = "MissionFilePath";
    public static final String MARINE_FILE_NAME = ".marines";
    public static final String BASE_RESOURCE_PATH_KEY = "BaseResourcePath";
    public static final String [] availableWeaponClasses = { "Pistol", "SubmachineGun", "Shotgun", "AssaultRifle", "Flamer", "AutoCannon", "MiniGun" };
    
    private Vector marines;
    private Vector missionTeam;
    private String missionDirectoryPath = null;
    private Properties preferences = null;
    private Marine selectedMarine = null;
    private int availableEquipmentPoints = 8;
    private int maximumEquipmentPoints = 8;

    private Game nextGame = null;
    private String nextMissionFileName = null;
    private Mission mission = null;
    private boolean needsSaving = false;
    private boolean canChangeMissionTeam = true;
    private GameWindow gameWindow = null;
    private MissionDialog missionDialog = null;
    
    private JButton selectMissionButton;
    private JButton recruitButton;
    private JButton createButton;
    private JButton dischargeButton;
    private JButton addToTeamButton;
    private JButton removeFromTeamButton;
    private JTextField missionNameField;
    private JTextArea missionBriefArea;
    private JButton startMissionButton;
    private MarineTeamPanel marineTeamPanel;
    private JLabel marineNameLabel;
    private MarineInfoPanel marineInfoPanel;
    private MarineTeamPanel missionTeamPanel;
    //private JTextField messageField;
    private JTextPane messageField;
    private StyledDocument messages;
    private MarineCreationDialog creationDialog;

    private JPopupMenu weaponMenu;

    private WeaponMenuItem [] weaponMenus;
    
    private Style errorStyle;
    private Style defaultStyle;

    /*
    private JMenuItem pistolMenu;
    private JMenuItem assaultRifleMenu;
    private JMenuItem shotgunMenu;
    private JMenuItem flamerMenu;
    private JMenuItem smartGunMenu;*/
    
    
    public TeamManagerWindow()
    {
        super( "TheGame: Team entry" );
	
        Box top = Box.createHorizontalBox();
        Box left = Box.createVerticalBox();

	Box labelBox = Box.createHorizontalBox();
	labelBox.add( Box.createHorizontalGlue() );
        labelBox.add( new JLabel( "Available soldiers" ) );
	labelBox.add( Box.createHorizontalGlue() );
	left.add( labelBox );
        marineTeamPanel = new MarineTeamPanel();

        marineTeamPanel.addMouseListener( this );
        JScrollPane marineScroller = new JScrollPane( marineTeamPanel );
        left.add( marineScroller );

        JPanel teamButtonsPanel = new JPanel();
        addToTeamButton = new JButton( "Add to team" );
        addToTeamButton.addActionListener( this );
        teamButtonsPanel.add( addToTeamButton );
        removeFromTeamButton = new JButton( "Remove from team" );
        removeFromTeamButton.addActionListener( this );
        teamButtonsPanel.add( removeFromTeamButton );
        left.add( teamButtonsPanel );
        
	labelBox = Box.createHorizontalBox();
	labelBox.add( Box.createHorizontalGlue() );
        labelBox.add( new JLabel( "Mission team" ) );
	labelBox.add( Box.createHorizontalGlue() );
	left.add( labelBox );
        missionTeamPanel = new MarineTeamPanel();
        missionTeamPanel.setAllowGrow( false );
        missionTeamPanel.setAllowWeaponSelection( true );
        missionTeamPanel.setSlots( 5 );
        missionTeamPanel.addMouseListener( this );
        JScrollPane missionTeamScroller = new JScrollPane( missionTeamPanel );
        left.add( missionTeamScroller );
	left.add( Box.createVerticalStrut( 12 ) );

	messageField = new JTextPane();
	messageField.setEditable( false );
	JScrollPane messageScroller = new JScrollPane( messageField );
	left.add( messageScroller );
	messages = messageField.getStyledDocument();
	defaultStyle = messageField.addStyle( "defaultStyle", null );
	StyleConstants.setForeground( defaultStyle, Color.black );
	errorStyle = messageField.addStyle( "errorStyle", defaultStyle );
	StyleConstants.setForeground( errorStyle, Color.red );	
        
        top.add( left );
        
        Box right = Box.createVerticalBox();
	labelBox = Box.createHorizontalBox();
	labelBox.add( Box.createHorizontalGlue() );
        marineNameLabel = new JLabel( "No soldier selected" );
        labelBox.add( marineNameLabel );
	labelBox.add( Box.createHorizontalGlue() );
	right.add( labelBox );
        marineInfoPanel = new MarineInfoPanel();
	marineInfoPanel.addMouseListener( this );
        right.add( marineInfoPanel );
        JPanel buttonsPanel = new JPanel();
        recruitButton = new JButton( "Recruit" );
        recruitButton.addActionListener( this );
        buttonsPanel.add( recruitButton );
        createButton = new JButton( "Create" );
        createButton.addActionListener( this );
        buttonsPanel.add( createButton );
        dischargeButton = new JButton( "Discharge" );
        dischargeButton.addActionListener( this );
        buttonsPanel.add( dischargeButton );
        right.add( buttonsPanel );

	right.add( Box.createVerticalGlue() );
        JPanel missionSpacer = new JPanel();
        missionSpacer.add( new JLabel( "Next mission" ) );
        right.add( missionSpacer );
        JPanel missionNamePanel = new JPanel();
        missionNameField = new JTextField( 20 );
        missionNamePanel.add( missionNameField );
        selectMissionButton = new JButton( "Select" );
        selectMissionButton.addActionListener( this );
        missionNamePanel.add( selectMissionButton );
        right.add( missionNamePanel );
        missionBriefArea = new JTextArea( 10, 30 );
        missionBriefArea.setLineWrap( true );
        missionBriefArea.setWrapStyleWord( true );
        JScrollPane descScroller = new JScrollPane( missionBriefArea );
        right.add( descScroller );
	Box startButtonBox = Box.createHorizontalBox();
	startButtonBox.add( Box.createHorizontalGlue() );
        startMissionButton = new JButton( "Start mission" );
        startMissionButton.addActionListener( this );
        startMissionButton.setEnabled( false );
	startButtonBox.add( startMissionButton );
        right.add( startButtonBox );
	top.add( Box.createHorizontalStrut( 12 ) );
        top.add( right );
	top.setBorder( new EmptyBorder( 10, 20, 20, 20 ) ); 
        getContentPane().add( top );
	setIconImage( ResourceManager.loadImage( "FubarIcon.png" ) );
        // Weapon selection menu
        weaponMenu = new JPopupMenu();

        weaponMenus = new WeaponMenuItem[ availableWeaponClasses.length ];
        for( int i=0; i < availableWeaponClasses.length; i++ )
        {
            weaponMenus[i] = new WeaponMenuItem( availableWeaponClasses[ i ] );
            weaponMenus[i].addActionListener( this );
            weaponMenu.add( weaponMenus[i] );
        }
	missionDialog = new MissionDialog( this );

	if( !loadPreferences() )
        {
            AboutDialog.showAboutDialog();
        }
	//missionDialog.setMissionDirectoryPath( missionDirectoryPath );
        missionTeam = new Vector();

        if( !loadPlayers() )
        {
	    // Try to load old style
	    System.out.println( "Could not load marines, trying old style" );
	    if( !loadPlayersOld() )
	    {
		// Create new team
		System.out.println( "Creating new team" );
		Marine marine;
		marine = CreatorMarine.createRandomSergeant();
		marines.add( marine );
		marine.setWeapon( new Pistol() );
		missionTeamPanel.addMarine( marine );
		for( int i=0; i < 4; i++ )
		{
		    marine = CreatorMarine.createRandomGrunt();
		    marines.add( marine );
		    missionTeamPanel.addMarine( marine );
		    checkWeapons();
		}
	    }
        }
        marineTeamPanel.setMarines( marines );
        pack();
	
        Dimension temp = missionNameField.getPreferredSize();
        missionNameField.setPreferredSize( temp );
        missionNameField.setMinimumSize( temp );
        missionNameField.setMaximumSize( temp );
	
	temp = new Dimension( temp );
	temp.height = temp.height*5;
	temp.width = missionTeamPanel.getWidth();
	messageScroller.setPreferredSize( temp );
	temp.width=200;
	messageScroller.setMinimumSize( temp );
	
	temp = marineScroller.getPreferredSize();
	temp.width=270;
	marineScroller.setMinimumSize( temp );
	temp = missionTeamScroller.getPreferredSize();
	temp.width=270;
	missionTeamScroller.setMinimumSize( temp );
	missionTeamPanel.invalidate();
	messageScroller.invalidate();
        addWindowListener( this );
        creationDialog = new MarineCreationDialog( this );
	validate();
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	if( getPreferredSize().width > screenSize.width )
	{
	    setSize( screenSize );
	}
	verifyStatus();
	setVisible( true );
        //setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
    }

    public void actionPerformed( ActionEvent e )
    {
        if( e.getSource() == startMissionButton )
        {
            if( nextGame != null )
            {
                startMission();
            }
        }
        if( e.getSource() == selectMissionButton )
        {
            //openMission();
	    if( missionDirectoryPath == null || !( new File( missionDirectoryPath ).exists() ) )
	    {
		String directory = System.getProperty( "user.dir" );
		JFileChooser fileChooser = new JFileChooser( directory );
		SimpleFileFilter filter = new SimpleFileFilter( ".mission" );
		fileChooser.setFileFilter( filter );
		fileChooser.setDialogTitle( "Please choose a mission file" );
		int returnVal = fileChooser.showOpenDialog( this );
		if( returnVal == JFileChooser.APPROVE_OPTION )
		{
		    File missionFile = fileChooser.getSelectedFile();
		    missionDirectoryPath = missionFile.getParent();
		    missionDialog.setMissionDirectoryPath( missionDirectoryPath );
		    ResourceManager.clearBaseURLs();
		    ResourceManager.addBaseURL( "file:"+missionDirectoryPath+"/Resources/" );
		}
	    }
	    if( nextMissionFileName != null )
	    {
		int changeStatus = MissionDialog.ONE_SHOTS_ONLY;
		if( !canChangeMissionTeam )
		{
		    changeStatus = MissionDialog.NEXT_MISSION_ONLY;
		}
		missionDialog.setMission( nextMissionFileName, changeStatus );
	    }
	    else
	    {
		missionDialog.setMission( null, MissionDialog.CAN_CHANGE_FREELY );
	    }
	    missionDialog.setVisible( true );
	    showMission( missionDialog.getMission() );
        }
        if( e.getSource() == recruitButton )
        {
            recruitRandomSoldier();
            needsSaving = true;
        }
        if( e.getSource() == createButton )
        {
            Marine marine;
            creationDialog.showNew();
            marine = creationDialog.getMarine();
            if( marine != null )
            {
                marines.add( marine );
                marineTeamPanel.addMarine( marine );
                marineInfoPanel.setMarine( marine );
                marineNameLabel.setText( Marine.ranks[ marine.getRank() ] + " " + marine.getName() );
                needsSaving = true;
            }
        }
        if( e.getSource() == dischargeButton && canChangeMissionTeam )
        {
            Marine marine = marineInfoPanel.getMarine();
            if( marine != null )
            {
                int reply = JOptionPane.showConfirmDialog( this, "Are you sure you want to discharge " + marine.getName() + "?", "Discharge soldier", JOptionPane.YES_NO_OPTION );
                if( reply != JOptionPane.OK_OPTION )
                {
                    return;
                }
                marines.remove( marine );
                if( missionTeam.contains( marine ) )
                {
                    missionTeamPanel.removeMarine( marine );
                    missionTeam = missionTeamPanel.getMarines();
                }
                marineTeamPanel.setMarines( marines );
                if( marine == marineInfoPanel.getMarine() )
                {
                    marineInfoPanel.setMarine( null );
                    marineNameLabel.setText( "No soldier selected" );
                }
                needsSaving = true;
                validate();
            }
        }
        if( e.getSource() == addToTeamButton && canChangeMissionTeam )
        {
            Marine marine = marineInfoPanel.getMarine();
            if( marine != null )
            {
                int reply = missionTeamPanel.addMarine( marine );
                if( reply == MarineTeamPanel.TEAM_FULL )
                {
                    Toolkit.getDefaultToolkit().beep();
                    //messageField.setText( "The team is full!" );
		    appendMessage( "The team is full!", errorStyle );
                }
                if( reply == MarineTeamPanel.MARINE_ALREADY_IN_TEAM )
                {
                    Toolkit.getDefaultToolkit().beep();
                    //messageField.setText( "The soldier is already in the team!" );
		    appendMessage( "The soldier is already in the team!", errorStyle );

                }
                if( reply == MarineTeamPanel.MARINE_ADDED_TO_TEAM )
                {
                    //messageField.setText( marine.getName() + " was added to team" );
		    appendMessage( marine.getName() + " was added to team", defaultStyle );

                    missionTeam = missionTeamPanel.getMarines();
                    if( missionTeam.size() == 1 )
                    {
                        marine.setWeapon( new Pistol() );
                        checkWeapons();
                        missionTeamPanel.repaint();
                    }
                    else
                    {
                        checkWeapons();
                        //marine.setWeapon( new AssaultRifle() );
                        missionTeamPanel.repaint();
                    }
                }
                validate();
                repaint();
            }
        }
        if( e.getSource() == removeFromTeamButton )
        {
            Marine marine = marineInfoPanel.getMarine();
            if( marine != null )
            {
                if( missionTeam.contains( marine ) )
                {
                    missionTeamPanel.removeMarine( marine );
                    missionTeam = missionTeamPanel.getMarines();
                }
            }
        }
        if( e.getSource() instanceof WeaponMenuItem )
        {
            WeaponMenuItem weaponMenu = (WeaponMenuItem)e.getSource();
            if( selectedMarine != null )
            {
                selectedMarine.setWeapon( weaponMenu.getWeapon() );
                checkWeapons();
                missionTeamPanel.repaint();
            }
        }
    }
    
    public boolean appendMessage( String message, Style theStyle )
    {
	Style style = theStyle;
	if( style == null )
	{
	    style = defaultStyle;
	}
	try
	{
	    messages.insertString( messages.getLength(), message+"\n", style );	
	}
	catch( BadLocationException blex )
	{
	    System.out.println( blex.toString() + " writing message: " + message );
	    return false;
	}
	return true;
    }
    
    public boolean loadPlayers()
    {
	boolean success = true;
        marines = new Vector();
        Marine marine;
        try
        {
            String directory = System.getProperty( "user.home" );
            File marineFile = new File( directory, MARINE_FILE_NAME );
            System.out.println( "Loading marines from " + marineFile.toString() );
            if( !marineFile.exists() )
            {
                System.out.println( "Marine file not found" );
                return false;
            }
            else
            {
		DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
		DOMImplementation builder = documentBuilder.getDOMImplementation();
		
		FileInputStream fis = new FileInputStream( marineFile );
		GZIPInputStream gis = new GZIPInputStream( fis );
		
		System.out.println( "Parsing marine file: " + marineFile.getName() );
		Document document = documentBuilder.parse( gis );
		gis.close();
		fis.close();
		
		Element root = document.getDocumentElement();
		canChangeMissionTeam = new Boolean( root.getAttribute( "canchangeteam" ) ).booleanValue();
		String missionFileName = root.getAttribute( "nextmissionfile" );
		if( missionFileName.equalsIgnoreCase( "Null" ) || missionFileName.trim().length() < 1 )
		{
		    nextMissionFileName = null;
		}
		else
		{
		    nextMissionFileName = missionFileName;
		}
		
		Element [] marineElements = XMLUtils.getNamedChildren( root, "marine" );
		for( int i=0; i < marineElements.length; i++ )
		{
		    marine = Marine.decodeFromElement( marineElements[i] );
		    if( marine != null )
		    {
			marines.add( marine );
                    }
                    else
                    {
                        success = false;
                        break;
                    }
		}
		Element missionTeamElement = XMLUtils.getNamedChild( root, "missionteam" );
		Element [] missionMarineElements = XMLUtils.getNamedChildren( missionTeamElement, "missionmarine" );
		for( int i=0; i < missionMarineElements.length; i++ )
		{
		    int index = XMLUtils.getIntFromAttribute( missionMarineElements[i], "index" );
		    String marineName = missionMarineElements[i].getAttribute( "name" );
		    String weaponClassName = missionMarineElements[i].getAttribute( "weapon" );
		    
		    if( index >= 0 && index < marines.size() )
		    {
			marine = (Marine)marines.elementAt( index );
			if( marine.getName().equals( marineName ) )
			{
			    missionTeamPanel.addMarine( marine );
			    try
			    {
				Weapon weapon = (Weapon)Class.forName( weaponClassName ).newInstance();
				marine.setWeapon( weapon );
			    }
			    catch( Exception e)
			    {}
			}
		    }
		    checkWeapons();
		}
	    }
	}
	catch( Exception e )
	{
	    System.out.println( "Exception loading marines! \n" + e.toString() );
	    success = false;
	}  
	if( success && nextMissionFileName != null )
	{
	    verifyStatus();
	}
	return success;
    }
    
    public boolean loadPlayersOld()
    {
        boolean success = true;
        marines = new Vector();
        Marine marine;
        try
        {
            String directory = System.getProperty( "user.home" );
            File marineFile = new File( directory, MARINE_FILE_NAME );
            System.out.println( "Loading marines from " + marineFile.toString() );
            if( !marineFile.exists() )
            {
                System.out.println( "Marine file not found" );
                return false;
            }
            else
            {
                FileInputStream fis = new FileInputStream( marineFile );
                DataInputStream dis = new DataInputStream( fis );
                // How many marines must be loaded
                int numMarines = dis.readInt();
                for( int i=0; i < numMarines; i++ )
                {
                    marine = Marine.load( dis );
                    if( marine != null )
                    {
                        marines.add( marine );
                    }
                    else
                    {
                        success = false;
                        break;
                    }
                }
                // Load mission team
                if( dis.available() > 0 )
                {
		    int count = 0;
		    try
		    {
			count = dis.readInt();
		    }
		    catch( Exception e )
		    {}
                    for( int i=0; i < count; i++ )
                    {
                        int index = dis.readInt();
                        String weaponClass = dis.readUTF();
                        if( index >= 0 && index < marines.size() )
                        {
                            marine = (Marine)marines.elementAt( index );
                            missionTeamPanel.addMarine( marine );
                            try
                            {
                                Weapon weapon = (Weapon)Class.forName( weaponClass ).newInstance();
                                marine.setWeapon( weapon );
                            }
                            catch( Exception e)
                            {}
                        }
                    }
                    checkWeapons();
                }
                dis.close();
                fis.close();
            }
        }
        catch( Exception e )
        {
            System.out.println( "Exception loading marines! \n" + e.toString() );
            success = false;
        }
        return success;

    }
    
    public boolean savePlayers()
    {
	boolean success = true;
	Marine marine;
	try
	{
	    String directory = System.getProperty( "user.home" );
            File marineFile = new File( directory, MARINE_FILE_NAME );
            System.out.println( "Saving marines to " + marineFile.toString() );
            if( !marineFile.exists() )
            {
                System.out.println( "Creating new marine file" );
                marineFile.createNewFile();
            }
            if( marineFile.canWrite() )
            {
		DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
		DOMImplementation builder = documentBuilder.getDOMImplementation();
		
		org.w3c.dom.Document document = builder.createDocument( null, "gamestatus", null );
		Element root = document.getDocumentElement();
		root.setAttribute( "canchangeteam", String.valueOf( canChangeMissionTeam ) );
		String missionFileName = nextMissionFileName;
		if( missionFileName == null )
		{
		    missionFileName = "Null";
		}
		root.setAttribute( "nextmissionfile", missionFileName );
		
		// Store marines
		for( int i=0; i < marines.size(); i++ )
		{
		    if( ((Marine)marines.elementAt( i )).isAlive() )
		    {
			marine = ((Marine)marines.elementAt( i ));
			Element marineElement = marine.encodeForDocument( document );
			root.appendChild( marineElement );
			root.appendChild( document.createTextNode( "\n" ) );
		    }
		}
		// Store mission team
		missionTeam = missionTeamPanel.getMarines();
		Element missionTeamElement = document.createElement( "missionteam" );
		for( int i=0; i < missionTeam.size(); i++ )
		{
		    // Get the marine index
		    marine = (Marine)missionTeam.elementAt( i );
		    int index = 0;
		    for( index = 0; index < marines.size(); index++ )
		    {
			if( marine == marines.elementAt( index ) )
			{
			    Element missionMarineElement = document.createElement( "missionmarine" );
			    missionMarineElement.setAttribute( "index", String.valueOf( index ) );
			    missionMarineElement.setAttribute( "name", marine.getName() );
			    missionMarineElement.setAttribute( "weapon", ( marine.getWeapon().getClass().getName() ) );
			    missionTeamElement.appendChild( missionMarineElement );
			    break;
			}
		    }
		}
		root.appendChild( missionTeamElement );
		// Save the document
		System.out.println( "Saving marines" );

		DOMSource domSource = new DOMSource( document );
		
		FileOutputStream fos = new FileOutputStream( marineFile );
		GZIPOutputStream gos = new GZIPOutputStream( fos );
		StreamResult streamResult = new StreamResult( gos );
		TransformerFactory transformerFactory = TransformerFactory.newInstance();
		Transformer transformer = transformerFactory.newTransformer();
		transformer.transform( domSource, streamResult );        
		
		gos.close();
		fos.close();
	    }
	}
	catch( Exception ex )
	{
	    System.out.println( "Exception saving marines! \n" + ex.toString() );
            success = false;
	}
	return success;
    }
    
    /*
    public boolean savePlayers()
    {
        boolean success = true;
        Marine marine;
        try
        {
            String directory = System.getProperty( "user.home" );
            File marineFile = new File( directory, MARINE_FILE_NAME );
            System.out.println( "Saving marines to " + marineFile.toString() );
            if( !marineFile.exists() )
            {
                System.out.println( "Creating new marine file" );
                marineFile.createNewFile();
            }
            if( marineFile.canWrite() )
            {
                FileOutputStream fos = new FileOutputStream( marineFile );
                DataOutputStream dos = new DataOutputStream( fos );
                // Count the living marines
                int living = 0;
                for( int i=0; i < marines.size(); i++ )
                {
                    if( ((Marine)marines.elementAt( i )).isAlive() )
                    {
                        living++;
                    }
                }
                dos.writeInt( living );
                // Save the living marine
                for( int i=0; i < marines.size(); i++ )
                {
                    if( ((Marine)marines.elementAt( i )).isAlive() )
                    {
                        marine = ((Marine)marines.elementAt( i ));
                        marine.save( dos );
                    }
                }
                // Save mission team
                missionTeam = missionTeamPanel.getMarines();
                dos.writeInt( missionTeam.size() );
                for( int i=0; i < missionTeam.size(); i++ )
                {
                    // Get the marine index
                    marine = (Marine)missionTeam.elementAt( i );
                    int index = 0;
                    for( index = 0; index < marines.size(); index++ )
                    {
                        if( marine == marines.elementAt( index ) )
                        {
                            dos.writeInt( index );
                            dos.writeUTF( marine.getWeapon().getClass().getName() );
                            break;
                        }
                    }
                }
                dos.close();
                fos.close();
            }
            else
                System.out.println( "Can't write to file" );
        }
        catch( Exception e )
        {
            System.out.println( "Exception saving marines! \n" + e.toString() );
            success = false;
        }
        return success;
    }
     */
    
    private boolean loadPreferences()
    {
        String directory = System.getProperty( "user.home" );
        Properties properties = new Properties();
        File prefsFile = new File( directory, ".theGame.properties" );
        if( prefsFile.exists() )
        {
            if( prefsFile.canRead() )
            {
                try
                {
                    FileInputStream fis = new FileInputStream( prefsFile );
                    properties.load( fis );
                    fis.close();
                }
                catch( Exception e )
                {
                    properties = null;
                }
            }
        }
        else
        {
            System.out.println( "Preference file not found" );
            preferences = new Properties();
            properties = null;
            return false;
        }
        if( properties != null )
        {
            preferences = properties;
            missionDirectoryPath = preferences.getProperty( MISSION_FILE_PATH_KEY );
	    if( missionDirectoryPath != null && new File( missionDirectoryPath ).exists() )
	    {
		missionDialog.setMissionDirectoryPath( missionDirectoryPath );
		System.out.println( "Mission directory: " + missionDirectoryPath );
		ResourceManager.addBaseURL( "file:"+missionDirectoryPath+"/Resources/" );
	    }
	    else
	    {
		missionDirectoryPath = null;
	    }
	}
        return true;
    }

    private void savePreferences()
    {
        if( missionDirectoryPath != null )
        {
            preferences.setProperty( MISSION_FILE_PATH_KEY, missionDirectoryPath );
        }
	preferences.setProperty( BASE_RESOURCE_PATH_KEY, ResourceManager.getBaseResourceURL() );
        String directory = System.getProperty( "user.home" );
        File prefsFile = new File( directory, ".theGame.properties" );
        if( preferences != null )
        {
            if( ! prefsFile.exists() )
            {
                try
                {
                    prefsFile.createNewFile();
                }
                catch( Exception e )
                {
                    System.out.println( "Couldn't save the preferences!" );
                    return;
                }
            }
            if( prefsFile.canWrite() )
            {
                try
                {
                    FileOutputStream fos = new FileOutputStream( prefsFile );
                    preferences.store( fos, "TheGame Preferences" );
                    fos.close();
                }
                catch( Exception e )
                {
                    System.out.println( "Couldn't save the preferences!" );
                }
            }
        }
    }
    
    public void showMission( Mission mission )
    {
	if( mission != null )
	{
	    this.mission = mission;
	    nextGame = new Game( mission );
	    missionNameField.setText( mission.getName() );
	    String missionDesc = mission.getMissionDescription();
	    missionDesc = missionDesc.replaceAll( "\\n\\n", "TEMPNEWLINE" );
	    missionDesc = missionDesc.replaceAll( "\\n", " " );
	    missionDesc = missionDesc.replaceAll( "TEMPNEWLINE", "\n\n" );
	    if( missionDesc.length() < 1 )
	    {
		missionDesc = "Random";
	    }
	    missionBriefArea.setText( missionDesc );
	    
	    if( missionTeamPanel.getSlots() != mission.getPlayerStartPoints().size() )
	    {
		System.out.println( "Reset mission team" );
		missionTeamPanel.setSlots( mission.getPlayerStartPoints().size() );
		missionTeam = new Vector();
		validate();
	    }
	    missionTeamPanel.repaint();
	    checkMissionStartConditions();
	}
	else
	{
	    missionNameField.setText( "No mission selected" );
	    missionBriefArea.setText( "" );
	}
    }
    
    public void openMission()
    {
        String directory;
	String missionName = "";
        String longMissionDescription = "";

        File missionFile;
        if( missionDirectoryPath == null )
        {
            directory = System.getProperty( "user.dir" );
        }
        else
        {
            directory = missionDirectoryPath;
        }
        JFileChooser fileChooser = new JFileChooser( directory );
        SimpleFileFilter filter = new SimpleFileFilter( ".mission" );
        fileChooser.setFileFilter( filter );
        fileChooser.setDialogTitle( "Choose a mission file" );
        int returnVal = fileChooser.showOpenDialog( this );
        if( returnVal == JFileChooser.APPROVE_OPTION )
        {
            missionFile = fileChooser.getSelectedFile();
            missionDirectoryPath = missionFile.getParent();
	    ResourceManager.clearBaseURLs();
	    ResourceManager.addBaseURL( "file:"+missionDirectoryPath+"/Resources/" );
        }
        else
        {
            return;
        }        
        try
        {
            // Load the mission data
	    mission = Mission.openMissionFromFile( missionFile );
	    if( mission.getPrimaryObjective() == null )
	    {
		mission.randomizeObjectives();
	    }
	}
        catch( FileNotFoundException e )
        {
            System.out.println( "Map file not found!" );
            resetMission();
            return;
        }
        catch( Exception e )
        {
            e.printStackTrace();
            resetMission();
            return;
        }
        if( mission != null )
        {
            nextGame = new Game( mission );
            missionNameField.setText( mission.getName() );
            String missionDesc = mission.getMissionDescription();
            if( missionDesc.length() < 1 )
            {
                missionDesc = "Random";
            }
            missionBriefArea.setText( missionDesc );

            if( missionTeamPanel.getSlots() != mission.getPlayerStartPoints().size() )
            {
                System.out.println( "Reset mission team" );
                missionTeamPanel.setSlots( mission.getPlayerStartPoints().size() );
                missionTeam = new Vector();
                validate();
            }
            missionTeamPanel.repaint();
            checkMissionStartConditions();
        }
        else
        {
            System.out.println( "No mission file" );
            System.exit( 1 );
        }
    }

    public void quit()
    {
	if( gameWindow != null )
	{
	    int answer = JOptionPane.showConfirmDialog( gameWindow, "Are you sure you want to quit the game?", "Quit game", JOptionPane.YES_NO_OPTION );
	    if( answer == JOptionPane.YES_OPTION || answer == JOptionPane.OK_OPTION )
	    {
		System.exit( 0 );
	    }
	    else
	    {
		setVisible( false );
	    }
	}
	else
	{
	    if( needsSaving )
            {
                savePlayers();
                savePreferences();
            }
            System.out.println( "Exiting... " );
            System.exit( 0 );
	}
	
    }
    
    public void resetMission()
    {
        nextGame = null;
	mission = null;
        startMissionButton.setEnabled( false );
    }
    
    public void startMission()
    {
        if( ( missionTeam.size() < mission.getPlayerStartPoints().size() ) && canChangeMissionTeam )
        {
            int reply = JOptionPane.showConfirmDialog( this, "Are you sure you want to proceed with this team?", "Incomplete team", JOptionPane.YES_NO_OPTION );
            if( reply != JOptionPane.OK_OPTION )
            {
                return;
            }
        }
        nextGame.setPlayers( missionTeam );
        nextGame.initialize();
	mission = null;
        setVisible( false );
        gameWindow = new GameWindow( nextGame, this );
    }

    public void recruitRandomSoldier()
    {
        Marine marine;
        // Check for team leaders
        boolean hasLeader = false;
        for( int i = 0; i < marines.size(); i++ )
        {
            marine = (Marine)marines.elementAt( i );
            if( marine.getRank() >= Marine.SERGEANT )
            {
                hasLeader = true;
            }
        }
        if( hasLeader )
        {
            marine = CreatorMarine.createRandomGrunt();
        }
        else
        {
            marine = CreatorMarine.createRandomSergeant();
        }
        marines.add( marine );
        marineTeamPanel.addMarine( marine );
        marineInfoPanel.setMarine( marine );
        marineNameLabel.setText( Marine.ranks[ marine.getRank() ] + " " + marine.getName() );
        validate();
    }

    public void mouseClicked( MouseEvent e )
    {
        if( e.getSource() == marineTeamPanel )
        {
            Marine marine = marineTeamPanel.getMarineAt( e.getPoint() );
            if( marine != null )
            {
                System.out.println( "Clicked on " + marine.getName() );
                marineNameLabel.setText( Marine.ranks[ marine.getRank() ] + " " + marine.getName() );
                marineInfoPanel.setMarine( marine );
		    
                if( e.getClickCount() >= 2  )
                {
		    if( canChangeMissionTeam )
		    {
			int reply = missionTeamPanel.addMarine( marine );
			if( reply == MarineTeamPanel.TEAM_FULL )
			{
			    Toolkit.getDefaultToolkit().beep();
			    //messageField.setText( "The team is full!" );
			    appendMessage( "The team is full!", errorStyle );
			}
			if( reply == MarineTeamPanel.MARINE_ALREADY_IN_TEAM )
			{
			    Toolkit.getDefaultToolkit().beep();
			    //messageField.setText( "The soldier is already in the team!" );
			    appendMessage( "The soldier is already in the team!", errorStyle );
			    
			}
			if( reply == MarineTeamPanel.MARINE_ADDED_TO_TEAM )
			{
			    //messageField.setText( marine.getName() + " was added to team" );
			    appendMessage( marine.getName() + " was added to team", defaultStyle );
			    
			}
			missionTeam = missionTeamPanel.getMarines();
			if( missionTeam.size() == 1 )
			{
			    marine.setWeapon( new Pistol() );
			    checkWeapons();
			    missionTeamPanel.repaint();
			}
			else
			{
			    checkWeapons();
			    //marine.setWeapon( new AssaultRifle() );
			    missionTeamPanel.repaint();
			}
		    }
		    else
		    {
			Toolkit.getDefaultToolkit().beep();
			appendMessage( "Cannot edit mission team at this time!", errorStyle );
		    }
		}
                validate();
                repaint();
            }
        }
	if( e.getSource() == marineInfoPanel )
	{
	    needsSaving = true;
	}
        if( e.getSource() == missionTeamPanel )
        {
            Marine marine = missionTeamPanel.getMarineAt( e.getPoint() );
            if( marine != null )
            {
                System.out.println( "Clicked on " + marine.getName() );
                marineNameLabel.setText( Marine.ranks[ marine.getRank() ] + " " + marine.getName() );
                marineInfoPanel.setMarine( marine );
                selectedMarine = marine;
                Point hitPoint = e.getPoint();
                
		if( canChangeMissionTeam )
		{
		    if( e.getClickCount() >= 2 && hitPoint.y < 165 )
		    {
			missionTeamPanel.removeMarineAt( e.getPoint() );
			missionTeam = missionTeamPanel.getMarines();
			selectedMarine = null;
			checkWeapons(); // In case the leader was removed
		    }
		    if( e.getClickCount() == 1 && hitPoint.y > 165 )
		    {
			hitPoint.x=(hitPoint.x/130)*130+10;
			hitPoint.y=165;
			checkWeaponMenuFor( marine );
			weaponMenu.show( missionTeamPanel, hitPoint.x, hitPoint.y );
		    }
		}
		else
		{
		    Toolkit.getDefaultToolkit().beep();
		    appendMessage( "Cannot edit mission team at this time!", errorStyle );
		}
	    }
        }
        checkMissionStartConditions();
    }

    public void cleanUpMission()
    {
        Marine marine;
	canChangeMissionTeam = nextGame.canChangeMissionTeamAfter();
	String missionFileName = nextGame.getNextMissionFileName();
	if( missionFileName != null && !missionFileName.equalsIgnoreCase( "Null" ) )
	{
	    // If we have a name, check for campaign end
	    if( missionFileName.equalsIgnoreCase( "End" ) )
	    {
		System.out.println( "Campaign finished" );
		nextMissionFileName = null;
		canChangeMissionTeam = true;
		missionDialog.setMission( null, MissionDialog.CAN_CHANGE_FREELY );
		missionDialog.setMissionDirectoryPath( missionDirectoryPath );		
	    }
	    else
	    {
		nextMissionFileName = missionFileName;
		int changeStatus = MissionDialog.ONE_SHOTS_ONLY;
		if( !canChangeMissionTeam )
		{
		    changeStatus = MissionDialog.NEXT_MISSION_ONLY;
		}
		missionDialog.setMission( nextMissionFileName, changeStatus );
		missionDialog.setMissionDirectoryPath( missionDirectoryPath );	
		savePreferences();
	    }
	    // Otherwise, keep the old. We just played a random mission
	}
	showMission( missionDialog.getMission() );
        for( int i = 0; i < marines.size(); i++ )
        {
            marine = (Marine)marines.elementAt( i );
            if( !marine.isAlive() )
            {
                marines.remove( marine );
                i--;
                if( missionTeam.contains( marine ) )
                {
                    missionTeamPanel.removeMarine( marine );
                }
                if( marine == marineInfoPanel.getMarine() )
                {
                    marineInfoPanel.setMarine( null );
                    marineNameLabel.setText( "No soldier selected" );
                }
            }
            else
            {
                marine.resetActionPoints();
                marine.resetHitPoints();
                marine.resetAmmunition();
		if( marine.hasNotification() )
		{
		    String rankPromotionMessage = marine.getRankPromotionDescription();
		    if( rankPromotionMessage.length() > 0 )
		    {
			appendMessage( rankPromotionMessage, null );
		    }
		    String attributeRaiseMessage = marine.getAttributeAdvanceDescription();
		    if( attributeRaiseMessage.length() > 0 )
		    {
			appendMessage( attributeRaiseMessage, null );
		    }
		}
            }
        }
        marineTeamPanel.setMarines( marines );
	if( !canChangeMissionTeam && missionTeam.size() > 0 )
	{
	    appendMessage( "Team is on extended mission and cannot be changed at this time.", null );
	    System.out.println( "Team changes denied" );
	}
	verifyStatus();

        nextGame = null;
	gameWindow = null;
        missionNameField.setText( "" );
        missionBriefArea.setText( "" );
        startMissionButton.setEnabled( false );
        needsSaving = true;
        validate();
    }
    
    private void checkMissionStartConditions()
    {
        missionTeam = missionTeamPanel.getMarines();
        if( nextGame != null && missionTeam.size() > 0 )
        {
            startMissionButton.setEnabled( true );
        }
        else
        {
            startMissionButton.setEnabled( false );
        }
    }
    
    private void checkWeapons()
    {
        // Check the available equipment points
        missionTeam = missionTeamPanel.getMarines();
	if( missionTeam.size() == 0 )
	{
	    return;
	}
        Marine leader = (Marine)missionTeam.elementAt( 0 );
	Marine marine = null;
	for( int i = 1; i < missionTeam.size(); i++ )
	{
	    marine = (Marine)missionTeam.elementAt( i );
	    if( marine != null )
	    {
		if( marine.getRank() > leader.getRank() )
		{
		    leader = marine;   
		}
	    }
	}
        availableEquipmentPoints = 8;
        if( leader != null )
        {
            int rank = leader.getRank();
            if( rank >= Marine.LIEUTENANT ) { availableEquipmentPoints = 14; }
            if( rank >= Marine.CAPTAIN ) { availableEquipmentPoints = 17; }
            if( rank >= Marine.MAJOR ) { availableEquipmentPoints = 40; }           
        }
	maximumEquipmentPoints = availableEquipmentPoints;
        Weapon weapon;
        int cost;
        for( int i = 0; i < missionTeam.size(); i++ )
        {
            marine = (Marine)missionTeam.elementAt( i );
            if( marine != null )
            {
                weapon = marine.getWeapon();
                if( weapon != null )
                {
                    cost = weapon.getCost();
                    if( availableEquipmentPoints < cost )
                    {
                        weapon = selectWeapon();
                        marine.setWeapon( weapon );
                        //messageField.setText( marine.getName() + " has been assigned " + weapon.getName() );
			appendMessage( marine.getName() + " has been assigned " + weapon.getName(), errorStyle );

                    }
                    availableEquipmentPoints-=weapon.getCost();
                }
                else
                {
                    weapon = selectWeapon();
                    marine.setWeapon( weapon );
                    //messageField.setText( marine.getName() + " has been assigned " + weapon.getName() );
		    appendMessage( marine.getName() + " has been assigned " + weapon.getName(), defaultStyle );
                    availableEquipmentPoints-=weapon.getCost();
                }
            }
        }
        System.out.println( availableEquipmentPoints + " equipment points left" );
    }


    private void checkWeaponMenuFor( Marine marine )
    {
        Weapon weapon = null;
        int cost = 0;
        if( marine != null )
        {
            weapon = marine.getWeapon();
            if( weapon != null )
            {
                cost = weapon.getCost();
            }
        }
        // Check the weapon menu items
        int effectiveEquipmentPoints = availableEquipmentPoints+cost;
        for( int i=0; i < weaponMenus.length; i++ )
        {
	    weaponMenus[i].setAvailable( maximumEquipmentPoints >= weaponMenus[ i ].getCost() );
            weaponMenus[i].setEnabled( effectiveEquipmentPoints >= weaponMenus[ i ].getCost() );
        }
        System.out.println( availableEquipmentPoints + " equipment points left" );
    }

    public Properties getPreferences()
    {
	return preferences;
    }
	
    
    private Weapon selectWeapon()
    {
        if( availableEquipmentPoints >= Flamer.COST ) { return new Flamer(); }
        if( availableEquipmentPoints >= AssaultRifle.COST ) { return new AssaultRifle(); }
        if( availableEquipmentPoints >= Shotgun.COST ) { return new Shotgun(); }
        return new Pistol();
    }

    public void verifyStatus()
    {
	if( nextMissionFileName == null || nextMissionFileName.trim().length() < 1 )
	{
	    nextMissionFileName = null;
	    canChangeMissionTeam = true;
	}
	if( nextMissionFileName != null )
	{
	    int canChangeStatus = MissionDialog.ONE_SHOTS_ONLY;
	    if( !canChangeMissionTeam )
	    {
		canChangeStatus = MissionDialog.NEXT_MISSION_ONLY;
	    }
	    missionDialog.setMission( nextMissionFileName, canChangeStatus );
	    Mission aMission = missionDialog.getMission();
	    if( aMission == null )
	    {
		// Next mission was not found, reset
		nextMissionFileName = null;
		canChangeMissionTeam = true;
		System.out.println( "Next mission not found!" );
		messageField.setText( "" );
	    }
	    showMission( aMission );
	    if( !canChangeMissionTeam && missionTeam.size() == 0 )
	    {
		appendMessage( "All soldiers have died on mission. Campaign is lost!", errorStyle );
		canChangeMissionTeam = true;
		nextMissionFileName = null;
	    }
	}
	missionTeamPanel.setEnabled( canChangeMissionTeam );
	addToTeamButton.setEnabled( canChangeMissionTeam );
	removeFromTeamButton.setEnabled( canChangeMissionTeam );
	dischargeButton.setEnabled( canChangeMissionTeam );
    }
    
    public void windowClosing( WindowEvent e )
    {
        if( e.getWindow() == this )
        {
            if( needsSaving )
            {
                savePlayers();
                savePreferences();
            }
            System.out.println( "Window closed. Exiting... " );
            System.exit( 0 );
        }
    }
    
    public void windowClosed( WindowEvent e )
    {
    }

    public void windowActivated( WindowEvent e )
    {}
    
    public void windowDeactivated( WindowEvent e )
    {}

    public void windowDeiconified( WindowEvent e )
    {}

    public void windowIconified( WindowEvent e )
    {}

    public void windowOpened( WindowEvent e )
    {}
    
    public void mousePressed( MouseEvent e )
    {}

    public void mouseReleased( MouseEvent e )
    {}

    public void mouseEntered( MouseEvent e )
    {}

    public void mouseExited( MouseEvent e )
    {}
}

class WeaponMenuItem extends JMenuItem
{
    static private Image unavailable = null;
    private Weapon weapon = new Pistol();
    private String weaponClassName = "Pistol";
    private ImageIcon theIcon;
    private ImageIcon theDisabledIcon;
    private Image icon = null;
    private Image disabledIcon = null;
    private BufferedImage unavailableIcon = null;
    private boolean available = true;

    public WeaponMenuItem( String weaponClassName ) throws IllegalArgumentException
    {
	if( unavailable == null )
	{
	    unavailable = ResourceManager.loadImage( "WeaponGraphics/UnavailableIcon.png" );
	}
        this.weaponClassName = weaponClassName;
	try
        {
            weapon = (Weapon)(Class.forName( "TheGame."+weaponClassName ).newInstance());
        }
        catch( ClassNotFoundException e )
        {
            throw new IllegalArgumentException( "Weapon class not found: " + weaponClassName );
        }
        catch( Exception e )
        {
            System.out.println( "Cannot instantiate weapon: " + weaponClassName );
        }
	
        //String iconPath = "WeaponGraphics/"+weaponClassName+"Icon.png";
	//icon = ResourceManager.loadImage( iconPath );
	icon = weapon.getIcon();
	disabledIcon = GrayFilter.createDisabledImage( icon );
	
	theIcon = new ImageIcon( icon );
        setIcon( theIcon );
	theDisabledIcon = new ImageIcon( disabledIcon );
	setDisabledIcon( theDisabledIcon );
	setText( weapon.getName() );
    }

    public int getCost()
    {
        return weapon.getCost();
    }
    
    public void setAvailable( boolean available )
    {
	this.available = available;
	if( available )
	{
	    theIcon.setImage( icon );
	    theDisabledIcon.setImage( disabledIcon );
	}
	else
	{
	    if( unavailableIcon == null )
	    {
		unavailableIcon = new BufferedImage( 100, 30, BufferedImage.TYPE_INT_ARGB_PRE );	
		Graphics2D g2 = unavailableIcon.createGraphics();
		g2.drawImage( disabledIcon, 0, 0, this );
		g2.drawImage( unavailable, 0, 0, this );
		g2.dispose();
	    }
	    theIcon.setImage( unavailableIcon );
	    theDisabledIcon.setImage( unavailableIcon );
	}
    }
    
    public Weapon getWeapon()
    {
        Weapon newWeapon = weapon;
        try
        {
            newWeapon = (Weapon)(Class.forName( weaponClassName ).newInstance());
        }
        catch( ClassNotFoundException e )
        {}
        catch( Exception e )
        {
            System.out.println( "Cannot instantiate weapon: " + weaponClassName );
        }
        
        return newWeapon;
    }
}