//
// TheGame.java
// Started on June 20th, 2001
// Copyright (c) Virtu Halttunen
// 
// June 20th, 2001: Loading and displaying map, loading player character and monsters, moving, monster behaviours,
// firing, turning to face target done. Pretty quick job...
// June 21st, 2001: Added MediaTracker to ImageCropper, so goodbye ugly flickering
// June 22nd - June 28th: Trip to the Netherlands
// June 29th, 2001: Added some debug information and disabled the call to super.paintComponent() in GameMapPanel, 
// reducing flicker even further
// Also added the overview map, and made monsters disappear from the main screen when they are not visible
// (Requires changes to the class structures, since now the corpses disappear as well)
// (Quickly hacked together an ugly fix for disappearing monster corpses) 
// Added support for moving by clicking on the game screen, and fixed a bug in Map.getDirection()
// Added action points and more players
// June 30th, 2001: Added shortest path finding and moving greater distances at a time. Pathfinding could be
// improved, though, as it gives twisting paths. As the result, a quick and dirty solution was added.
// July 1st, 2001: Stopped dead monsters from registering as targets and being drawn (stains handle that)
// Added keyboard scrolling and hit points. Oids can now fight and kill the players.
// Added simple mission end conditions, a better system is needed, though
// But hey! It's now a game
// Badly injured marines now have less action points
// July 2nd, 2001: Fixed some bugs in sprite classes
// July 5th, 2001: Altered action system so that monsters can do several actions
// Implemented a more dangerous Smart Oid
// July 6th, 2001: Overhauled the multiple actions system. The previous system didn't work properly
// July 12th, 2001: Fixed a bug in SmartOid
// July 20th, 2001: Changed target selection so that the topmost monster is selected
// Added random faces for marines
// July 21st, 2001: Added mission files that set player starting points
// July 24th, 2001: Added Rippers
// July 28th, 2001: Page up and down move to next and previous player, respectively, and Home centers on the 
// active one
// July 30th, 2001: Changed target handling when shooting so that characters on the way can be shot
// July 30th, 2001: While on the subject, changed the LOS and LOR checking to use the Bresenham algorithm. Much 
// more efficient than the old home-grown one
// August 7th, 2001: Started implementing the doors
// October 22nd, 2001: Finally got the computer back after a HD failure and started to work on the Game again. 
// Implemented "Use" button and enabled opening doors with it
// October 25th, 2001: Implemented "Use" graphics for marines and made opening doors look nicer. Animated the 
// doors and made it possible to close them. The animation is still a bit hackish, though. Doors don't inflict damage yet
// October 27th, 2001: Started to implement objectives. They sort of work now. Also added selectedIcons to buttons. 
// Fixed the bug that caused the marines to jump around when shot
// October 28th, 2001: Improved monster initialization routines. The monsters no longer can start underneath the 
// player characters
// October 29th, 2001: Improved game statistics
// October 30th, 2001: Made the Game class a subclass of Observable, made GameMapPanel implement Observer, changed 
// the door handling.
// Rippers now can open doors and seek players they have spotted even if they no longer see them
// November 1st, 2001: Fixed a bug in the shortest path finding. Added weights to diagonals, making the paths less 
// winding. Removed the hacky fix from June 30th. Fixed a bug in line of fire calculation (was blocked by dead marines)
// November 15th, 2001: Added an option to press the shift key along with page up and down to change the character 
// without centering the map. For some reason doesn't seem to work with ActionEvents gotten from JButtons, though
// November 16th, 2001: New marine face graphics
// November 24th, 2001: Added Sabotage Objective and accompanying CommSentries and CommSprites. Changed the target 
// handling for shooting to allow Objectives to be fired at 
// November 28th, 2001: Made the initial file chooser open to the current user directory
// November 30th, 2001: Added a quick and ugly hack to stop the monster corpses from being drawn over closed doors
// December 4th, 2001: Changed the animation system. Character sprites are now stored in a HashMap, and
// visibility calculations are done only when needed. The interface to game actions was changed
// December 6th, 2001: Changed the interface so that the game is always set to some mode. Added -noSound command line
// argument. Disabled the run button 
// December 9th, 2001: Added target rectangles that tell whether a monster is visible to the current character when
// shooting. Added RipperSpawnObjective
// December 11th, 2001: Added target rectangles to those Objectives that can be shot. Buffered the animation of
// door actions
// December 12th, 2001: Closing doors now kill the characters standing on them
// December 16th, 2001: Added the possibility to show target reticle in GameMapPanel. This isn't used yet, though
// December 18th, 2001: Added a progress monitor to GameMapPanel constructor
// December 19th, 2001: Added an option to show the mission brief with Help key
// December 22nd, 2001: Added ReleaseLavaObjective. Added an option to the Map to lock doors. Currently it locks all
// doors on their places. Needs a better system. Added MapEvent flag class.
// December 23rd, 2001: VK_INSERT duplicates VK_HELP for PC keyboards (GameWindow). Fixed the progress monitor in GameMapPanel
// December 26th, 2001: Made Objectives handle UseActions. Still needs tweaking, though
// December 27th, 2001: Fixed a bug that allowed using Objectives anywhere on the map
// December 30th, 2001: Added Brutes. Changed the target selection order when shooting, so monsters will be shot before Objectives
// January 2nd, 2002: Fixed a bug in LavaFlowObjective
// February 5th, 2002: Fixed a couple minor bugs in SabotageCommObjective and ReleaseLavaObjective
// February 12th, 2002: Added skills and ranks to the Marine class, as well as rudimentary way for saving and loading Marines
// February 13th, 2002: Added freeAttributePoints field to Marines. Made MarineStatisticsPanel show the hit points proportional to
// the marine's maximum hit points. Added a marine info dialog
// February 18th, 2002: Started adding different weapons and taking weapon skills into account when shooting
// February 22nd, 2002: Added (kludgy) area damage handling, and grenade firing mode to Assault Rifle. No graphics yet
// February 23rd, 2002: Fixed a bug in handleAreaDamage which let characters be "killed" multiple times
// February 25th, 2002: Added weapon effect graphics
// February 27th, 2002: Added an option to move the freebie experience points to attributes in MarineInfoDialog
// February 28th, 2002: Marines with Luck can now evade death in certain conditions. Added a combo box to change weapon firing modes.
// That was almost more work than it was worth. Problems with focus managers and action listeners galore. As a result, a couple nasty
// hackish workarounds had to be added. Added methods that enable monsters to fight each other, and changed Ripper to use them
// March 1st, 2002: Added Bobs and RescueCiviliansObjective. Finally got fed up with the monster action system that could get
// stuck into an eternal loop and made it use for loop instead. Updated Brutes to use similar behaviour as Rippers
// March 2nd, 2002: Changed LavaFlowObjective so that the lava won't flow into player starting area. Added BobBQueObjective. Fixed
// the size of weaponModes combobox, so that mapPanel gets all space on resize. Added BomberBrutes, and made the necessary changes to
// Game.handleRangedAttack, including the introduction of Armed interface. Added BomberBruteParty for fun.
// March 3rd, 2002: Changed weapon sprite system so one sprite handles all the weapons of the same class. Made the game check the
// objectives for each monster action. Now they can't float past that lava any more, among other things
// March 4th, 2002: Fixed an issue in door animations (hopefully)
// March 5th, 2002: Made Bob roaming somewhat smarter. Sheltering still needs work. Properly centers the on the character even when
// the window is resized
// March 8th, 2002: Hopefully managed to fix some bugs in Bob panic behaviours
// March 12th, 2002: Added some OS X specific stuff (which unfortunately doesn't work properly yet). Of course it's encapsulated
// to a separate file, so the game should still run perfectly on other OSes. Added an option to select the mission file from
// command line. Mission statistics now appear in a dialog
// March 20th, 2002: Added Shotgun Brutes, which also required some changes to the sprite and weapon animation systems.
// Game.handleRangedAttack() now expects a Character that also implements the Armed interface. Not very clean code API-wise
// but I was feeling lazy.
// March 21st, 2002: Fixed a bug in Game.handleRangedAttack(). If the weapon had area damage, it required no action points to fire
// March 23rd, 2002: Minor tweaks here and there... GameMapPanel.animateShootAction(), MarineStatisticsPanel, CommSentry...
// Sabotage now starts spawning three Rippers if the alarm is set
// March 25th, 2002: Added RecoverDataObjective and Messages
// March 29th, 2002: New map format, allowing for tileable floor pieces, breakable objects, and so on
// April 1st, 2002: The scenery objects can now be destroyed. Honestly!
// April 3rd, 2002: The marines can no longer move through other characters or be stacked on them. The monsters still can, though
// April 10th, 2002: Hopefully managed to squash a bug that sometimes caused the doors to flicker
// April 11th, 2002: Some animation pauses now depend on the elapsed time, so the actual animation time varies somewhat less than before
// April 17th, 2002: Missions now save the player starting orientations as well as positions
// April 19th, 2002: Mouse buttons are now checked in GameWindow.mouseClicked(). Left button performs the action selected with game
// mode, right button always shoots (with second trigger) and middle button acts as Use button. Now it is possible to play the Game
// without having to switch between different modes. Also, extra buttons such as the scroll wheel no longer try to shoot unnecessarily
// April 22nd, 2002: GameMapPanel no longer tries to animate sprites that are outside the viewport in animateBufferedMoveActions
// April 24th, 2002: Objects now block movement and sight separately. Fixed a ClassCastException in Bob and added extra checks to
// MapCell.isPassable()
// April 26th, 2002: Fixed some bugs in LavaFlowObjective. Lava now flows on objects and destroys them. Brutes now have
// graphics for WoundAction
// May 19th - May 23rd, 2002: Been working on the MissionEditor
// May 24th, 2002: Changed Objective saving and loading process.
// May 27th, 2002: The Game now loads monsters, Objectives and mission descriptions from mission files
// May 28th, 2002: Added LockedDoorObjective
// May 29th, 2002: Mission rewards are now smaller on failed missions
// May 31st, 2002: Started adding GameWindow preferences
// June 1st, 2002: The game window location and size, and the visibility and location of the marine info dialog are now saved
// June 3rd, 2002: Made the area calculation in Shotgun check whether the points can be seen
// June 4th, 2002: Added CyberBrute and AssassinateObjective. Fixed a bug in Game.handleRangedAttack()
// June 5th, 2002: Added CyberBrute as a possibility to random missions, made normal Brutes use their chain saws
// more eagerly. The marines now expect proper leadership (or morale), or they will have less action points
// June 6th, 2002: Marines now get experience to Leadership if others look up to them. Also, they remember last turn's
// leadership value. Implemented first aid. Made the soldiers turn to face the object or character they are using
// June 10th, 2002: Fixed a bug in Brute
// June 11th, 2002: Tweaked first aid handling. Marines can now be promoted after a successful mission. Fixed a
// bug in SabotageCommObjective
// June 13th, 2002: Started adding Team Manager. Mission info is now shown before starting it
// June 15th, 2002: Team manager now functional. It's possible to hire and discharge soldiers. No support for
// custom soldiers yet
// June 16th, 2002: Tweaked MarineStatisticsPanel so it can show only the face and name. Added a proper
// about dialog
// June 17th, 2002: Minor interface refinement in TeamManagerWindow. Added Character.resetHitPoints, so
// marines can be healed between missions
// June 19th, 2002: Made the image vectors in MarineStatisticsPanel static
// June 20th, 2002: One year anniversary! The Game website goes live
// June 23rd, 2002: The Game now returns to team manager window even if all marines die on a mission
// July 1st, 2002: The action points as well as hit points are now reset after a mission
// July 2nd, 2002: Added CloneVatObjective. Changed Objective sprite getting in GameMapPanel a little
// (from unused index to coordinates). Added an option to abort a mission
// July 3rd, 2002: Added AirlockDoorsObjective. Added boolean to String and vice versa methods to
// Objective. Minor tweaks to LockedDoorObjective. Fixed misaligned pieces in objects.png
// July 5th, 2002: This comment line is dedicated to Nella, an American Cocker Spaniel, the best dog
// in the world
// July 8th, 2002: Minor tweaks to marine rank requirements
// August 2nd, 2002: Random sergeants now have mimimum morale of 4 instead of 2, which is enough to
// compensate one action point lost to bad leadership. Added CreatorMarine and moved the appearance
// and attribute randomizer methods there
// August 4th, 2002: It is now possible to create custom soldiers
// August 5th, 2002: Minor tweaks to weapon action point handling when shooting. Minor tweaks to
// Bob escape routines (they still are rather bad)
// August 8th, 2002: Added CreatorMarine.convert() that transforms a CreatorMarine to regular one
// August 9th, 2002: You can now right click to remove points from attributes when creating a
// marine. Had forgotten to write that code... Added + and - "buttons" to attribute tables in
// MarineInfoPanel and MarineCreationDialog as interface hints
// December 4th, 2002: Added rudimentary weapon selection
// December 18th, 2002: Hopefully managed to fix the occasionally disappearing sprites. Ammunition
// is now reset between missions. Fixed a bug that could place monsters in cells that don't have floor
// December 24th, 2002: The sum of marines' kills is now separate from total kills in mission statistics
// January 16th, 2003: Added weapon icons to MarineStatisticsPanel and team manager window
// March 11th, 2003: Installed Java 1.4.1. The Game exploded. Now somewhat works again after adding
// KeyListeners to every GUI Component in GameWindow to compensate for the new focus management.
// Graphics are still broken in OS X
// March 27, 2003: Made ImageCropper return the images as BufferedImages with type TYPE_INT_ARGB_PRE.
// This fixed the broken wall shadows. Animations and cursors are still broken
// June 17th, 2003: Objectives now block movement and line of fire as characters.
// June 18th, 2003: Changed Bob behaviour so the Bobs are less inclined to stay put if they are close
// to the marines. Also, Bob roaming now obeys character movement blocking (but escape routine does not)
// June 20th, 2003: Happy birthday, The Game!
// June 21st, 2003: Started adding proper base for monster movement blocking. Rippers now obey this,
// and cannot run into other monsters. They will try to walk past obstacles when seeking targets, but
// aren't too bright, though
// June 22nd, 2003: Continued the monster behaviour overhaul, adding SeekerAI and RoamerAI classes.
// Not exactly according to the State Design Pattern, but it made the monster code somewhat shorter, easier
// to read, and most monsters now avoid collisions. Monster.doAction() now handles GameActions and costs for
// movement and shooting. Added visual feedback to first aid
// June 23rd, 2003: Added flamer. Monster corpses and other stains are now only drawn after the player
// characters have seen them at least once
// June 24th, 2003: Added movement penalties and points costs for weapons. The run button starts to look
// potentially useful again, and the whole team cannot be loaded with heavy weapons any more. Flamer now
// leaves stains. Added even more error checking to ImageCropper
// July 7th, 2003: Added mines and related Objectives (MineBehaviorObjective, AddSomeMinesObjective and
// DisarmMinesObjective). Changed the way GameActions and especially ShootActions are handled during
// monster moves. ShootAction now triggers buffered MoveActions before it, so mines work properly.
// Added a discrete health icon for full health
// August 5th, 2003: Tweaked action points so the game isn't as difficult for beginners
// August 11th, 2003: Several bug fixes with team selection and shooting. Added automatic sentry guns
// and SurvivalObjective.
// August 13th, 2003: Fixed a bug in mine avoidance
// August 14th, 2003: Fixed a sprite display bug when opening doors. Made weapon menus more user-friendly
// August 16th, 2003: Fixed moving sentry guns. They now slide nicely on the floor
// August 31st, 2003: New sprites for Rippers
// September 1st, 2003: Added Slashers. Added monster factions. Fixed an obscure bug in
// RescueCiviliansObjective which caused the mission to fail if there were no bobs to begin with, even
// if the objective was set non-primary. Added Scratchers
// September 2nd, 2003: Fixed a small bug in FlamerSprite. MapPanel now only repaints necessary areas
// after a player character moves instead of the whole panel
// September 4th, 2003: It is now possible to shoot monsters by clicking on their bounding box. Added
// WoundAction handling to MapPanel.animateBufferedAttackActions, so all wounds should now be drawn
// properly
// September 5th, 2003: Added flamer brutes. Shrunk the flamer self-hit zone
// September 7th, 2003: Added BattleRobots and DroidFaction. Had to make some changes to ImageStain to
// allow replacing corpse images
// September 8th, 2003: Added AlarmDrones and AssaultDrones
// September 13th, 2003: Added DestroyerRobots. Reactivated BattleRobots now get progressively more
// damaged
// September 14th, 2003: Added BrainRobots. Weapon menus are now more dynamic. Mission team and weapons
// are now saved between missions
// September 16th, 2003: Added RobotFactoryObjective
// November 2nd, 2003: If you try to pass two rounds without moving in between, you'll get a warning. Same for
// closing the game window. Added new available, unavailable and disabled icons for WeaponMenus
// November 9th, 2003: Some minor interface tweaks
// November 20th, 2003: You can now shift-click an attribute to assign 10 experience points at a time
// January 9th, 2004: Packaged under TheGame. It blew up. As a result, had to add a bunch of "TheGame."+ for
// calls to Class.forName()
// January 28th, 2004: Changed the ImageCropper. New code is shorter and more secure and efficient, especially
// under Windows
// January 31st, 2004: Added a bunch of new soldier face pieces. Stopped the player from accidentally
// closing the doors when attempting first aid
// February 2nd, 2004: Tweaks to TeamManagerWindow. The marine with the highest rank is now the leader,
// instead of the first one. Available weapons are now checked when someone is removed from team. 
// Fixed MarineTeamPanel.removeMarineAt( Point ). Was removing the wrong person if you clicked 
// on the last one
// February 13th, 2004: Added ImageCropper.loadImage(). Should probably add crop methods that
// use the loading, too. Added visual feedback to notify when player characters don't have
// enough action points to perform the action
// February 14th, 2004: Changed the way first aid and using objects is handled. As a nasty
// result, opening and closing doors takes action points. No more killing helpless enemies
// for free. Added textual messages to GameMapPanel. Finally got around to implementing
// automatic build date display
// February 20th, 2004: Changed the message field in TeamManagerWindow into a styled JTextPane.
// Much hilarity ensued with the layout blowing up. That was solved by using
// setSize( getPreferredSize() ). Yeah, ugly. Also changed ImageCropper into ResourceManager
// with support for external resource directories. Getting closer to plug-ins...
// February 21st, 2004: Added CommandBobsObjective that lets you tell those pesky
// civilians to get out of the way
// February 24th, 2004: Changed the way the movements and attacks are handled and animated. 
// Took a lot of work, but now the bomber brute death sequences look better
// February 25th, 2004: Game.addMonster now sorts the monsters acording to their sizes,
// so larger monsters should not appear beneath smaller ones any more. Shortest path
// finding for characters now only avoids those other characters that can initially
// be seen. Promotions and attribute advances are now displayed in TeamManagerWindow
// February 26th, 2004: Added the Subordinate interface. Mines and sentry guns can no
// longer be told to move
// March 13th, 2004: CommandBobsObjective now properly lets you give the Subordinates
// a target to seek
// May 26th, 2004: And this line is for Sepe, another American cocker spaniel. A sad little
// runt when he came to us, 13 years ago, but he turned out okay at the end
// December 26th, 2004: Fixed CommandBobsOjective so dead Subordinates can no longer be commanded
// May 10th, 2005: Fixed mouse keys and modifiers and the display of inline messages for 1.4+
// May 11th, 2005: Added quick and dirty support for displaying number of grenades left
// July 17th, 2005: After a long hiatus, added melee combat. Added new About and Quit handlers. The Game
// now requires Java 1.4
// July 21st, 2005: New XML-based (and gzipped) map file format. Objects are now destroyed at 
// once instead of piece by piece
// July 23rd, 2005: Path finding now tries to use the room graph to optimise long distance searches.
// Weapons being fired makes noise that can alert monsters 
// July 27th, 2005: Mission file format has been XML-fied. It was arguably a good idea to
// separate the primary and secondary objectives from the others in the file, but less so
// in the actual Game. After a lot of headache, it now hopefully works. 
// Objective.getDescription() and isPrimary() have been removed as unnecessary. New 
// getDescriptionClip() gives more flexibility, although it could be better, too
// July 28th, 2005: Team status and marines are now saved as XML like the rest of the data
// July 29th, 2005: Added the mission selection dialog and finished the support for mission
// prequisites, disallowing team changes and automatically opening next mission. Ladies and gentlemen,
// we have a campaign mode.
// July 30th, 2005: Added GenericMonster class that will hopefully make it easier to add new monsters.
// Now they only need the graphics and a fairly simple XML file.
// July 31st, 2005: Added Juve, the first GenericMonster type, tweaked the animations and
// monster numbers
// August 1st, 2005: Added DestroySceneryObjective, obsoleting CloneVatObjective.
// New graphics for Brute
// August 2nd, 2005: Changed the animation system slightly, to better account for variable
// frame numbers on each character type. Also moved the monster action handling from GameWindow to 
// Game where it belongs
// August 3rd, 2005: Updated graphics for ShotgunBrutes, BomberBrutes and the Marines. The soldiers
// now show their weapons in their sprites, and have more animation, including a wound action
// August 4th, 2005: Added Tribbles and CollectTribblesObjective. Fun?
// August 7th, 2005: Map objects can now be pushed around, causing damage to characters
// August 8th, 2005: Added keyboard support for shooting and using stuff. Finally fixed the bugs
// in moving objects
// August 10th, 2005: Added Minigun, replacing the Smartgun as a player choice
// August 12th, 2005: Added Auto-cannon, VipidiotObjective
// August 30th, 2005: Map objects can now be shot with non-area effect weapons. Objects
// can no longer be moved to closed doors, and are now destroyed when closing a door on
// them. It is now possible to choose one-shot missions between campaign missions.
// August 31st, 2005: RecoverObjective is now shown on the map only after it has been discovered in
// random missions. Added regrouping. Sort of works, but is a bit odd at times.
// September 2nd, 2005: Regrouping is now more robust
// September 3rd, 2005: Added plasma blaster
// September 5th, 2005: Added WeaponSwitchObjective
// September 17th, 2005: Civilians now blow up mines in the same way as Marines
// September 19th, 2005: Added VipidiotArrestObjective and VisualTriggerObjective
// September 21st, 2005: Broken objects are now drawn as stains, so other objects cannot
// be pushed under them, and new monster corpses are drawn on top, too
// September 28th, 2005: Fixed a bug in SeekerAI that could lead to stuck game if a room
// didn't have neighbours. Fixed a visual bug with collected Tribbles.
// April 1st, 2006: Fixed a NullPointerException in Vipidiot
// April 2nd, 2006: It seems Rippers didn't roam properly. Now they do again
// April 8th, 2006: Bloody heck. Finally changed the mission statistics to use a NumberFormat
// April 10th, 2006: BattleRobots may reactivate again
// April 12th, 2006: Bobs are now tiny bit less moronic about escaping enemies when they also 
// see player characters
// April 15th, 2006: Fixed a NullPointerException in CollectTribblesObjective that could
// happen with AirLockDoors. Stains are now revealed properly


// - More weapons
// - More mission types
// - More monster types

package TheGame;

import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.Vector;
import com.apple.eawt.*;

/**
 * Main application class for Mac OS X. Serves the same purpose
 * as PureGame on generic OS platforms, but adds some Mac-specific
 * functionality.
 *
 * @author	Virtu Halttunen
 * @version	0.1
 */
public class TheGame extends JFrame implements ApplicationListener
{
    private PureGame theGame;
    private boolean hasMission = false;
    private static String version = "";
    private TeamManagerWindow theWindow = null;

    public TheGame()
    {
        /*
         try {
             UIManager.setLookAndFeel( UIManager.getCrossPlatformLookAndFeelClassName() );
         }
         catch( Exception e )
         {}
        */
	System.out.println( "Game starting" );
	AboutDialog dialog = new AboutDialog();
        System.out.println( "App running" );
	Application.getApplication().addApplicationListener( this ); // Register the listener for the Apple extensions
    }

    public void startGame()
    {
	theWindow = new TeamManagerWindow();
    }
    
    public void handleAbout( ApplicationEvent e )
    {
        System.out.println( "Handle about" );
        AboutDialog.showAboutDialog();
	e.setHandled( true );
    }    
    
    public void handleOpenApplication( ApplicationEvent e )
    {
        System.out.println( "Open app" );
        if( !hasMission )
        {
            hasMission = true;
	    //theWindow = new TeamManagerWindow();
            //theGame = new PureGame();
            //theGame.openMission( missionFile );
        }
	e.setHandled( true );
    }
    
    public void handleOpenFile( ApplicationEvent e )
    {
        System.out.println( "Open file" );
        if( !hasMission )
        {
            hasMission = true;
            //theGame = new PureGame( );
        }
	e.setHandled( true );
    }
    
    public void handlePreferences( ApplicationEvent e )
    {}
    
    public void handlePrintFile( ApplicationEvent e )
    {}
    
    public void handleQuit( ApplicationEvent e )
    {
	e.setHandled( false );
	if( theWindow != null )
	{
	    theWindow.quit();
	}
	else
	{
	    System.exit( 0 );
	}
    }
    
    public void handleReOpenApplication( ApplicationEvent e )
    {}
    
    public static void main( String [] args ) 
    {
        
        System.out.println( "Starting" );
        if( args.length > 0 )
        {
            for( int i=0; i < args.length; i++ )
            {
                System.out.println( "Checking parameters" );
                if( args[i].equalsIgnoreCase( "-noSound" ) )
                {
                    GameMapPanel.playSounds=false;
                }
            }
        }
        TheGame me = new TheGame();
	me.startGame();
        System.out.println( "Started" );
    }
}
