// VisualTriggerObjective.java
// Started on September 19th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;
import java.util.Vector;

public class VisualTriggerObjective extends Objective
{
    public static final String ADD_ON_TARGET_KEY = "Add on target";
    public static final String REMOVE_ON_TARGET_AT_KEY = "Remove on target at";
    public static final String SHOW_ON_MAP_KEY = "Show on map";
    public static final String ON_TARGET_KEY = "On target ";
    public static final String ON_TARGET_KEY_KEY = "On target key ";
    public static final String ON_TARGET_VALUE_KEY = "On target value ";
    
    private Vector targets = new Vector();
    private Vector onTargets = new Vector();
    private boolean showOnMap = true;
    private Point location = new Point( -1, -1 );
    private boolean triggered = false;

    private String [] properties = { Objective.LOCATION_KEY, SHOW_ON_MAP_KEY, ADD_ON_TARGET_KEY, REMOVE_ON_TARGET_AT_KEY };
    
    public VisualTriggerObjective()
    {
    }

    public void setGame( Game game )
    {
        activeGame = game;
    }

    public int getNumberOfSprites()
    {
        return 0;
    }    
    
    public String [] getPropertyNames()
    {
        int propertyCount = properties.length + (onTargets.size()*3);
        //System.out.println( "PressureSwitchObjective has " + propertyCount + " properties with " + targets.size() + " targets" );
        String [] propertyNames = new String[ propertyCount ];
	int i, index;
        for( i = 0; i < properties.length; i++ )
        {
            propertyNames[ i ] = properties[ i ];
        }
        index = i;
        for( i = 0; i < onTargets.size(); i++ )
        {
            propertyNames[ index++ ] = ON_TARGET_KEY+i;
            propertyNames[ index++ ] = ON_TARGET_KEY_KEY+i;
            propertyNames[ index++ ] = ON_TARGET_VALUE_KEY+i;
        }
	return propertyNames;
    }

    private ObjectiveTarget getOnTarget( String id )
    {
	for( int i=0; i < targets.size(); i++ )
	{
	    ObjectiveTarget target = (ObjectiveTarget)onTargets.elementAt( i );
	    if( target.getIndex().equals( id ) )
	    {
		return target;
	    }
	}
	return null;
    }
        
    public Class getPropertyClass( String key )
    {
	if( key.equals( SHOW_ON_MAP_KEY ) )
	{
	    return Boolean.class;
	}
	return String.class;
    }
    
    public String getProperty( String key )
    {
        if( key.equals( Objective.LOCATION_KEY ) )
        {
            return Objective.pointToString( location );
        }
	if( key.equals( SHOW_ON_MAP_KEY ) )
        {
            return String.valueOf( showOnMap );
	}
        if( key.startsWith( ON_TARGET_KEY_KEY) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_KEY_KEY.length()-1 ).trim() );
            return ((ObjectiveTarget)onTargets.elementAt( index )).getKey();
        }
        if( key.startsWith( ON_TARGET_VALUE_KEY ) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_VALUE_KEY.length()-1 ).trim() );
            return ((ObjectiveTarget)onTargets.elementAt( index )).getValue();
        }
	if( key.startsWith( ON_TARGET_KEY ) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_KEY.length()-1 ).trim() );
            return ((ObjectiveTarget)onTargets.elementAt( index )).getIndex();
        }
        return null;
    }

    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( Objective.LOCATION_KEY ) )
        {
            location = new Point( Objective.pointFromString( value ) );
        }
	if( key.equals( SHOW_ON_MAP_KEY ) )
        {
	    showOnMap = new Boolean( value ).booleanValue();
	}
	if( key.equals( ADD_ON_TARGET_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            String id = value;
	    //System.out.println( "Adding ON target: " + id );
            ObjectiveTarget newTarget = new ObjectiveTarget( id );
            onTargets.add( newTarget );
        }
        if( key.equals( REMOVE_ON_TARGET_AT_KEY ) )
        {
            if( value == null )
            {
                return;
            }
            int index = Objective.intFromString( value );
	    onTargets.removeElementAt( index );
	}
        if( key.startsWith( ON_TARGET_KEY_KEY ) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_KEY_KEY.length()-1 ).trim() );
            if( onTargets.size() > index )
            {
                ((ObjectiveTarget)onTargets.elementAt( index )).setKey( value );
            }
	    return;
        }
	if( key.startsWith( ON_TARGET_VALUE_KEY ) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_VALUE_KEY.length()-1 ).trim() );
            if( onTargets.size() > index )
            {
                ((ObjectiveTarget)onTargets.elementAt( index )).setValue( value );
            }
	    return;
        }
	if( key.startsWith( ON_TARGET_KEY ) )
        {
            int index = Integer.parseInt( key.substring( ON_TARGET_KEY.length()-1 ).trim() );
	    //System.out.println( "Setting ON target: " + index );
            // Check whether target exists
            if( onTargets.size() > index )
            {
                ((ObjectiveTarget)onTargets.elementAt( index )).setIndex( value );
            }
            else
            {
                // If it doesn't exist, add it
                setProperty( ADD_ON_TARGET_KEY, value );
            }
	    //System.out.println( onTargets.size() + " ON targets" );
        }
    }    
    
    public boolean isCompleted()
    {
        return true;
    }

    public boolean isSuccessful()
    {
	return true;
    }
    
    public int getSize()
    {
        return 0;
    }

    public boolean isLocatedIn( Point point )
    {
        return point.equals( location );
    }

    public boolean isLocatedIn( int x, int y )
    {
        return isLocatedIn( new Point( x, y ) );
    }

    public boolean isDestructible()
    {
        return false;
    }

    public boolean isShownOnMap()
    {
        return showOnMap;
    }
    
    public boolean isUsable()
    {
	return false;
    }

    public boolean handleAction( GameAction action )
    {
	Character character;
	
	for( int i=0; i < activeGame.getPlayerCharacters().size() && !triggered; i++ )
	{
	    character = activeGame.getPlayerCharacter( i );
	    if( character.isAlive() && activeGame.isCellVisibleToCharacter( character, location ) )
	    {
		triggered = true;
		// Activate on targets
		for( int t = 0; t < onTargets.size(); t++ )
		{
		    try
		    {
			//System.out.println( "Activating target " + (t+1) );
			ObjectiveTarget target = (ObjectiveTarget)onTargets.elementAt( t );
			Objective objective = activeGame.getObjectiveById( target.getIndex() );
			objective.setProperty( target.getKey(), target.getValue() );
			//System.out.println( objective.getClass().toString() + ": " + target.getKey() + " = " + target.getValue() ); 
		    }
		    catch( Exception e )
		    {
			System.out.println( e.toString() );
		    }
		}		
	    }
	}
	return false;
    }
}
