// Weapon.java
// Started on July 25th, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;

/**
 * This class is the abstract base for all weapon classes in the game.
 *
 * @author	Virtu Halttunen
 * @version	0.1
 */
public abstract class Weapon
{
    // Damage types
    public static final int IMPACT = 0;
    public static final int FLAME = 1;

    // Firing modes
    public static final int DEFAULT = 0;
    public static final int PRIMARY = 0;
    public static final int SECONDARY = 1;
    
    // Range... Template weapons?
    
    // Ammunition..?

    /**
     * Get the amount of damage the weapon causes for a shot
     *
     * @return 		amount of damage
     */
    public int getDamage()
    {
        return 0;
    }

    /**
     * Get the amount of damage the weapon causes for a shot
     *
     * @param critical	is the shot critical
     * @return 		amount of damage
     */
    public int getDamage( boolean critical )
    {
        return 0;
    }
    
    /**
     * Get the amount of damage the weapon causes for a shot
     *
     * @param range	the range of the shot
     * @param critical	is the shot critical
     * @return 		amount of damage
     */
    public int getDamage( int range, boolean critical )
    {
	return 0;
    }

    /**
     * Get the number of action points required to fire. Zero or negative
     * means the weapon cannot be fired
     *
     * @return 		required action points
     */
    public int getActionsToFire()
    {
	return 1;
    }

    /**
     * Get the number of action points lost per turn when carrying the weapon
     *
     * @return 		lost action points
     */
    public int getMovementCost()
    {
        return 0;
    }

    /**
     * Get the number of equipment points the weapon costs
     *
     * @return 		cost
     */
    public int getCost()
    {
        return 0;
    }
    
    public boolean hasAreaEffect()
    {
        return false;
    }

    public int getBlockingSize()
    {
        return 50;
    }
    
    public String getName()
    {
	return "Weapon";
    }
    
    /**
     * Get the area that the shot covers
     *
     * @param theGame	Game where the shot was fired
     * @param fireFrom	where the shot was fired from
     * @param fireTo	where the shot was fired at
     * @return 		the area covered by the shot
     */
    public abstract Point [] getArea( Game theGame, Point fireFrom, Point fireTo );
    
    /**
     * Get the human readable names for the different firing modes.
     * In The Game, these will be put to a combo box, where the
     * player can select the firing mode. The second firing mode, if
     * present, can also be accessed quickly with right mouse key,
     * or pressing the appropriate modifier key.
     *
     * @return 		the names of the firing modes
     */
    public abstract String [] getFiringModeNames();
    
    public int getNumberOfFiringModes()
    {
        return 1;
    }

    public int getFiringMode()
    {
        return 0;
    }

    public void setFiringMode( int mode )
    {
    }

    public void resetAmmunition()
    {
    }
    
    public int getRequiredAttribute()
    {
	return Marine.HANDGUN;
    }

    public abstract Image getIcon();
}